(* Module: Contains a function that takes an integer n as argument and
 * returns a list of lenght n! consisting of all permutations of the
 * set {1,..,n}.  The other functions in this file are auxiliary
 * functions to accomplish this. 
  
 *
 * Description:  Design/CPN OG Tool
 *
 * Created by:   Lars Kristensen (kris@daimi.aau.dk)   
 *               Jens Baek Jorgensen (jbj@daimi.aau.dk)
 * Date:         01/02/96   
 *
 * Design/CPN (tm)  Copyright (c) Meta Software Corp 1989--1995
 *)

(* $Source: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OEUtilities/OEAllPerms.sml,v $

$Log: OEAllPerms.sml,v $
Revision 1.1  1996/02/01 11:04:19  kris
Initial revision
 

*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OEUtilities/OEAllPerms.sml,v 1.1 1996/02/01 11:04:19 kris Exp $";

(*************** CREATED OCTOBER 25 1995 BY JBJ ***************)
(*************** LAST MODIFIED: NOVEMBER 23 1995 ***************)

structure CPN'OEAllPerms =
struct

local
fun allpairs' CPN'a [] = []
 |  allpairs' CPN'a (CPN'b::CPN'l) = (CPN'a,CPN'b)::(allpairs' CPN'a CPN'l);

fun allpairs'' [] CPN'l' = []
 |  allpairs'' (CPN'a::CPN'l) CPN'l' =
                        (allpairs' CPN'a CPN'l')^^(allpairs'' CPN'l CPN'l');

fun comb' (CPN'a,CPN'b) [] = []
 |  comb' (CPN'a,CPN'b) ((CPN'a',CPN'b')::CPN'l') =
               [(CPN'a,CPN'b),(CPN'a',CPN'b')]::(comb' (CPN'a,CPN'b) CPN'l');

(* The following function turns out to be just a (restricted
form of - only between pairs) Cartesian product. *)
fun comb'' [] CPN'l' = []
 |  comb'' ((CPN'a,CPN'b)::CPN'l) CPN'l' =
                         (comb' (CPN'a,CPN'b) CPN'l')^^(comb'' CPN'l CPN'l');
 
(* Special-purpose insertion function. Inserts the given element a
in the given position i in the given list. The first element has 
number one (not zero). If i is greater than the length of the list,
the element is inserted after the last element of the given list *)
fun ins_pos CPN'i CPN'a [] = [CPN'a]
 |  ins_pos CPN'i CPN'a (CPN'b::CPN'l) = 
        if  CPN'i=1 then CPN'a::CPN'b::CPN'l
        else CPN'b::(ins_pos (CPN'i-1) CPN'a CPN'l);

fun ins_everywhere' CPN'a CPN'l 0 CPN'acc = CPN'acc
 |  ins_everywhere' CPN'a CPN'l CPN'i CPN'acc = 
      ins_everywhere' CPN'a CPN'l (CPN'i-1) ((ins_pos CPN'i CPN'a CPN'l)::CPN'acc);

(* Takes an element and a list as arguments. Say the list has
length n. Then a list of n+1 lists is returned. There is a result
list for every position in which the given element can be merged
into the given list. *)
fun ins_everywhere CPN'a CPN'l =
                         ins_everywhere' CPN'a CPN'l ((length CPN'l)+1) [];

fun allperms' 0 CPN'l = CPN'l
 |  allperms' CPN'i CPN'l =
            allperms' (CPN'i-1) (flatten (map (ins_everywhere CPN'i) CPN'l));

(* Function to sort a given list of permutations lexicographically. *)
fun sortperms CPN'l = CPN'OEListBasics.sort CPN'OEListBasics.lexi CPN'l;
in
(* Takes an integer n as parameter. Produces a list of length n! of all
permutations of the numbers {1,2,...,n}. *)
fun allperms CPN'n = sortperms(allperms' CPN'n [[]])
end; 

end (* struct CPN'OEAllPerms *)
