(*
 * Module: Contains functions for converting multi-sets into component
and restriction sets. The two main functions is one that computes
the indexed component set for a given multi-set, and one that
computes the restriction set for two given symmetrical multi-sets.
In addition, a function that computes the coefficient multi-set is
defined.
 *
 * Description:  Design/CPN OE/OS Graph Tool
 *
 * Created by:   Lars Kristensen (kris@daimi.aau.dk)   
 *               Jens Baek Jorgensen (jbj@daimi.aau.dk)
 * Date:         01/02/96   
 *
 * Design/CPN (tm)  Copyright (c) Meta Software Corp 1989--1995
 *)

(* $Source: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OEUtilities/OEConvertMuls.sml,v $

$Log: OEConvertMuls.sml,v $
Revision 1.2.4.2  1999/04/06 10:48:33  rblyngso
Changed i to CPN'i (not tested).

Revision 1.2.4.1  1999/01/04 09:16:31  mailund
char vs. string

Revision 1.2  1996/02/02 14:01:06  jbj
Added component stuff.

 Revision 1.1  1996/02/01  11:26:14  jbj
 Initial revision
 

*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OEUtilities/OEConvertMuls.sml,v 1.2.4.2 1999/04/06 10:48:33 rblyngso Exp $";


structure CPN'OEConvertMuls =
struct

local
(* Takes a multi-set as input. Returns the ordered list of positive
coefficients. *)
fun ms_to_cfs empty = []
 |  ms_to_cfs ((CPN'cf,CPN'col)!!CPN'ms) = 
       CPN'OEListBasics.oinsert_int CPN'cf (ms_to_cfs CPN'ms);

(* Takes a multi-set and a coefficient as input. Returns a pair consisting
of the coefficient and a list of all elements having that coefficient
(using an accumulating parameter. *)
fun cons_comp empty CPN'cf' CPN'acc = (CPN'cf',CPN'acc)
 |  cons_comp ((CPN'cf,CPN'col)!!CPN'ms) CPN'cf' CPN'acc = 
       if CPN'cf=CPN'cf' 
       then
          cons_comp CPN'ms CPN'cf' (CPN'col::CPN'acc)
       else
          cons_comp CPN'ms CPN'cf' CPN'acc;

(* Aux function for ms_to_comp. *)
fun ms_to_comp' CPN'ms [] = []
 |  ms_to_comp' CPN'ms (CPN'a::CPN'l) = 
                       (cons_comp CPN'ms CPN'a [])::(ms_to_comp' CPN'ms CPN'l);

fun cons_comp_ord empty CPN'cf' CPN'acc = CPN'acc
 |  cons_comp_ord ((CPN'cf,CPN'col)!!CPN'ms) CPN'cf' CPN'acc = 
       if CPN'cf=CPN'cf' 
       then
          cons_comp_ord CPN'ms CPN'cf' (CPN'col::CPN'acc)
       else
          cons_comp_ord CPN'ms CPN'cf' CPN'acc;
(* Aux function for ms_to_comp_ord. *)
fun ms_to_comp_ord' CPN'ms [] = []
 |  ms_to_comp_ord' CPN'ms (CPN'a::CPN'l) = 
                       (cons_comp_ord CPN'ms CPN'a [])::(ms_to_comp_ord' CPN'ms CPN'l);

in
 (* Takes a multi-set as input. Returns the corresponding indexed
 component set. *)
 fun ms_to_comp CPN'ms = ms_to_comp' CPN'ms (remdupl(ms_to_cfs CPN'ms));

 (* Takes a multi-set as input. Returns the coefficient multi-set. *)
 fun ms_to_cfms CPN'ms = 

   List.foldr op++
   empty
   (map (fn CPN'i => (length(CPN'OEListBasics.snd(cons_comp CPN'ms CPN'i [])))`CPN'i) 
    (remdupl (ms_to_cfs CPN'ms)));


(* Takes a multi-set as input. Returns the corresponding ordinary, i.e.,
non-indexed component set. *)
 fun ms_to_comp_ord CPN'ms = ms_to_comp_ord' CPN'ms (remdupl(ms_to_cfs CPN'ms));
end;

exception ex_mss_to_rss

(* Takes two multi-sets as input. If a bijection mapping one to the other
exists, the corresponding restriction set is returned. Otherwise an 
exception is raised. *)
fun mss_to_rss CPN'ms CPN'ms' =
let
(* Aux function for mss_to_rss. *)
 fun mss_to_rss' [] [] = []
  |  mss_to_rss' ((CPN'i,CPN'cs)::CPN'css) ((CPN'i',CPN'cs')::CPN'css') =
       if (CPN'i=CPN'i') andalso (length CPN'cs = length CPN'cs') 
       then
          (CPN'cs,CPN'cs')::(mss_to_rss' CPN'css CPN'css')
       else
          raise ex_mss_to_rss
  |  mss_to_rss' _ _ = raise ex_mss_to_rss;
in
 mss_to_rss' (ms_to_comp CPN'ms) (ms_to_comp CPN'ms') 
end;

end (* struct CPN'OEConvertMuls *)
