(*
 * Module:       Functions and types for permutations restriction 
 *               and component sets
 *
 * Description:  Design/CPN OE/OS Graph Tool
 *
 * Created by:   Lars Kristensen (kris@daimi.aau.dk)   
 *               Jens B. Jorgensen (jbj@daimi.aau.dk)
 * Date:         Tue Jun 25 16:18:13 MET DST 1996
 *
 * Design/CPN (tm)  Copyright (c) Meta Software Corp 1989--1995
 *)

(* $Source: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OEUtilities/OEInterface.sml,v $

$Log: OEInterface.sml,v $
Revision 1.1  1996/06/27 14:45:15  cpn
Initial revision
 

*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OEUtilities/OEInterface.sml,v 1.1 1996/06/27 14:45:15 cpn Exp $";


(* stuff that must be renamed *)
(* val CPN'IntersectResSets = IntersectResSets; *)
(* val CPN'IntersectCompSets = IntersectCompSets *)
val proj_ms1 = Proj1;
val proj_ms2 = Proj2;

(* Definition of the basic data types *)
datatype 'a Perm = Perm of ('a * 'a) list 

(* Restrictions Sets *)
(* EmptyRes is equivalent to no restrictions *)
(* ResPermEmpty means that the Restriction Set does not 
   determine any permutations *)
datatype 'a ResSet = ResSet of ('a list * 'a list) list | EmptyRes | ResPermEmpty

datatype 'a CompSet = CompSet of 'a list list | EmptyComp | CompPermEmpty

(* Functions for RestrictionSets *)
fun CreateResSet CPN'ms1 CPN'ms2 = (ResSet (CreateRes CPN'ms1 CPN'ms2)
				    handle ex_mss_to_rss => ResPermEmpty)


(* Utility functions for Restriction Sets *)
exception Ex_PermEmpty;

fun CPN'filterres [] = []
  | CPN'filterres ((ResSet CPN'res)::CPN'resl) = 
    CPN'res::(CPN'filterres CPN'resl)
  | CPN'filterres (EmptyRes::CPN'resl) = CPN'filterres CPN'resl
  | CPN'filterres (ResPermEmpty::CPN'resl) = raise Ex_PermEmpty;

fun IntersectResSets CPN'resl =
    (case (CPN'filterres CPN'resl) of
	  [] => EmptyRes 
	| CPN'l => ((ResSet (CPN'IntersectResSets CPN'l))
		    handle ex_intersect => ResPermEmpty
			handle Ex_PermEmpty => ResPermEmpty));

(* Utility functions for Permutations *)
fun CPN'addperm [] = []
  | CPN'addperm (CPN'x::CPN'xs) = (Perm CPN'x)::(CPN'addperm CPN'xs)

fun ListPermutations (EmptyRes,CPN'set) = 
    CPN'addperm (CreateCanonicals (CreateRes CPN'set CPN'set))
  | ListPermutations (ResPermEmpty,_) = []
  | ListPermutations (ResSet CPN'res,CPN'set) = 
    CPN'addperm (CreateCanonicals 
		 (CPN'IntersectResSets 
		  [CPN'res,(CreateRes CPN'set CPN'set)]))

fun CPN'removeperm [] = []
  | CPN'removeperm ((Perm CPN'x)::CPN'xs) = CPN'x::(CPN'removeperm CPN'xs)

fun CPN'TestOne CPN'perm [] = true
  | CPN'TestOne CPN'perm ((CPN'ms1,CPN'ms2)::CPN'msl) = 
    (((ApplyCanonical CPN'perm CPN'ms1) == CPN'ms2) andalso 
     (CPN'TestOne CPN'perm CPN'msl))

fun TestPermutations ([],_) = false
  | TestPermutations (((Perm CPN'perm)::CPN'perml),CPN'msl) = 
    (CPN'TestOne CPN'perm CPN'msl) orelse (TestPermutations (CPN'perml,CPN'msl))
fun TestPermutationsPair ([],_) = false
  | TestPermutationsPair (CPN'pls,CPN'msl) = 
    (OKCanonicalPair CPN'msl (CPN'removeperm CPN'pls))
    
fun FilterPermutationsPair ([],_) = []
  | FilterPermutationsPair (((Perm CPN'perm)::CPN'perml),CPN'msl) = 
    (if (OKCanonicalPair CPN'msl [CPN'perm])
	 then
	     (Perm CPN'perm)::(FilterPermutationsPair (CPN'perml,CPN'msl))
     else
	 (FilterPermutationsPair (CPN'perml,CPN'msl)))

fun FilterPermutations ([],_) = []
  | FilterPermutations (((Perm CPN'perm)::CPN'perml),CPN'msl) =
    (if (CPN'TestOne CPN'perm CPN'msl)
	 then
	     (Perm CPN'perm)::(FilterPermutations (CPN'perml,CPN'msl))
     else
	 (FilterPermutations (CPN'perml,CPN'msl)))

fun ApplyPermutation ((Perm CPN'perm),CPN'ms) =
    (ApplyCanonical CPN'perm CPN'ms)

fun ApplyPermutationPair ((Perm CPN'perm),CPN'ms) =
    (ApplyCanonicalPair CPN'perm CPN'ms)

(* Utility functions for Component Sets *)
fun CPN'filtercomp [] = []
  | CPN'filtercomp ((CompSet CPN'res)::CPN'resl) = 
    CPN'res::(CPN'filtercomp CPN'resl)
  | CPN'filtercomp (EmptyComp::CPN'resl) = CPN'filtercomp CPN'resl
  | CPN'filtercomp (CompPermEmpty::CPN'resl) = raise Ex_PermEmpty;


fun IntersectCompSets CPN'resl =
    (case (CPN'filtercomp CPN'resl) of
	  [] => EmptyComp 
	| CPN'l => ((CompSet (CPN'IntersectCompSets CPN'l))
		    handle ex_intersect => CompPermEmpty
			handle Ex_CompEmpty => CompPermEmpty));

fun CreateCompSet CPN'ms1 = (CompSet (CreateComp CPN'ms1))

(* Transforms a component set into the corresponding restriction set *)
fun CPN'comp_to_res [] = []
 |  CPN'comp_to_res (CPN'c::CPN'cl) = (CPN'c,CPN'c)::(CPN'comp_to_res CPN'cl)
				   
fun CompSetToResSet EmptyComp = EmptyRes
  | CompSetToResSet CompPermEmpty = ResPermEmpty
  | CompSetToResSet (CompSet CPN'comp) = (ResSet (CPN'comp_to_res CPN'comp))

(* Rotations *)
fun ApplyRotation CPN'f CPN'ms = (ext_col CPN'f CPN'ms);

fun TestRotation CPN'f [] = true
  | TestRotation CPN'f ((CPN'ms1,CPN'ms2)::CPN'msl) =
    (((ApplyRotation CPN'f CPN'ms1) == CPN'ms2) andalso
    (TestRotation CPN'f CPN'msl));
