(*
 * Module: Contains general-purpose list functions
 *
 * Description:  Design/CPN OE/OS Graph Tool
 *
 * Created by:   Lars Kristensen (kris@daimi.aau.dk)   
 *               Jens Baek Jorgensen (jbj@daimi.aau.dk)
 * Date:         01/02/96   
 *
 * Design/CPN (tm)  Copyright (c) Meta Software Corp 1989--1995
 *)

(* $Source: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OEUtilities/OEListBasics.sml,v $

$Log: OEListBasics.sml,v $
Revision 1.2.4.1  1999/01/04 09:16:31  mailund
char vs. string

Revision 1.2  1996/11/27 17:25:19  cpn
*** empty log message ***

Revision 1.1  1996/02/01 11:27:48  jbj
Initial revision
 

*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OEUtilities/OEListBasics.sml,v 1.2.4.1 1999/01/04 09:16:31 mailund Exp $";


structure CPN'OEListBasics =
struct

(* Takes an integer an a list of integers. Inserts the integer in the list
in a position such that the next element is greater or equal. I.e., if the
given list is ordered, the returned list is as well. *)
fun oinsert_int (CPN'i:int) [] = [CPN'i]
 |  oinsert_int (CPN'i:int) ((CPN'j:int)::CPN'l) = 
      if CPN'i <= CPN'j 
      then 
         CPN'i::CPN'j::CPN'l else CPN'j::(oinsert_int CPN'i CPN'l);

(* Takes an element and a list as input. Returns the list with all
appearances of the element removed. *)
fun remv CPN'a [] = []
 |  remv CPN'a (CPN'b::CPN'l) = 
        if CPN'a = CPN'b then remv CPN'a CPN'l else CPN'b::(remv CPN'a CPN'l);

(* Takes an element and a list as input. Returns true if the element is
member of the list and false otherwise. *)
fun rmem CPN'a CPN'l = mem CPN'l CPN'a;

(* Takes a pair as input. Returns the first element. *)
fun fst (CPN'a,CPN'b) = CPN'a;

(* Takes a pair as input. Returns the second element. *)
fun snd (CPN'a,CPN'b) = CPN'b;

(* Takes two lists as input. Returns a list of all elements in the first
list also appearing in the second. I.e., if the two given lists represent
sets, the intersection is produced. *)
fun intsct_lst [] CPN'l = []
 |  intsct_lst (CPN'a::CPN'l) CPN'l' = 
       if rmem CPN'a CPN'l' then CPN'a::(intsct_lst CPN'l CPN'l')
       else (intsct_lst CPN'l CPN'l');

(* Takes two lists as input. Returns a list of all elements in the first
list not appearing in the second. I.e., if the two given lists represent
sets, the set difference is produced. *)
fun subtr_lst CPN'l [] = CPN'l
 |  subtr_lst CPN'l (CPN'a::CPN'l') = 
         subtr_lst (remv CPN'a CPN'l) CPN'l';

(* Takes a list of list as input. Returns the flattening of the input. *)

val flatten = List.concat; 


(* General-purpose insertion function. Takes an element and a 
list as argument. If the given list is ordered, so is the result. 
The argument CPN'leq is expected to define the order *)
fun order_ins (CPN'leq: ''a -> ''a -> bool) CPN'a [] = [CPN'a]
 |  order_ins CPN'leq CPN'a (CPN'b::CPN'l) = 
            if CPN'leq CPN'a CPN'b then CPN'a::CPN'b::CPN'l
            else CPN'b::(order_ins CPN'leq CPN'a CPN'l);

(* General-purpose sorting function. Takes an order function and a list
as arguments. Returns the ordered list. *)
local 
fun sort' CPN'leq [] CPN'acc = CPN'acc
 |  sort' CPN'leq (CPN'a::CPN'l) CPN'acc =
           sort' CPN'leq CPN'l (order_ins CPN'leq CPN'a CPN'acc);
in
fun sort CPN'leq CPN'l = sort' CPN'leq CPN'l []
end;

(* Special-purpose ordering function for lists of integers. Takes two
lists as arguments. Returns true if the first is lexicographically
smaller than the second and false otherwise. *)
fun lexi [] CPN'l' = true
 |  lexi (CPN'a::CPN'l) [] = false
 |  lexi ((CPN'a:int)::CPN'l) ((CPN'b:int)::CPN'l') = 
         if CPN'a<CPN'b then true
         else if CPN'a>CPN'b then false
         else lexi CPN'l CPN'l';

end (* struct CPN'OEListBasics *)

