(*
 * Module:Contains functions manipulating permutations.
The canonical representation
of a permutation is a list of pairs of values:
f=[(e11,e12),(e21,e22),..,(en1,en2)],
meaning that f(ei1)=ei2, for i=1,..,n.
If e does not appear as first coordinate in any pair of the list, then
f(e)=e
 *
 * Description:  Design/CPN OE/OS Graph Tool
 *
 * Created by:   Lars Kristensen (kris@daimi.aau.dk)   
 *               Jens Baek Jorgensen (jbj@daimi.aau.dk)
 * Date:         01/02/96   
 *
 * Design/CPN (tm)  Copyright (c) Meta Software Corp 1989--1995
 *)

(* $Source: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OEUtilities/OEPerm.sml,v $

$Log: OEPerm.sml,v $
Revision 1.2  1996/11/27 17:25:40  cpn
*** empty log message ***

Revision 1.1  1996/02/01 11:29:44  jbj
Initial revision
 

*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OEUtilities/OEPerm.sml,v 1.2 1996/11/27 17:25:40 cpn Exp $";

structure CPN'OEPerm =
struct

(* The compose function takes a argument a list of lists. Each list represents
a set of functions. The result is a list of all possible function compositions:
compose [[f11,..,f1n1],..., [fm1,..,fmnm] =
                      [f11.f21....fm1,..,f1n1.f2n2...fmnm]. *)

exception ex_compose;
local
 fun comp' CPN'f [] = []
  |  comp' CPN'f (CPN'g::CPN'r) = (CPN'f^^CPN'g)::(comp' CPN'f CPN'r);

 fun comp [] CPN'gs = []
  |  comp (CPN'p::CPN'r) CPN'gs = 
                (comp' CPN'p CPN'gs)^^(comp CPN'r CPN'gs);
in
 fun compose [] = raise ex_compose
  |  compose [CPN'fs]  =  CPN'fs
  |  compose (CPN'fs::CPN'fss) = comp CPN'fs (compose CPN'fss)
end;

(* The short_to_canon function converts a short representation of an 
integer permutation into the canonical form, i.e., into a list of pairs:
short_to_canon [e1,e2,...,en] = [(1,e1),(2,e2),...,(n,en)]. *)
local
   fun make_pair ([],CPN'i) = []
    |  make_pair (CPN'e::CPN'l,CPN'i) = (CPN'i,CPN'e)::(make_pair(CPN'l, CPN'i+1))
in
  fun short_to_canon CPN'l = make_pair (CPN'l,1)
end;

end (* struct CPN'OEPerm *)

