(*
 * Module:Defines two functions. res_to_canon converts a restriction
into a list of canonically represented permutations.  rss_to_canon
converts a restriction set into a list of canonically represented
permutations.
 *
 * Description:  Design/CPN OE/OS Graph Tool
 *
 * Created by:   Lars Kristensen (kris@daimi.aau.dk)   
 *               Jens Baek Jorgensen (jbj@daimi.aau.dk)
 * Date:         01/02/96   
 *
 * Design/CPN (tm)  Copyright (c) Meta Software Corp 1989--1995
 *)

(* $Source: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OEUtilities/OEResToCanon.sml,v $

$Log: OEResToCanon.sml,v $
Revision 1.2.4.1  1999/01/04 09:16:32  mailund
char vs. string

Revision 1.2  1996/11/27 17:25:57  cpn
*** empty log message ***

Revision 1.1  1996/02/01 11:32:40  jbj
Initial revision
 

*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OEUtilities/OEResToCanon.sml,v 1.2.4.1 1999/01/04 09:16:32 mailund Exp $";

structure CPN'OEResToCanon =
struct

local

fun subst' (CPN'rc,CPN'lc) [] = []
 |  subst' (CPN'rc,CPN'lc) ((CPN'i,CPN'j)::CPN'l) = 

            (List.nth(CPN'rc, CPN'i-1),
	     List.nth(CPN'lc,CPN'j-1))::(subst' (CPN'rc,CPN'lc) CPN'l);


fun subst CPN'r CPN'l = map (subst' CPN'r) CPN'l;

in
(* converts a restriction into a list of canonical representations *)
fun res_to_canon CPN'r =  
                 subst CPN'r
                       (map CPN'OEPerm.short_to_canon
                            (CPN'OEAllPerms.allperms (length (CPN'OEListBasics.fst CPN'r))));

(* converts a restriction set into a list of canonical representations *)
fun rss_to_canon CPN'rs = CPN'OEPerm.compose (map res_to_canon CPN'rs);

end;

end (* struct CPN'OEResToCanon *)

