(*
 * Module: Contains special-purpose functions for restriction sets.
 *
 * Description:  Design/CPN OE/OS Graph Tool
 *
 * Created by:   Lars Kristensen (kris@daimi.aau.dk)   
 *               Jens Baek Jorgensen (jbj@daimi.aau.dk)
 * Date:         01/02/96   
 *
 * Design/CPN (tm)  Copyright (c) Meta Software Corp 1989--1995
 *)

(* $Source: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OEUtilities/OEResUtils.sml,v $

$Log: OEResUtils.sml,v $
Revision 1.2.4.1  1999/01/04 09:16:32  mailund
char vs. string

Revision 1.2  1996/02/02 14:01:27  jbj
Added CompSize.

 Revision 1.1  1996/02/01  11:33:59  jbj
 Initial revision
 

*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OEUtilities/OEResUtils.sml,v 1.2.4.1 1999/01/04 09:16:32 mailund Exp $";


structure CPN'OEResUtils =
struct

local
 fun maps_to'' CPN'i CPN'j (CPN'lc, CPN'rc) = (mem CPN'lc CPN'i) andalso (mem CPN'rc CPN'j);

 fun maps_to' CPN'i CPN'j [] = false
  |  maps_to' CPN'i CPN'j (CPN'res::CPN'rss) = 
                    (maps_to'' CPN'i CPN'j CPN'res) orelse (maps_to' CPN'i CPN'j CPN'rss)
in
(* Takes two elements and a restriction set as arguments. Returns true if the restriction
set determnines a permutation that maps the first element to the second, and false
otherwise. *)
 fun maps_to CPN'i CPN'j CPN'rss =
    if (not(mem (CPN'OEResAndComp.res_to_lsum CPN'rss) CPN'i)) andalso
       (not (mem (CPN'OEResAndComp.res_to_rsum CPN'rss) CPN'j))
    then true
    else maps_to' CPN'i CPN'j CPN'rss
end

(* General-purpose function to compute the faculty i! - would be more
appropriate in another place than here *)
exception ex_fac;
fun fac 0 = 1
 |  fac CPN'i = if CPN'i>0 
                then CPN'i*(fac (CPN'i-1)) 
                else raise ex_fac; (* negative integers not allowed *)

(* Takes an integer and a component set as arguments. If the
integer is the size of the domain of the component set, the
number of permutations determined by the component set is
returned *)

fun CompSize CPN'i CPN'cs = 
    let

	val CPN'l = List.foldr (op +) 0 (map length CPN'cs)

    in

	(List.foldr (op * ) 1 (map (fac o length) CPN'cs)) *
        (fac (CPN'i-CPN'l))

    end;

end (* struct CPN'OEResUtils *)

open CPN'OEResUtils;

