structure CPN'OESizes =
struct

(* Takes a node as argument. Returns the set of binding elements that
are enabled in the given node. *)
fun NoOfEnabledBE CPN'n =
    (           
     CPN'OGEnabData.init();  (* initialization for immediate trans*)
     CPN'OGTimeEnabData.init();  (* initialization for timed trans*)
     
     CPN'OGToSimData.copy CPN'n;  (* prepare the simulator data records *)
     
     CPN'ValuePrint:=false; (* disable communication with C part for reg. sim. *)

     CPN'OGCalcEnab(); (* calculate the enabled binding elements in CPN'n *)
	 
	 if ((!CPN'env.Timedef)<>CPN'env.timenotdef)andalso
	     (with_time())andalso
	     (CPN'OGEnabData.get_cands()<>[])andalso

	     (case !CPN'OGTimeEnabData.curr_min of
		  SOME CPN't => IntInf.>(CPN't,(time()))
		| NONE => true)

	     then
     (* only immediate trans should be executed at current model time *)
		 CPN'OGTimeEnabData.init()
	 else
             CPN'OGCalcEnabTimeBinds();
	     
	     CPN'ValuePrint:=true; 
 (* enable communication with C part for reg. sim. -- to be nice  *)
    length ((CPN'OGEnabData.get_cands())) + (length (CPN'OGTimeEnabData.get_cands())));

(* For the two following functions SizeOfClass and SizeOfOGraph, it 
It is assumed that the selfsymmetry group of a marking can be represented
with a component set. *)

(* Takes two integers and a node as argument. If the integer CPN'sizePhi is
the size of the group of permutation symmetries, the integer CPN'sizedomain
is the size of the domain of the restriction set, then the size of the
equivalence class of the node is returned - according to Lagrange's theorem
relating cosets for the self-symmetry group with the orbit of an element.
If the integer arguments are not the as required, this function makes no
sense. *)
fun OSSizeOfClass CPN'sizePhi CPN'sizedomain SelfSymFun CPN'n =
    CPN'sizePhi div (CompSize CPN'sizedomain (SelfSymFun CPN'n));


(* SizeOfOGraph calculates the size of the O-graph from the OS-graph.
SizeOfOGraph takes the size of the group of permutation symmetries
as argument CPN'sizePhi, the size of the domain of the component set
as argument CPN'sizedomain.
SizeOfOGraph returns the size of the O-graph as a pair where the
first component is the number of nodes and the second component is the
number of arcs. *)
local
 fun evalfun CPN'sizePhi CPN'sizedomain SelfSymFun CPN'n = 
    let 
	val eqsize = OSSizeOfClass CPN'sizePhi CPN'sizedomain SelfSymFun CPN'n 
    in
	(eqsize,eqsize*(NoOfEnabledBE CPN'n))
    end;

 fun combfun ((CPN'a:int,CPN'b:int),(CPN'c,CPN'd)) = (CPN'a+CPN'c,CPN'b+CPN'd)
in
 fun OSSizeOfOGraph CPN'sizePhi CPN'sizedomain SelfSymFun = 
      SearchAllNodes(fn n => true, 
                     evalfun CPN'sizePhi CPN'sizedomain SelfSymFun, 
                     (0,0),
                     combfun)
end;

end (* struct CPN'OESizes *)

(* For Lamport, N=3:
OSSizeOfOGraph 6 4;
*)

