(*
 * Module: Includes the functions that we make available for the user
to help him write equivalence specifications. 
This file is going to be expanded with more useful functions.
 *
 * Description:  Design/CPN OE/OS Graph Tool
 *
 * Created by:   Lars Kristensen (kris@daimi.aau.dk)   
 *               Jens Baek Jorgensen (jbj@daimi.aau.dk)
 * Date:         01/02/96   
 *
 * Design/CPN (tm)  Copyright (c) Meta Software Corp 1989--1995
 *)

(* $Source: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OEUtilities/OESpecInterface.sml,v $

$Log: OESpecInterface.sml,v $
Revision 1.4  1996/06/27 14:43:34  cpn
Added CPN' to intersect functions cf. new interface

Revision 1.3  1996/02/02 14:10:54  jbj
Additions.

 Revision 1.1  1996/02/01  11:34:58  jbj
 Initial revision
 

*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OEUtilities/OESpecInterface.sml,v 1.4 1996/06/27 14:43:34 cpn Exp $";

structure CPN'OESpecInterface =
struct

(* Proj1 : (''a * 'b) ms -> ''a ms *)
val Proj1 = CPN'OEMulBasics.proj1;

(*  Proj2 : ('a * ''b) ms -> ''b ms *)
val Proj2 = CPN'OEMulBasics.proj2;

(* CreateRes : 'a ms -> 'b ms -> ('a list * 'b list) list *)
val CreateRes = CPN'OEConvertMuls.mss_to_rss;    

(* CreateComp : 'a ms -> 'a list list *)
val CreateComp = CPN'OEConvertMuls.ms_to_comp_ord;

(* CompSize: int -> ''a list -> int *)
val CompSize = CPN'OEResUtils.CompSize;

(* IntersectResSets : (''a list * ''b list) list list -> (''a list * ''b list) list *)
val CPN'IntersectResSets = CPN'OEResAndComp.intersectL;

(* IntersectCompSets: ''a list list list -> ''a list list *)
val CPN'IntersectCompSets = CPN'OEResAndComp.intersectIIL;

(* CreateCanonicals : ('a list * 'b list) list -> ('a * 'b) list list *)
val CreateCanonicals = CPN'OEResToCanon.rss_to_canon;

(* PermExists : 'a ms -> 'b ms -> bool *)
val PermExists = CPN'OEApplyAndTest.perm_exists;

(* ApplyCanonical : (''a * ''a) list -> ''a ms -> ''a ms *)
val ApplyCanonical = CPN'OEApplyAndTest.apply_canon;

(* ApplyCanonicalPair : (''a * ''a) list
                              -> (''a * ''a) ms -> (''a * ''a) ms *)
val ApplyCanonicalPair = CPN'OEApplyAndTest.apply_canon_on_pair;

(* OKCanonicalPair : 
((''a * ''a) ms * (''a * ''a) ms) list -> (''a * ''a) list list -> bool *)
val OKCanonicalPair = CPN'OEApplyAndTest.OKCanonicalPair;

(* ApplyRotation : ('a * 'b -> 'c) -> 'b -> 'a ms -> 'c ms *)
val ApplyRotation = CPN'OEApplyAndTest.apply_rot;

(* OKRotation : ('a * 'b -> ''c) -> ('a ms * ''c ms) -> 'b list -> bool *)
val OKRotation = CPN'OEApplyAndTest.OKRotation;

(* NoOfEnabledBE : Node -> int *)
val  NoOfEnabledBE = CPN'OESizes.NoOfEnabledBE;

(* SizeOfClass : int -> int -> ('a -> 'b list list) -> 'a -> int *)
val OSSizeOfClass = CPN'OESizes.OSSizeOfClass;

(* SizeOfOGraph : int -> int -> (Node -> 'a list list) -> int * int *)
val OSSizeOfOGraph = CPN'OESizes.OSSizeOfOGraph; 

end (* struct CPN'OESpecInterface *)

open CPN'OESpecInterface;

