(*
 * Module:       Design/CPN Branching Library
 *
 * Description:  Main function for Branching Library
 *
 * Created by:   Lars Michael Kristensen (kris@daimi.aau.dk)   
 *
 * Date:         08/12/97   
 *
*)

structure OGSetBranching = 
    struct
	val CPN'BranchingFun = ref (NONE : ((Node * Bind.Elem list -> Bind.Elem list) option));

	fun DisableBranchingFun () = (CPN'BranchingFun := NONE);
	    
	fun EnableBranchingFun CPN'bfun = (CPN'BranchingFun := SOME (CPN'bfun));
end


fun CPN'ReduceBEs (CPN'n,CPN'enab) = 
   (case !OGSetBranching.CPN'BranchingFun of 
    NONE => CPN'enab
  | SOME (CPN'branchingfun) =>
	let
	    (* map enabled binding elements to external representation *)
	    val CPN'extbel = (map CPN'IntBEToExtBE CPN'enab);
		
	    (* reduce according to the branching function set by user *)
	    val CPN'redbel = (CPN'branchingfun (CPN'n,CPN'extbel));
		
	    (* map the reduced binding element list back into internal representation *)
	    val CPN'intbel = (map BEToArc CPN'redbel)
	in
	    (* make sure only enabled binding elements are in the resulting list *)     
	    (mapfilter (ref (fn CPN'x => (mem CPN'enab CPN'x,CPN'x)),CPN'intbel))
	end);
	
