(*
 * Module:       Basic drawing functions
 *
 * Description:  Design/CPN OG Tool
 *
 * Created by:   Lars Kristensen (kris@daimi.aau.dk)   
 *
 * Date:         29/09/95   
 *
 * Design/CPN (tm)  Copyright (c) Meta Software Corp 1989--1995
 *)

(* $Source: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OGDrawFuns.sml,v $

$Log: OGDrawFuns.sml,v $
Revision 1.2.4.1  1999/01/14 16:46:33  mailund
time port

Revision 1.2  1996/02/04 13:57:58  kris
Display predecessors now uses max_faninref

 Revision 1.1  1995/09/29  14:37:03  kris
 Initial revision
 

*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OGDrawFuns.sml,v 1.2.4.1 1999/01/14 16:46:33 mailund Exp $";

(* this structure defines all the basic drawing function to be used in the *)
(* OG Analyser -- installed as coded in OGInitFilters.sml *)

structure CPN'OGDrawFuns
= struct

(* Checks if the given node is already drawn on the given
page *)
fun IsNodeDrawn CPN'n CPN'pg=
   ((CPN'GetOGNode {no=CPN'n,  id=CPN'pg}; true) 
     handle CPN'GetOGNodeExc => false);   

(* Checks if the given arc is already drawn on the given
page *)
fun IsArcDrawn CPN'a CPN'pg=
   ((CPN'GetOGArc {no=CPN'a,  id=CPN'pg}; true) 
     handle CPN'GetOGArcExc => false);   

(* creates the OG node for a node spec by the user - if not already there *)
fun CreateSpecNode (CPN'n, CPN'pg) =
      ((if (not (IsNodeDrawn CPN'n CPN'pg))
      then
       DSStr_SetCurObject(CPN'OGDrawUtils.CreateNode
         {x=0,y=0,page=DSStr_GetCurPage(),ognode=CPN'n})
      else ());
      CPN'Edit.out_val "1")
          handle CPN'AvlTree.ExcAvlLookup => CPN'Edit.out_val "0";

fun CreateSpecArc (CPN'a, CPN'pg) =
      ((if (not (IsArcDrawn CPN'a CPN'pg))
      then
         DSStr_SetCurObject(CPN'OGDrawUtils.CreateConn({page=CPN'pg,
           node1 = CPN'OGDrawUtils.CreateNodeCheck({x=0,y=0,page=CPN'pg,
                                         ognode=CPN'OGUtils.GetNodeNo(!(CPN'OGUtils.GetArcSrc(CPN'OGUtils.ArcToArcRec(CPN'a))))}),
       	   node2 = CPN'OGDrawUtils.CreateNodeCheck({x=(#horz(!CPN'OGDrawUtils.nodesuccoffsetref)),
                       y=(#vert(!CPN'OGDrawUtils.nodesuccoffsetref)),page = CPN'pg,
                       ognode = CPN'OGUtils.GetNodeNo(!(CPN'OGUtils.GetArcDest(CPN'OGUtils.ArcToArcRec(CPN'a))))})},CPN'a))
      else ());
      CPN'Edit.out_val "1")
         handle CPN'AvlTree.ExcAvlLookup => CPN'Edit.out_val "0";

fun listpick(CPN'hd::CPN'tl,CPN'c) (* pick the CPN'c first elems of a list *)
   = if CPN'c<=0
     then []
     else CPN'hd::(listpick(CPN'tl,CPN'c-1))
  | listpick([],_) = [];

local
(* Special-purpose difference function used to find
non-drawn arcs in the following DrawSuccs and
DrawPreds functions. *)
fun diff [] CPN'l' CPN'acc = CPN'acc
 |     diff (CPN'ar::CPN'al) CPN'l' CPN'acc 
             = if mem CPN'l' (CPN'OGUtils.GetArcNo (!CPN'ar)) 
                then diff CPN'al CPN'l' CPN'acc
                else diff CPN'al CPN'l' (CPN'ar::CPN'acc); 
in

(* Draws the successors to an OG node given by it's *)
(* modstruct, CPN'gn; new OG nodes are created *)
(* as needed, and are for now placed displaced from CPN'gn or an *)
(* already drawn successor without any user interaction. *)
fun DrawSuccs ((CPN'n1, CPN'gn1, CPN'al), CPN'gp) =
 let
  val {x=CPN'x,y=CPN'y}=(DSRdAttr_GetObjectCenter CPN'gn1)
  val nextsuccpos = ref {x=CPN'x+(#horz(!CPN'OGDrawUtils.nodesuccoffsetref)),
                         y=CPN'y+(#vert(!CPN'OGDrawUtils.nodesuccoffsetref))}
  val newnodes = ref ([]:int list);
 in
  if CPN'n1<>0
  then
   map
    (fn ref(CPN'OGarc{dest=
         ref(CPN'OGnode{no=CPN'n2,...}),no=CPN'a,...})
       =>  let
              val CPN'gn2 = CPN'GetOGNode {no=CPN'n2,id=CPN'gp}
                   handle CPN'GetOGNodeExc =>
                     (CPN'OGDrawUtils.CreateNode{x= #x(!nextsuccpos),
                      y= #y(!nextsuccpos),page=CPN'gp,ognode=CPN'n2})
            in
             newnodes:=CPN'gn2::(!newnodes);
             nextsuccpos:= {x=   #x(!nextsuccpos)+(#horz(!CPN'OGDrawUtils.succsuccoffsetref)),
                            y= #y(!nextsuccpos)+(#vert(!CPN'OGDrawUtils.succsuccoffsetref))};
             CPN'OGDrawUtils.CreateConn
               ({page=CPN'gp,node1=CPN'gn1,node2=CPN'gn2},CPN'a)
            end
	)
    (
	  let
	      val allarcs = 
		  (!(CPN'OGUtils.GetNodeSucclist(CPN'OGUtils.NodeToNodeRec CPN'n1)));
	      val nondrawnarcs 
		  = diff allarcs CPN'al []
	  in
	      if !CPN'OGDrawUtils.max_fanoutref=0 (*no limit*)
		  then nondrawnarcs
	      else listpick(nondrawnarcs,!CPN'OGDrawUtils.max_fanoutref)
	  end(*let*)
	  )
  else
      [];
      (!newnodes)
 end;

(* Draws the predecessors to an OG node given by it's   *)
(* modstruct, CPN'gn; new  OG nodes are created *)
(* as needed, and are for now placed displaced from CPN'gn or an *)
(* alrady drawn predecessor without any user interaction. *)
fun DrawPreds ((CPN'n1, CPN'gn1, CPN'al), CPN'gp) =
let
  val {x=CPN'x,y=CPN'y}=DSRdAttr_GetObjectCenter CPN'gn1;
  val nextpredpos = ref {x=CPN'x+(#horz(!CPN'OGDrawUtils.nodepredoffsetref)),
                         y=CPN'y+(#vert(!CPN'OGDrawUtils.nodepredoffsetref))}
  val newnodes = ref ([]:int list);
 in
  if CPN'n1<>0
  then
   map
    (fn ref(CPN'OGarc{src=
         ref(CPN'OGnode{no=CPN'n2,...}),no=CPN'a,...})
       =>  let
              val CPN'gn2 = CPN'GetOGNode {no=CPN'n2, id=CPN'gp}
                   handle CPN'GetOGNodeExc =>
                     (CPN'OGDrawUtils.CreateNode{x= #x(!nextpredpos),
                      y= #y(!nextpredpos),page=CPN'gp,ognode=CPN'n2})
            in
             newnodes:=CPN'gn2::(!newnodes);
             nextpredpos:= {x=   #x(!nextpredpos)+(#horz(!CPN'OGDrawUtils.predpredoffsetref)),
                            y= #y(!nextpredpos)+(#vert(!CPN'OGDrawUtils.predpredoffsetref))};
             CPN'OGDrawUtils.CreateConn
               ({page=CPN'gp,node1=CPN'gn2,node2=CPN'gn1},CPN'a)
            end
)(
    let
     val allarcs = 
(!(CPN'OGUtils.GetNodePredlist(CPN'OGUtils.NodeToNodeRec CPN'n1)));
     val nondrawnarcs 
          = diff allarcs CPN'al []
    in
     if !CPN'OGDrawUtils.max_faninref=0 (*no limit*)
     then nondrawnarcs 
     else listpick(nondrawnarcs,!CPN'OGDrawUtils.max_faninref)
    end(*let*)
    )
   else
    [];
   (!newnodes)
 end;
end (* local *);

(* For the user to set the delimiters etc for the OG node inscriptions *)
fun SetInscrParams{
     NodeNumber = {ShowNumber= CPN'nnum,
                   Split= CPN'sp},
     NeighborInfo = {Unknown= CPN'up,
                     PartlyKnown= CPN'pp,
                     Separator= CPN'ps},
     Deadlock= CPN'dl}
   = (CPN'OGDrawUtils.nodenumref:=CPN'nnum;   CPN'OGDrawUtils.unprocref:=CPN'up;
      CPN'OGDrawUtils.partprocref:=CPN'pp;    
   CPN'OGDrawUtils.predsuccref:=CPN'ps;
      CPN'OGDrawUtils.deadlockref:=CPN'dl);
      
 fun CreateNodeDescr CPN'gp (CPN'n, CPN'gn)
    = let
       val {x=CPN'x,y=CPN'y}=DSRdAttr_GetObjectCenter CPN'gn;
       val CPN'dr=CPN'OGDrawUtils.CreateNodeInscrReg{obj=CPN'gn,
                  text= !st_NodeDescrRef CPN'n,
                  x=CPN'x,y=CPN'y,page=CPN'gp}
      in
        DSUI_Align{obj=CPN'dr,aligntype=ALN_LR,ref1=CPN'gn,ref2=0};
        DSUI_Align{obj=CPN'dr,aligntype=ALN_BT,ref1=CPN'gn,ref2=0};
        CPN'Edit.out_val (makestring CPN'dr)
      end(*let*);

 fun CreateArcDescr CPN'gp (CPN'a, CPN'ga)
    = let
       val {node1=CPN'gn1,node2=CPN'gn2}
          = DSRdAttr_GetConnEnds{conn=CPN'ga,rgn=false};
       val {x=CPN'x1,y=CPN'y1}=DSRdAttr_GetObjectCenter CPN'gn1;
       val {x=CPN'x2,y=CPN'y2}=DSRdAttr_GetObjectCenter CPN'gn2;
       val CPN'dr = 
       CPN'OGDrawUtils.CreateArcInscrReg{obj=CPN'ga,
        text= !st_ArcDescrRef CPN'a,
         x=CPN'x1-(CPN'x1-CPN'x2)div 2,
         y=CPN'y1-(CPN'y1-CPN'y2)div 2,page=CPN'gp}
      in
        CPN'Edit.out_val (makestring CPN'dr)
      end(*let*);
               
end (*struct*);

  CPN'OGDrawFuns.SetInscrParams{
     NodeNumber = {ShowNumber= true,
                   Split= "*"},
     NeighborInfo = {Unknown= "?",
                     PartlyKnown= "+",
                     Separator= "="},
     Deadlock= "DEAD"};
     
