(*
 * Module:       Additional functions for drawing
 *
 * Description:  Design/CPN OG Tool
 *
 * Created by:   Lars Kristensen (kris@daimi.aau.dk)   
 *
 * Date:         29/09/95   
 *
 * Design/CPN (tm)  Copyright (c) Meta Software Corp 1989--1995
 *)

(* $Source: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OGDrawSuppl.sml,v $

$Log: OGDrawSuppl.sml,v $
Revision 1.1  1995/09/29 14:37:53  kris
Initial revision
 

*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OGDrawSuppl.sml,v 1.1 1995/09/29 14:37:53 kris Exp $";

exception NotAPath;

fun DisplayNodePath (CPN'nl : Node list) 
    = let
      val nextsuccpos = ref {x=0, y=0};
      val CPN'gp = DSStr_GetCurPage();

      fun CPN'findGrNode CPN'n
         = let
            val CPN'gn = CPN'GetOGNode {no=CPN'n,id=CPN'gp}
                          handle CPN'GetOGNodeExc =>
                           (CPN'OGDrawUtils.CreateNode{x= #x(!nextsuccpos),
                            y= #y(!nextsuccpos),page=CPN'gp,ognode=CPN'n});
            val {x=CPN'x,y=CPN'y} = DSRdAttr_GetObjectCenter CPN'gn;
           in
            nextsuccpos := {x=CPN'x+(#horz(!CPN'OGDrawUtils.nodesuccoffsetref)),
                            y=CPN'y+(#vert(!CPN'OGDrawUtils.nodesuccoffsetref))};
            CPN'gn
           end;

      fun CPN'DrawArcs _ _ [] = ()
        | CPN'DrawArcs CPN'gn1 CPN'gn2 (CPN'a::CPN'al)
         = ((CPN'GetOGArc{no=CPN'a, id=CPN'gp})
             handle CPN'GetOGArcExc =>
              CPN'OGDrawUtils.CreateConn
               ({page=CPN'gp,node1=CPN'gn1,node2=CPN'gn2},CPN'a);
            CPN'DrawArcs CPN'gn1 CPN'gn2 CPN'al);

      fun CPN'find_arcs [] = []
        | CPN'find_arcs [_] = []
        | CPN'find_arcs (CPN'n1::(CPN'n2::CPN'rest))
         = (case Arcs(CPN'n1,CPN'n2)
            of [] => raise NotAPath
             | CPN'al => CPN'al)::(CPN'find_arcs (CPN'n2::CPN'rest));
      fun CPN'DrawPath' [] _ = [] (* should not happen *)
        | CPN'DrawPath' [CPN'gn1] _ = [CPN'gn1]
        | CPN'DrawPath' (CPN'gn1::(CPN'n2::CPN'rest)) (CPN'al::CPN'all)
         = let
            val CPN'gn2 = CPN'findGrNode CPN'n2
           in
            CPN'DrawArcs CPN'gn1 CPN'gn2 CPN'al;
            CPN'gn1::(CPN'DrawPath' (CPN'gn2::CPN'rest) CPN'all)
           end;

      fun CPN'DrawPath [] _ = []
        | CPN'DrawPath (CPN'n1::CPN'rest) CPN'all
         = CPN'DrawPath' ((CPN'findGrNode CPN'n1)::CPN'rest) CPN'all;

      val CPN'all = CPN'find_arcs CPN'nl
     in
      DSStr_SetCurGroup (CPN'OGListUtils.remdupl (CPN'DrawPath CPN'nl CPN'all))
     end;

fun DisplayArcPath (CPN'al : Arc list)
    = let
	  val nextpos = ref {x=0,y=0};
	  val CPN'gp = DSStr_GetCurPage();
	      
	  fun CPN'find_nodes [] = []
	    | CPN'find_nodes (CPN'a::nil) = [SourceNode(CPN'a),DestNode(CPN'a)]
	    | CPN'find_nodes (CPN'a1::(CPN'a2::CPN'rest)) =
						 if DestNode(CPN'a1) = SourceNode(CPN'a2)
						     then SourceNode(CPN'a1)::(CPN'find_nodes (CPN'a2::CPN'rest))
						 else
						     raise NotAPath

	  fun CPN'DrawPath' (CPN'a,CPN'n1,CPN'n2) =
	      let
		  val CPN'ga = ((CPN'GetOGArc{no=CPN'a, id=CPN'gp})
				handle CPN'GetOGArcExc =>
				    (CPN'OGDrawUtils.CreateConn({page = CPN'gp,
								 node1 = CPN'OGDrawUtils.CreateNodeCheck({x= #x(!nextpos),y= #y(!nextpos),page=CPN'gp,ognode= CPN'n1}),
								 node2 = CPN'OGDrawUtils.CreateNodeCheck({x= #x(!nextpos) + #horz(!CPN'OGDrawUtils.nodesuccoffsetref),y = #y(!nextpos) + #vert(!CPN'OGDrawUtils.nodesuccoffsetref),page=CPN'gp,ognode=CPN'n2})},CPN'a)));
		  val CPN'gn1 = CPN'GetOGNode{no=CPN'n1,id=CPN'gp};
		  val CPN'gn2 = CPN'GetOGNode{no=CPN'n2,id=CPN'gp};
		  val {x=CPN'x, y=CPN'y} = DSRdAttr_GetObjectCenter CPN'gn2;
              in
                  nextpos := {x= CPN'x +(#horz(!CPN'OGDrawUtils.nodesuccoffsetref)) , y= CPN'y + (#vert(!CPN'OGDrawUtils.nodesuccoffsetref))};
                  ([CPN'gn1,CPN'gn2])
              end;
		  
	  fun CPN'DrawPath ([], CPN'nl) = []
	    |  CPN'DrawPath ((CPN'a::CPN'arest),(CPN'n1::(CPN'n2::CPN'nrest)))
	       =
	       (CPN'DrawPath'(CPN'a,CPN'n1,CPN'n2))::(CPN'DrawPath(CPN'arest,CPN'n2::CPN'nrest))
	       
      in
	   DSStr_SetCurGroup (CPN'OGListUtils.remdupl (CPN'OGListUtils.flatten(CPN'DrawPath (CPN'al, (CPN'find_nodes CPN'al)))))
      end;

fun DisplayArcs(CPN'al : Arc list)
    = let 
	  val nextpos = ref {x=0, y= 0}
	  val CPN'gp = DSStr_GetCurPage();
	      
	  fun CPN'findGrArc CPN'a
	      = 
	      let
		  val CPN'ga = CPN'GetOGArc{no=CPN'a, id=CPN'gp}
		      handle CPN'GetOGArcExc =>
			  (CPN'OGDrawUtils.CreateConn({page = CPN'gp,
				node1 = CPN'OGDrawUtils.CreateNodeCheck({x= #x(!nextpos),y= #y(!nextpos),page=CPN'gp,ognode= CPN'OGUtils.GetNodeNo(!(CPN'OGUtils.GetArcSrc(CPN'OGUtils.ArcToArcRec(CPN'a))))}),
						       node2 = CPN'OGDrawUtils.CreateNodeCheck({x= #x(!nextpos) + #horz(!CPN'OGDrawUtils.nodesuccoffsetref),y = #y(!nextpos) + #vert(!CPN'OGDrawUtils.nodesuccoffsetref),page=CPN'gp,ognode=CPN'OGUtils.GetNodeNo(!(CPN'OGUtils.GetArcDest(CPN'OGUtils.ArcToArcRec(CPN'a))))})},
			  CPN'a));
	      in
		  nextpos := {x= #x(!nextpos) + 2*(#horz(!CPN'OGDrawUtils.nodesuccoffsetref)) , y= #y(!nextpos) + 2*(#vert(!CPN'OGDrawUtils.nodesuccoffsetref))};
		  [(CPN'GetOGNode{no=SourceNode(CPN'a),id=CPN'gp}),(CPN'GetOGNode{no=DestNode(CPN'a),id=CPN'gp})]
	      end;
	  
	  fun CPN'DrawArcs [] = []
	    | CPN'DrawArcs (CPN'a1::CPN'rest) =
	      (CPN'findGrArc CPN'a1)::(CPN'DrawArcs CPN'rest)
      in
	  DSStr_SetCurGroup (CPN'OGListUtils.remdupl (CPN'OGListUtils.flatten (CPN'DrawArcs (CPN'al))))
      end;
      

fun DisplayNodes (CPN'nl : Node list) 
    = let 
	  val nextpos = ref {x=0, y= 0};
	  val CPN'gp = DSStr_GetCurPage();

	  fun CPN'findGrNode CPN'n 
	      = let
		    val CPN'gn = CPN'GetOGNode {no=CPN'n, id = CPN'gp}
			handle CPN'GetOGNodeExc => (CPN'OGDrawUtils.CreateNode{x= #x(!nextpos),y= #y(!nextpos), page= CPN'gp, ognode= CPN'n});
val {x=CPN'x, y= CPN'y} = DSRdAttr_GetObjectCenter CPN'gn;
		in
		    nextpos := {x=CPN'x+ (#horz(!CPN'OGDrawUtils.nodesuccoffsetref)), y= CPN'y + (#vert(!CPN'OGDrawUtils.nodesuccoffsetref))};
		    CPN'gn
		end;

	  fun CPN'DrawNodes [] = []
	    | CPN'DrawNodes (CPN'n1::CPN'rest) = 
	      (CPN'findGrNode CPN'n1)::(CPN'DrawNodes CPN'rest)
	      
      in
	  DSStr_SetCurGroup (CPN'OGListUtils.remdupl (CPN'DrawNodes CPN'nl))
      end;
