(*
 * Module:       Functions corresponding to the menu entries on the C side.
 *
 * Description:  Design/CPN OG Tool
 *
 * Created by:   Lars Kristensen (kris@daimi.aau.dk)   
 *
 * Date:         29/09/95   
 *
 * Design/CPN (tm)  Copyright (c) Meta Software Corp 1989--1995
 *)

(* $Source: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OGMenu.sml,v $

$Log: OGMenu.sml,v $
Revision 1.8.4.1  1999/01/04 09:16:29  mailund
char vs. string

Revision 1.8  1996/11/27 19:24:04  sorenchr
fixed problem with unclosed strings.

Revision 1.7  1996/09/27 10:15:26  sorenchr
Changed OGSimStatetoOG to return values to C at this level.

Revision 1.6  1996/08/27 12:10:56  cpn
Mac Port v3.0.2

Revision 1.5  1996/06/06 10:09:10  cpn
Integrated OE/OS Tool modifications (1)

Revision 1.4  1996/05/14 12:31:16  cpn
*** empty log message ***

Revision 1.3  1996/04/12 09:42:51  cpn
Added CPN' to IsSccGen

 Revision 1.2  1995/12/08  12:03:18  kris
 Added use of the new save report

 Revision 1.1  1995/09/29  14:44:51  kris
 Initial revision
 

*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OGMenu.sml,v 1.8.4.1 1999/01/04 09:16:29 mailund Exp $";

structure OG'menu =
struct
 fun CalculateOG ()
    = GenOG [];

fun CalculateSuccessors (CPN'nl:Node list) 
    =   map
           (fn CPN'n =>
                if !(CPN'OGUtils.GetNodeCalcStat
                      (CPN'OGUtils.NodeToNodeRec CPN'n))<>FullProc
               then
                   (CPN'OGNodeSel.StartSelOnlyOne();
                   GenOG [CPN'n] handle ? => ();
                   CPN'OGNodeSel.RestoreStdSel())             
              else
                  ()
             ) CPN'nl;

fun CalculateScc () = 
    (DSUI_SetStatusBarMessage("SCC Generation...");
     CalculateSccGraph ();
     DSUI_SetStatusBarMessage("SCC Generation: "^
			      "#Nodes = "^(makestring(SccNoOfNodes()))^
     " #Arcs  = "^(makestring(SccNoOfArcs()))^
			      "#Secs = "^(makestring(SccNoOfSecs()))))

val DisplayNode = CPN'OGDrawFuns.CreateSpecNode;

val DisplayArc = CPN'OGDrawFuns.CreateSpecArc;

local
fun merge [] CPN'l' = CPN'l'
 |    merge  (CPN'a::CPN'l) CPN'l' =
        if mem CPN'l' CPN'a then 
           merge CPN'l CPN'l' 
       else 
          CPN'a::(merge CPN'l CPN'l');

fun DisplaySuccessors' [] CPN'gp CPN'acc 
                  = (if CPN'acc<>[] then DSStr_SetCurGroup CPN'acc
                     else ();
                     CPN'acc)
   |    DisplaySuccessors'  (CPN'argl::CPN'l) CPN'gp CPN'acc
                  = DisplaySuccessors' CPN'l CPN'gp 
                           (merge CPN'acc (CPN'OGDrawFuns.DrawSuccs (CPN'argl, CPN'gp)));

fun DisplayPredecessors' [] CPN'gp CPN'acc 
                  = (if CPN'acc<>[] then DSStr_SetCurGroup CPN'acc
                     else ();
                     CPN'acc)
   |    DisplayPredecessors'  (CPN'argl::CPN'l) CPN'gp CPN'acc
                  = DisplayPredecessors' CPN'l CPN'gp 
                           (merge CPN'acc (CPN'OGDrawFuns.DrawPreds (CPN'argl, CPN'gp)))

in

 fun DisplaySuccessors (CPN'argl, CPN'gp) = 
               DisplaySuccessors' CPN'argl CPN'gp [];

fun DisplayPredecessors (CPN'argl, CPN'gp) = 
               DisplayPredecessors' CPN'argl CPN'gp []

end;


fun DisplayScc () = (SccDisplay ()
		     handle SccGraphNotCalculated => 
			 DSUI_UserAckMessage ("The Scc graph has not been calculated"))


fun DisplayNodeDescriptor (CPN'argl, CPN'gp) =             
         app (CPN'OGDrawFuns.CreateNodeDescr CPN'gp) CPN'argl;

fun DisplayArcDescriptor (CPN'argl, CPN'gp) =             
         app (CPN'OGDrawFuns.CreateArcDescr CPN'gp) CPN'argl;


val UpdateInscriptions =
    app CPN'OGDrawUtils.InscrNode;

val OGStatetoSim = OGStatetoSim;

fun SimStatetoOG ()
    = let
       val (CPN'wasthere, CPN'number) = OGSimStatetoOG();
      in
	  CPN'Edit.out_val(if CPN'wasthere then "0" else "1");
	  CPN'Edit.out_val(makestring(CPN'number))
      end;
      

val SaveReport = ();



fun ShowStatistics ()
    = ("Occurrence Graph"
       ^"\n  Nodes:   "^(makestring(NoOfNodes()))
       ^"\n  Arcs:    "^(makestring(NoOfArcs()))
       ^"\n  Secs:    "^(CPN'OGStopCrit.GetTotalTime())
       ^"\n  Status:  "
       ^(if EntireGraphCalculated() then "Full" else "Partial")^"\n"
       ^(if (!CPN'IsSCCGen) 
	     then "\nScc Graph"
		 ^"\n  Nodes:   "
		 ^(makestring(SccNoOfNodes()))
		 ^"\n  Arcs:    "^(makestring(SccNoOfArcs()))
		 ^"\n  Secs:    "^(makestring(SccNoOfSecs()))
	 else ""))




val AttrOpt = CPN'OGDrawUtils.AttrOptionsGen;			   

end(*struct*);
