(*
 * Module:
 *
 * Description:  Design/CPN OG Tool
 *
 * Created by:   Lars Kristensen (kris@daimi.aau.dk)   
 *
 * Date:         29/09/95   
 *
 * Design/CPN (tm)  Copyright (c) Meta Software Corp 1989--1995
 *)

(* $Source: /users/cpn/designcpnR/src/current/oga/OGMLfiles/Attic/OGNewCalcSucc.sml,v $

$Log: OGNewCalcSucc.sml,v $
Revision 1.1.2.3  2000/08/18 12:09:25  kris
Modification to reflect that branching predicate now work on state recs

Revision 1.1.2.2  2000/07/13 13:43:26  kris
Added support for canonical states in the OG part

Revision 1.1.2.1  2000/05/12 19:23:24  kris
Initial Revision


*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/oga/OGMLfiles/Attic/OGNewCalcSucc.sml,v 1.1.2.3 2000/08/18 12:09:25 kris Exp $";


structure CalculateStateSpace = 
struct

    val newnodecount = ref 0;  (* temp counter for new nodes *)
    val newarccount = ref 0;  (* temp counter for new arcs *)
 
    val CPN'curtod = ref 0;

    (* exception used to catch exception raised during generation *)
    exception CPN'OGExcDuringGen (*  of Node *) ;

    (* --- insert an arc between two nodes --- *)
    fun createnewarc (srcnoderef,destnoderef,(CPN'tri,CPN'b)) = 
	let
	    val _ = OGTimeProfile.BeginObs 3;
	    val newarcnum = CPN'OGNodeSel.NewArcNum();
		
	    val newarc = ref (CPN'OGarc {src=srcnoderef,
					 dest=destnoderef,
					 no=newarcnum,
					 inscr=(CPN'tri,CPN'BindSearchTree.Insert
						(CPN'Binds,CPN'b,CPN'OGEncodeBinds CPN'b))})
	in
	    (* --- update the graphical part --- *)
	    CPN'AvlTree.AvlInsert CPN'OGArcs(mkst_Arc newarcnum,newarc);
	    CPN'inc newarccount;
	    
	    (* --- update successors/predecessor field properly --- *)
	    CPN'OGUtils.GetNodePredlist(!destnoderef):=
	    newarc::(!(CPN'OGUtils.GetNodePredlist(!destnoderef)));
	    
	    CPN'OGUtils.GetNodeSucclist(!srcnoderef):=
	    newarc::(!(CPN'OGUtils.GetNodeSucclist(!srcnoderef)));
	    OGTimeProfile.EndObs 3
	end;

 (* --- execute CPN'tri with CPN'b in CPN'n and store the result in the OG structures --- *)
 fun processBindElem CPN'n (CPN'tri,CPN'b) =
     (
      OGTimeProfile.BeginObs 4;
      OGTimeProfile.BeginObs 6;      
      CPN'OGToSimData.copyStateRec (!CPN'n); (* prepare the simulator data records *)
      let
	  (* --- execute CPN'tri with CPN'b, and record the new binding --- *)
          (* --- this is to handle output from codesegments in the right way --- *)
	  val CPN'b = (CPN'OGExecute(CPN'tri,CPN'b) 
		       handle CPN'e => 
			   (DSUI_UserAckMessage 
			    ("ML exception "^(exnName CPN'e)^
			     " raised during processing of binding element: "^
			     (st_BE (CPN'IntBEToExtBE (CPN'tri,CPN'b)))^" in node: "^
			     (makestring (CPN'OGUtils.GetNodeNo (!CPN'n))));
			    raise CPN'OGExcDuringGen));
	  val _ = OGTimeProfile.EndObs 6;	      
	  (* --- the the internal reference to CPN'n --- *)
	  val srcnoderef = CPN'n; 


	      val staterec = CPN'OEUtils.CPN'OESetTempMarkRec 
		  (CPN'OEstore_state(srcnoderef,CPN'tri)); 
	      val (insertstaterec,wasthere) = 
		  CPN'RecSearchTree.Member(CPN'State'recs, staterec, 
					   ((!CPN'OEEncode.CPN'EncodeMarkFun) staterec));



	  val _ =  OGTimeProfile.EndObs 7;
      in
	  
	  if not wasthere 
              then
		  (* --- we have a new state and hence we create new node --- *)
		  let
	           val _ =  OGTimeProfile.BeginObs 8;

                   val staterec = CPN'OGstore_state(srcnoderef,CPN'tri);
		   val (insertstaterec as 
			ref(CPN'OGState(innerrec as {owner=ref(noderef),...})),wasthere) = CPN'RecSearchTree.Insert(CPN'State'recs,staterec,
			      ((!CPN'OEEncode.CPN'EncodeMarkFun) staterec));      


		      val newnodenum = CPN'OGNodeSel.NewNodeNum();
		      val destnoderef 
			  = ref( CPN'OGnode {state=insertstaterec,
					     no=newnodenum,
					     succlist=ref [],
					     predlist=ref [],
					     calcstat=ref UnProc})
		  in

		      (* --- update fields --- *)
		      #owner innerrec:=destnoderef;

		      (* --- create the new arc --- *)
		      createnewarc(srcnoderef,destnoderef,(CPN'tri,CPN'b));

		      (* --- insert the node in the graphical part --- *)
		      CPN'AvlTree.AvlInsert CPN'OGNodes(mkst_Node newnodenum,destnoderef);
		      CPN'inc newnodecount;
		      OGTimeProfile.EndObs 4;
	              OGTimeProfile.EndObs 8;
		      (wasthere,destnoderef)
		  end
	  else 
	      (* --- we have reached an existing state --- *)
	      let 
		val _ = OGTimeProfile.BeginObs 9;

                val destnoderef = (case insertstaterec of  
				       ref(CPN'OGState(innerrec as {owner=ref(noderef),...})) => noderef)

	      in

		   if CPN'OGListUtils.predlist(fn ref(CPN'OGarc{dest=CPN'dnr,inscr=(CPN'tri2,ref CPN'b2),...})
					       => (CPN'dnr=destnoderef)andalso (CPN'OEBindUtils.CPN'OEBindingIsEq((#1 CPN'tri),(#2 CPN'tri),CPN'b,(#1 CPN'tri2),(#2 CPN'tri2),CPN'b2)))
		       
		       (!(CPN'OGUtils.GetNodeSucclist(!srcnoderef)))

		      (!(CPN'OGUtils.GetNodeSucclist(!srcnoderef)))
		      then ()
		  else 
		      createnewarc(srcnoderef,destnoderef,(CPN'tri,CPN'b));

	           OGTimeProfile.EndObs 4;
		   OGTimeProfile.EndObs 9;
		  (wasthere,destnoderef)
	      end
	  handle _ => (DSUI_UserAckMessage("processBindElem CPN'n = "^(makestring (CPN'OGUtils.GetNodeNo (!CPN'n))));
		       (true,CPN'n))
      end)

 (* --- calculate the enabled transitions in CPN'n --- *)
 fun CPN'CalculateEnabled CPN'n =
     (           
      OGTimeProfile.BeginObs 5;

      CPN'OGEnabData.init();  (* initialization for immediate trans*)
      CPN'OGTimeEnabData.init();  (* initialization for timed trans*)
      
      CPN'OGToSimData.copyStateRec (!CPN'n);  (* prepare the simulator data records *)
      
      CPN'ValuePrint:=false; (* disable communication with C part for reg. sim. *)
      
      ((CPN'OGCalcEnab() (* calculate the enabled binding elements in CPN'n *)
	handle CPN'OGEnabData.TransInstLimitReached
	=> CPN'OGUtils.GetNodeCalcStat (!CPN'n) := PartProc)
       handle CPN'e => 
	   (DSUI_UserAckMessage ("ML exception "^
				 (exnName CPN'e)^
				 " raised during calculation of enabling in node: "^
				 (makestring (CPN'OGUtils.GetNodeNo (!CPN'n))));
	    raise CPN'OGExcDuringGen));
      
      if ((!CPN'env.Timedef)<>CPN'env.timenotdef) andalso
	  (with_time()) andalso (CPN'OGEnabData.get_cands()<>[]) andalso
	  (case (!CPN'OGTimeEnabData.curr_min) of
	       SOME CPN't => IntInf.>(CPN't,(time()))
	     | NONE => true)        (* NONE is larger than anything *)
	  then
	      (* only immediate trans should be executed at current model time *)
	      CPN'OGTimeEnabData.init()
      else
	  CPN'OGCalcEnabTimeBinds();
	  
	  CPN'ValuePrint:=true;
	  
	  if (!CPN'OGEnabData.bindlimit_reached)orelse
	      (CPN'OGTimeEnabData.any_limit()) 
	      then CPN'OGUtils.GetNodeCalcStat (!CPN'n):=PartProc
	  else ();
	      
	      (let
		   val CPN'enabled = (CPN'ReduceBEs 
				      ((CPN'OGUtils.GetNodeNo (!CPN'n)),
				       remdupl ((CPN'OGEnabData.get_cands ())^^
						(CPN'OGTimeEnabData.get_cands()))));
	       in
	           OGTimeProfile.EndObs 5;

		   CPN'enabled
	       end))
		   

 (* --- calculate the enabled binding elements and restore state --- *)
 fun CalculateEnabled CPN'n = 
     let
	 val CPN'CurState = !(CPN'OGCreateStateRec());
	 val CPN'enabled = CPN'CalculateEnabled CPN'n;
	 val _ = CPN'OGToSimData.copyStateRec CPN'CurState
     in
	 CPN'enabled
     end;



 fun CalcSucc' (CPN'n,(CPN'tri,CPN'b)) 
     = (let
	   val _ = (newarccount:=0);  (* initialization *)
	   val _ = (newnodecount:=0); (* initialization *)
	   val (CPN'wasthere,CPN'new) = processBindElem CPN'n (CPN'tri,CPN'b);

       in
	   (* --- mark the node as being partially processed --- *)
	   CPN'OGUtils.GetNodeCalcStat (!CPN'n) := PartProc;

	   if not CPN'wasthere
	       then
		   ((* --- successors should be calculated for this node --- *)
		    (let
			 val CPN'enabled = CPN'CalculateEnabled CPN'new
		     in			
			 (* --- insert the new nodes in the list to be processed --- *)
			 CPN'OGNodeSel.AppendList 
			 (CPN'new,(CPN'ReduceBEs 
				   ((CPN'OGUtils.GetNodeNo (!CPN'new)),CPN'enabled)))
		     end))
	   else
	       ();
	       
	   (* --- freeze time --- *)
	   CPN'curtod := tod();
			  
	   (* --- check stop options --- *)
	   (if CPN'OGStopCrit.IncrAndTest
		(!newnodecount,!newarccount,[CPN'OGUtils.NodeRecToStateRec (!CPN'new)],!CPN'curtod)
		then 
		    raise CPN'OGNodeSel.StopCritSatisfied
	    else ());
		
		
		(* --- check inspection options --- *)
		CPN'OGInspection.Test (!CPN'curtod);
		
		(* --- select and treat next node --- *)
		CalcSucc' (CPN'OGNodeSel.SelectNextNode())
		
	end)
    
     
    fun GenOG' CPN'nl 
	= (
	   CPN'OGStopCrit.InitCounts();
	   CPN'OGInspection.InitCounts();

	   let
	       val CPN'CurState = !(CPN'OGCreateStateRec())
		   handle CPN'AvlTree.ExcAvlLookup => 
		       (DSUI_UserAckMessage("CPN'OGCreateStateRec");
			!(CPN'OGCreateStateRec()))

	   in 
	       if CPN'nl = []
		   then (CPN'OGNodeSel.BuildList
		       (fn CPN'n => (CPN'CalculateEnabled CPN'n))
			handle CPN'AvlTree.ExcAvlLookup => DSUI_UserAckMessage("BuildList"))
	       else
		   (CPN'OGNodeSel.AssignList CPN'nl
		    (fn CPN'n => (CPN'CalculateEnabled CPN'n))
		    handle CPN'AvlTree.ExcAvlLookup => (DSUI_UserAckMessage("AssignList")));

	       (((CalcSucc' (CPN'OGNodeSel.SelectNextNode());
		  CPN'OGToSimData.copyStateRec CPN'CurState)
		 handle CPN'OGNodeSel.NoSelPossible (* we are done *)
		 => (CPN'OGToSimData.copyStateRec CPN'CurState;
		     CPN'OGInspection.StandardReport()))
		handle CPN'OGNodeSel.StopCritSatisfied
		=> (CPN'OGToSimData.copyStateRec CPN'CurState;
		    CPN'OGInspection.StandardReport();
		    raise CPN'OGNodeSel.StopCritSatisfied))
	       handle CPN'OGExcDuringGen (* CPN'n *)
	       => (CPN'OGToSimData.copyStateRec CPN'CurState;
		   DSUI_SetStatusBarMessage("Generation Terminated."))
	     | CPN'AvlTree.ExcAvlLookup => (DSUI_UserAckMessage("OutSide CalcSucc'");
		     raise CPN'AvlTree.ExcAvlLookup)
	   end)
	
    fun CalculateFullStateSpace CPN'nl  = GenOG' CPN'nl;	

    fun DepthFirstGeneration () = (CPN'OGNodeSel.SetGenOrder CPN'OGNodeSel.OG'DepthFirst;
				   CalculateFullStateSpace []);

    fun BreadthFirstGeneration () = (CPN'OGNodeSel.SetGenOrder CPN'OGNodeSel.OG'BreadthFirst;
				     CalculateFullStateSpace []);

end;

(*
 fun ResumeGenOG()
    = (
       CPN'OGStopCrit.InitCounts();
       CPN'OGInspection.InitCounts();
       
       let
	   val CPN'CurState = !(CPN'OGCreateStateRec())
       in 
	   (((CalcSucc'(CPN'OGNodeSel.SelectNextNode());
              CPN'OGToSimData.copyStateRec CPN'CurState)
	     handle CPN'OGNodeSel.NoSelPossible (* we are done *)
	     => (CPN'OGToSimData.copyStateRec CPN'CurState;
		 CPN'OGInspection.StandardReport()))
            handle CPN'OGNodeSel.StopCritSatisfied
	    => (CPN'OGToSimData.copyStateRec CPN'CurState;
		CPN'OGInspection.StandardReport();
		raise CPN'OGNodeSel.StopCritSatisfied)
	    handle CPN'OGExcDuringGen
	    => (CPN'OGToSimData.copyStateRec CPN'CurState;
		DSUI_SetStatusBarMessage("Generation Terminated.")
		))

       end)
*)


fun GenOG CPN'nl =  CalculateStateSpace.CalculateFullStateSpace 
    (map CPN'OGUtils.NodeToNodeRef CPN'nl);


val CalculateOEGraphDepthFirst = CalculateStateSpace.DepthFirstGeneration;
val CalculateOEGraphBreadthFirst = CalculateStateSpace.BreadthFirstGeneration;
val CalculateOEGraph  = CalculateOEGraphBreadthFirst;

