(*
 * Module:
 *
 * Description:  Design/CPN OG Tool
 *
 * Created by:   Lars Kristensen (kris@daimi.aau.dk)   
 *
 * Date:         12/05/00
 *
 * Design/CPN (tm)  Copyright (c) Meta Software Corp 1989--1995
 *)

(* $Source: /users/cpn/designcpnR/src/current/oga/OGMLfiles/Attic/OGNewNodeSel.sml,v $

$Log: OGNewNodeSel.sml,v $
Revision 1.1.2.2  2000/08/18 12:09:15  kris
Modification to reflect that branching predicate now work on state recs

Revision 1.1.2.1  2000/05/12 19:22:01  kris
Initial revision


*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/oga/OGMLfiles/Attic/OGNewNodeSel.sml,v 1.1.2.2 2000/08/18 12:09:15 kris Exp $";

structure CPN'OGNodeSel =
    struct
    
	exception StopCritSatisfied;
	exception NoSelPossible;

	(* --- generation order: breadth or depth first --- *)
	datatype CPN'GenOrder = OG'DepthFirst | OG'BreadthFirst;
	    
	val SS'genogorder = ref OG'BreadthFirst;
	    
	fun SetGenOrder SS'val = (SS'genogorder:= SS'val);
	    
	fun GetGenOrder SS'val = (!SS'genogorder);
	    
	(* --- node/arcs numbering --- *)
	val NextNodeNum = ref 0;
	fun NewNodeNum () = (CPN'inc NextNodeNum;!NextNodeNum);
	    
	val NextArcNum = ref 0;
	fun NewArcNum () = (CPN'inc NextArcNum;!NextArcNum);

	(* --- elements on the stack/queue --- *)
	type QueueElement = ((CPN'OGnoderec ref) * 
			     (((int * Inst) * CPN'Binding) list));
 
	val ToProcess = ref ([]:QueueElement list);

	(* --- selection of next node/binding elements to process --- *)
	fun StdSelNextNode [] = raise NoSelPossible
	  | StdSelNextNode ((CPN'nextnode,[])::CPN'nextnodes) = 
	    (CPN'OGUtils.GetNodeCalcStat
	     (!CPN'nextnode):=FullProc;
	     ToProcess:= CPN'nextnodes;
	     StdSelNextNode CPN'nextnodes)
	  | StdSelNextNode ((CPN'nextnode,(CPN'be::CPN'bes))::CPN'nextnodes) = 
	    if (!CPN'OGBranching.Pred (CPN'OGUtils.NodeRecToStateRec (!CPN'nextnode)))
		then
		    (ToProcess:= ((CPN'nextnode,CPN'bes)::CPN'nextnodes);
		     (CPN'nextnode,CPN'be))
	    else
		(ToProcess := CPN'nextnodes;
		 (CPN'OGUtils.GetNodeCalcStat
		  (!CPN'nextnode):=PartProc;
		  StdSelNextNode CPN'nextnodes));
  
	val OnlyThisList=ref false;

	fun SelectNextNode () = StdSelNextNode (!ToProcess);
  
   	fun AssignList CPN'nl CPN'calcsucc
	    = ToProcess:= CPN'OGListUtils.mapfilter
	    (
	     ref(fn (CPN'n : CPN'OGnoderec ref)
		 => ((CPN'AvlTree.AvlLookup (CPN'OGNodes,mkst_Node (CPN'OGUtils.GetNodeNo (!CPN'n)));
		      true) handle CPN'AvlTree.ExcAvlLookup => false,
		     (CPN'n,CPN'calcsucc CPN'n))),
	     CPN'nl);

	(* --- assigns list of all non-full processed nodes to the list to be processed *)
	fun BuildList CPN'calcsucc 
	    = (CPN'AvlTree.AvlApp(
				  fn ref(CPN'OGnode{calcstat,no,...})
				  => if !calcstat<>FullProc
					 then
					     if !CPN'OGBranching.Pred (CPN'OGState (CPN'OGUtils.GetStateRec no))
						 then ToProcess:= 
						     (CPN'OGUtils.NodeToNodeRef no,
						      (CPN'calcsucc (CPN'OGUtils.NodeToNodeRef no)))::(!ToProcess)
					     else calcstat:= PartProc
				     else
					 (),
					 CPN'OGNodes);
	       ToProcess:= rev (!ToProcess))

	(* --- appends list to list of nodes to be processed --- *)
	fun AppendList CPN'nl 
	    = if !OnlyThisList
		  then ()
	      else 
		  (case (GetGenOrder ()) of 
		       OG'DepthFirst => (ToProcess:= CPN'nl::(!ToProcess))
		     | _ => (ToProcess:= (!ToProcess)^^[CPN'nl]));
		  
	fun InitList () = (ToProcess:=[])
	    
	fun StartSelOnlyOne() = OnlyThisList:=true;
	    
	fun RestoreStdSel() =  OnlyThisList:=false;

end(*struct*); 
