(*
 * Module:       Mechanisms for selecting the next node
 *
 * Description:  Design/CPN OG Tool
 *
 * Created by:   Lars Kristensen (kris@daimi.aau.dk)   
 *
 * Date:         29/09/95   
 *
 * Design/CPN (tm)  Copyright (c) Meta Software Corp 1989--1995
 *)

(* $Source: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OGNodeSel.sml,v $

$Log: OGNodeSel.sml,v $
Revision 1.1.4.2  2000/08/07 13:37:52  kris
Modification to reflect that branching options now works directly on state recs

Revision 1.1.4.1  1999/10/28 10:04:54  sts
Added InitList function.

Revision 1.1  1995/09/29 14:45:50  kris
Initial revision
 

*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OGNodeSel.sml,v 1.1.4.2 2000/08/07 13:37:52 kris Exp $";

(* this structure hold the mechanims for selecting the next node *)
(* for processing in the OG generation                           *)

structure CPN'OGNodeSel =
struct

exception StopCritSatisfied;
exception NoSelPossible;

val NextNodeNum = ref 0;
fun NewNodeNum () = (CPN'inc NextNodeNum;!NextNodeNum);

val NextArcNum = ref 0;
fun NewArcNum () = (CPN'inc NextArcNum;!NextArcNum);


local
 
 val ToProcess = ref ([]:Node list);

 fun StdSelNextNode():Node
   = if !ToProcess=[] 
      then
         raise NoSelPossible
      else
         let
          val nexthead::nexttail = (!ToProcess)
         in
          ToProcess:= nexttail;
          if !CPN'OGBranching.Pred (CPN'OGState (CPN'OGUtils.GetStateRec nexthead))
          then
           nexthead
          else (CPN'OGUtils.GetNodeCalcStat(CPN'OGUtils.NodeToNodeRec nexthead):=PartProc;
            StdSelNextNode())
         end(*let*); 
  
 val OnlyThisList=ref false;

in
  val SelectNextNode = StdSelNextNode;
  
  fun AssignList CPN'nl (* assigns list of nodes to be processed - omits illegal nodes *)
     = ToProcess:= CPN'OGListUtils.mapfilter(
                    ref(fn CPN'n
                          => ((CPN'AvlTree.AvlLookup (CPN'OGNodes,mkst_Node CPN'n);
                               true) handle CPN'AvlTree.ExcAvlLookup => false,
                              CPN'n)),
                    CPN'nl);
  
  fun AppendList CPN'nl (* appends list to list of nodes to be processed *)
     = if !OnlyThisList
       then ()
       else ToProcess:= (!ToProcess)^^(CPN'nl);
                    
  fun BuildList() (* assigns list of all non-full processed nodes to the list to be processed *)
     = (CPN'AvlTree.AvlApp(
        fn ref(CPN'OGnode{calcstat,no,...})
          => if !calcstat<>FullProc
             then
              if !CPN'OGBranching.Pred (CPN'OGState (CPN'OGUtils.GetStateRec no))
              then ToProcess:= no::(!ToProcess)
              else calcstat:= PartProc
             else
              (),
         CPN'OGNodes);
        ToProcess:= rev (!ToProcess))

  fun InitList () = (ToProcess:=[])

  fun StartSelOnlyOne() = OnlyThisList:=true;

  fun RestoreStdSel() =  OnlyThisList:=false;

end(*local*)

end(*struct*); 
