(*
 * Module:
 *
 * Description:  Design/CPN OG Tool
 *
 * Created by:   Lars Kristensen (kris@daimi.aau.dk)   
 *
 * Date:         29/09/95   
 *
 * Design/CPN (tm)  Copyright (c) Meta Software Corp 1989--1995
 *)

(* $Source: /users/cpn/designcpnR/src/current/oga/OGMLfiles/Attic/OGProfile.sml,v $

$Log: OGProfile.sml,v $
Revision 1.1.2.3  2000/08/06 15:07:59  kris
Changed the default prefix

Revision 1.1.2.2  2000/07/13 13:44:51  kris
Minor improvements

Revision 1.1.2.1  2000/05/12 19:34:06  kris
Initial revision


*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/oga/OGMLfiles/Attic/OGProfile.sml,v 1.1.2.3 2000/08/06 15:07:59 kris Exp $";


structure OGTimeProfile = 
struct

    val CPN'max = 10; (* --- maximum number of timers --- *)
    val CPN'prefix = ref "/tmp/";

    val CPN'timer = Timer.startCPUTimer ();
    val CPN'timers = Array.array (CPN'max,{start = (Time.fromReal 0.0), 
					   acc = (Time.fromReal 0.0),
					   file = "_dummy.dat",
					   count = 0,
				           obsfun = SOME (fn () => 1)});

    (* --- reset the time profiling --- *)
    fun Reset CPN'prefix' = 
	(CPN'prefix:=CPN'prefix';
	 (* --- main observation corresponding to state space generation --- *)
	 Array.update (CPN'timers,0,{start = (Time.fromReal 0.0), 
				     acc = (Time.fromReal 0.0),
				     file = "_time.dat",
				     count = 0,
				     obsfun = NONE});
	 (* --- observation of the nodes generated --- *)
	 Array.update (CPN'timers,1,{start = (Time.fromReal 0.0), 
				     acc = (Time.fromReal 0.0),
				     file = "_nodes.dat",
				     count = 0,
				     obsfun = SOME (fn () => (!CPN'OGNodeSel.NextNodeNum))});
	 (* --- observation of the nodes generated --- *)		       
	 Array.update (CPN'timers,2,{start = (Time.fromReal 0.0), 
				     acc = (Time.fromReal 0.0),
				     file = "_arcs.dat",
				     count = 0,
				     obsfun = SOME (fn () => (!CPN'OGNodeSel.NextArcNum))});

	 (* --- observation arc creation --- *)		       
	 Array.update (CPN'timers,3,{start = (Time.fromReal 0.0), 
				     acc = (Time.fromReal 0.0),
				     file = "_createarc.dat",
				     count = 0,
				     obsfun = NONE});
	 (* --- processing of binding elements --- *)		       
	 Array.update (CPN'timers,4,{start = (Time.fromReal 0.0), 
				     acc = (Time.fromReal 0.0),
				     file = "_processbind.dat",
				     count = 0,
				     obsfun = NONE});
	 (* --- enabling calculation --- *)		       
	 Array.update (CPN'timers,5,{start = (Time.fromReal 0.0), 
				     acc = (Time.fromReal 0.0),
				     file = "_enabling.dat",
				     count = 0,
				     obsfun = NONE});
	 (* --- execution in simulator --- *)		       
	 Array.update (CPN'timers,6,{start = (Time.fromReal 0.0), 
				     acc = (Time.fromReal 0.0),
				     file = "_simexecute.dat",
				     count = 0,
				     obsfun = NONE});
	 (* --- check whether node exists --- *)		       
	 Array.update (CPN'timers,7,{start = (Time.fromReal 0.0), 
				     acc = (Time.fromReal 0.0),
				     file = "_nodesexists.dat",
				     count = 0,
				     obsfun = NONE});
	 (* --- create new node --- *)		       
	 Array.update (CPN'timers,8,{start = (Time.fromReal 0.0), 
				     acc = (Time.fromReal 0.0),
				     file = "_newnodes.dat",
				     count = 0,
				     obsfun = NONE});
	 (* --- create new node --- *)		       
	 Array.update (CPN'timers,9,{start = (Time.fromReal 0.0), 
				     acc = (Time.fromReal 0.0),
				     file = "_existingnodes.dat",
				     count = 0,
				     obsfun = NONE})
	 );
	

    (* --- set the ith statisticcal variable --- *)
    fun Set CPN'i CPN'obsfun CPN'filename =
	Array.update(CPN'timers,CPN'i,{start = (Time.fromReal 0.0),
				       acc = (Time.fromReal 0.0),
				       file = CPN'filename,
				       count = 0,
				       obsfun = CPN'obsfun});
	
    (* --- start the ith position --- *)
    fun BeginObs CPN'i = 
	let
	    val CPN'start = (#usr (Timer.checkCPUTimer CPN'timer))
	in
	    (case (Array.sub(CPN'timers,CPN'i)) of 
		 {acc = CPN'acc, file = CPN'file,obsfun = CPN'obsfun,count=CPN'count,...} =>
		     Array.update(CPN'timers,CPN'i,{start = CPN'start,
						    acc = CPN'acc,
						    count = CPN'count+1,
						    file = CPN'file,
						    obsfun = CPN'obsfun}))
	end;


    (* --- end the ith observation --- *)
    fun EndObs CPN'i = 
	let
	    val CPN'end = (#usr (Timer.checkCPUTimer CPN'timer))
	in
	   (case (Array.sub(CPN'timers,CPN'i)) of
		{start = CPN'start, file = CPN'file,acc = CPN'acc,
		 obsfun=CPN'obsfun, count = CPN'count} =>
		    Array.update(CPN'timers,CPN'i,{start = CPN'start, 
						   acc = Time.+(CPN'acc,Time.-(CPN'end,CPN'start)),
						   file = CPN'file,
						   count = CPN'count,
						   obsfun = CPN'obsfun}))

	end;

    (* --- dump the observed values --- *)
    fun DumpTimeObs () = 
	Array.app (fn {start=CPN'start,file=CPN'filename,acc=CPN'acc,
		       obsfun=CPN'obsfun,count=CPN'count} =>
		   let
		       val CPN'file = TextIO.openAppend((!CPN'prefix)^CPN'filename);
		       val CPN'usr = Time.-((#usr (Timer.checkCPUTimer CPN'timer)),
					    (#start (Array.sub(CPN'timers,0))))

		       val _ = (case CPN'obsfun of 
				    SOME (CPN'obsfun') =>
					TextIO.output(CPN'file,
						      (Time.toString CPN'usr)^
						      " "^
						      (Int.toString (CPN'obsfun' ()))^
						      " "^(Int.toString CPN'count)^"\n")
				  | NONE =>  TextIO.output(CPN'file,
							   (Time.toString CPN'usr)^
							   " "^
							   (Time.toString CPN'acc)^
							   " "^
							   (Int.toString CPN'count)^
							   " "^
							   (Real.toString 
							    (Real./(Time.toReal CPN'acc,Real.fromInt CPN'count)))
							   ^"\n"))
		       val _ = TextIO.closeOut(CPN'file)
		   in
		       ()
		   end) CPN'timers;

    fun Generation CPN'prefix CPN'compute = 
	(OGSet.InspectionOptions{
				 Frequency = 20,
				 Action = (fn () => 
					   (DSUI_SetStatusBarMessage 
					    ("Nodes: "^
					     (Int.toString (!CPN'OGNodeSel.NextNodeNum))^
					     "  Arcs : "^
					     (Int.toString (!CPN'OGNodeSel.NextArcNum)));
					    DumpTimeObs ()))};

	 Reset CPN'prefix;
	 BeginObs 0;
	 CPN'compute ();
	 EndObs 0;
	 DumpTimeObs () 
	 handle CPN'xx => 
	     (DumpTimeObs ();
	      raise CPN'xx)
	     )
end;
