(*
 * Module:       Main structure for generating a Report on the CP-net
 *
 * Description:  Design/CPN OG Tool
 *
 * Created by:   Lars Kristensen (kris@daimi.aau.dk)   
 *
 * Date:         29/09/95   
 *
 * Design/CPN (tm)  Copyright (c) Meta Software Corp 1989--1995
 *)

(* $Source: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OGSaveReport.sml,v $

$Log: OGSaveReport.sml,v $
Revision 1.1  1995/09/29 14:48:47  kris
Initial revision
 

*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OGSaveReport.sml,v 1.1 1995/09/29 14:48:47 kris Exp $";

structure CPN'OGReport =
struct

    type UserNodesType =((Node->bool) * string) list;
    type UserArcsType  = ((Arc->bool) * string) list;
    type UserSccsType  = ((Scc->bool) * string) list;    
  
    val CPN'UserNodes = ref ([]:UserNodesType);
    val CPN'UserArcs = ref ([]:UserArcsType);
    val CPN'UserSccs = ref ([]:UserSccsType);

    fun SetUserFunctions(CPN'NodeFuncs,CPN'ArcFuncs,CPN'SccsFuncs) =
	(CPN'UserNodes := CPN'NodeFuncs;
	 CPN'UserArcs:= CPN'ArcFuncs;
	 CPN'UserSccs:= CPN'SccsFuncs;
	 ());

    fun Report () = 
	(Stats.GetStats())^
	(SearchAllNodes(fn _ => true,Bound.LowerInteger o Bound.UpperInteger,0,op +);
	 (Bound.Result())^
	 (Bound.MSBounds()))^
	(Home.HomeMarkings())^ 
	(Deadlock.ListDeadlocks())^
	(Liveness.LiveTrans())^
	(if (length(!CPN'UserNodes) > 0) 
	     then CPN'OGCustom.ResultNodes((!CPN'UserNodes))
	 else
	     "")^
        (if (length(!CPN'UserArcs) > 0)
	     then CPN'OGCustom.ResultArcs(!CPN'UserArcs)
	 else
	     "")^
        (if (length(!CPN'UserSccs) > 0)
	     then CPN'OGCustom.ResultSccs(!CPN'UserSccs)
	 else
	     "");
		  
    fun DumpReport (CPN'path) =
	let val OUT = open_out(CPN'path)
	in
	    (output(OUT,Report());
	     close_out(OUT))
	end


    fun SaveReport () = if (!IsSCCGen)
			    then
				(DumpReport(DSUI_GetFileName{path = "",prompt = "Save the report as"})
				 handle _  => (DSUI_UserAckMessage("Internal Error");
					       ()))
			else
			    DSUI_UserAckMessage("Scc Graph not calculated.");
			    

end (* struct CPN'OGReportGen *);
