(*
 * Module:       Functions for settings attributes and options
 *
 * Description:  Design/CPN OG Tool
 *
 * Created by:   Lars Kristensen (kris@daimi.aau.dk)   
 *
 * Date:         29/09/95   
 *
 * Design/CPN (tm)  Copyright (c) Meta Software Corp 1989--1995
 *)

(* $Source: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OGSetAttrOptions.sml,v $

$Log: OGSetAttrOptions.sml,v $
Revision 1.3  1996/08/27 12:09:43  cpn
Mac Port v3.0.2. EML does not like one member records.

Revision 1.2  1996/02/04 14:02:46  kris
Modification according to new schema for strinreps

 Revision 1.1  1995/09/29  14:51:50  kris
 Initial revision
 

*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OGSetAttrOptions.sml,v 1.3 1996/08/27 12:09:43 cpn Exp $";

structure OGSet = struct

    val StandardInspectionReport = ref CPN'OGInspection.StandardReport;

    fun NodeAttributes {Size = nodesize,
			Graphics = graphics,
			Text = textattr}
	= (CPN'OGDrawUtils.nodesizeref := nodesize;
	   CPN'OGDrawUtils.nodeattrref := graphics;
	   CPN'OGDrawUtils.nodetextref := textattr);


    fun ArcAttributes{ Graphics = graphics }
	= CPN'OGDrawUtils.arcattrref := graphics
	


    fun NodeDescriptorAttributes {Size = nodedescrsize,
				  Graphics = graphics,
				  Text = textattr }
	= (CPN'OGDrawUtils.nodedescrsizeref := nodedescrsize;
	   CPN'OGDrawUtils.nodedescrattrref := graphics;
	   CPN'OGDrawUtils.nodedescrtextref := textattr);
	
    fun ArcDescriptorAttributes {Size = arcdescrsize,
				 Graphics = graphics,
				 Text = textattr }
	= (CPN'OGDrawUtils.arcdescrsizeref := arcdescrsize;
	   CPN'OGDrawUtils.arcdescrattrref := graphics;
	   CPN'OGDrawUtils.arcdescrtextref := textattr);
	
    fun NodeDescriptorOptions (userNodeFilter) =
	st_NodeDescrRef := userNodeFilter;
	
    fun ArcDescriptorOptions (userArcFilter) = 
	st_ArcDescrRef := userArcFilter;
	
    fun SuccPredOptions{FirstSucc = firstsucc,
			NextSucc = nextsucc,
			MaxSucc = maxsucc,
			FirstPred = firstpred,
			NextPred = nextpred,
			MaxPred = maxpred}
	= (CPN'OGDrawUtils.nodesuccoffsetref := firstsucc;
	   CPN'OGDrawUtils.succsuccoffsetref := nextsucc;
	   CPN'OGDrawUtils.max_fanoutref := maxsucc;
	   CPN'OGDrawUtils.nodepredoffsetref := firstpred;
	   CPN'OGDrawUtils.predpredoffsetref := nextpred;
	   CPN'OGDrawUtils.max_faninref := maxpred);
	
    fun StopOptions {Nodes = nodes,
		     Arcs = arcs,
		     Secs = secs,
		     Predicate = userpredfun }
	= CPN'OGStopCrit.SetParams {NodeLimit = nodes,
				    ArcLimit = arcs,
				    CompTimeLimit = secs,
				    ProcNodePred = userpredfun};
	
    fun BranchingOptions {TransInsts = transinsts,
			  Bindings = bindings,
			  Predicate = userpredicate }
	= CPN'OGBranching.SetParams {Pred = userpredicate,
				   TransInstLimit = transinsts,
				   BindingLimit = bindings};
	
    fun InspectionOptions {Frequency = freq,
			   Action = useractionfun}
	= CPN'OGInspection.SetParams {Frequency = freq,
				      Action = useractionfun};
	

    val stringrepnoderef = ref (fn (CPN'n:string) => CPN'n);

    val stringreparcref = ref (fn (CPN'a, CPN's, CPN'd) => 
			       (CPN'a)^":"^(CPN's)^"->"^(CPN'd))

    val stringreppiref = ref (fn (CPN'pa,CPN'pl,CPN'in) =>
			      (CPN'pa)^"'"^(CPN'pl)^" "^(CPN'in))

    val stringreptiref = ref (fn (CPN'pa, CPN'tr, CPN'in) =>
			      (CPN'pa)^"'"^(CPN'tr)^" "^(CPN'in))

    val stringrepberef = ref (fn (CPN'tr, CPN'be) =>
			      CPN'tr^": "^CPN'be)

    val stringrepmarkref = ref (fn (CPN'pl, CPN'ma) =>
				CPN'pl^": "^CPN'ma)
	
    fun StringRepOptions'Node (userfilterfun) = stringrepnoderef := userfilterfun;
	
    fun StringRepOptions'Arc (userfilterfun) = stringreparcref := userfilterfun;
	
    fun StringRepOptions'PI (userfilterfun) = stringreppiref := userfilterfun;
	
    fun StringRepOptions'TI (userfilterfun) = stringreptiref := userfilterfun;
	
    fun StringRepOptions'BE (userfilterfun) = stringrepberef := userfilterfun;
	
    fun StringRepOptions'Mark (userfilterfun) = stringrepmarkref := userfilterfun;


end (*struct*)


fun st_Node (CPN'n:Node) = (!OGSet.stringrepnoderef)(makestring(CPN'n));	      

fun st_Arc (CPN'a:Arc)
   = let
      val CPN'ar = CPN'OGUtils.ArcToArcRec CPN'a
     in
	 (!OGSet.stringreparcref)((makestring(CPN'a)),
				  (st_Node (CPN'OGUtils.GetNodeNo(!(CPN'OGUtils.GetArcSrc CPN'ar)))),
				  (st_Node (CPN'OGUtils.GetNodeNo(!(CPN'OGUtils.GetArcDest CPN'ar)))))
     end(*let*);

(*
st_Mark.format :=
  (fn (_,"empty")
     => ""
    | (_,"tempty")
     => ""
    | (CPN'pli_st,CPN'mark_st)
     => CPN'pli_st^" = "^CPN'mark_st^"\013");
*)
