(*
 * Module:       Storage for stop/limitation criteria
 *
 * Description:  Design/CPN OG Tool
 *
 * Created by:   Lars Kristensen (kris@daimi.aau.dk)   
 *
 * Date:         29/09/95   
 *
 * Design/CPN (tm)  Copyright (c) Meta Software Corp 1989--1995
 *)

(* $Source: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OGStopLimitCrit.sml,v $

$Log: OGStopLimitCrit.sml,v $
Revision 1.1.4.1  2000/08/06 15:04:29  kris
Branching function now works directly on staterecs

Revision 1.1  1995/09/29 14:55:40  kris
Initial revision
 

*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OGStopLimitCrit.sml,v 1.1.4.1 2000/08/06 15:04:29 kris Exp $";

(* this file contains structures for storing: *)
(* 1) the stop criteria and a test function   *)
(* 2) the limitation criteria                 *)


structure CPN'OGStopCrit = struct
val NodeLimit=ref 0;         
val ArcLimit=ref 0;
val CompTimeLimit=ref 0;               
val ProcNodePred = ref (fn(CPN'n:CPN'OGrec)=>false);
local
 val NodeCount=ref 0;
 val ArcCount=ref 0;
 val NextStopTime=ref 0;
 val TotalTime=ref 0;
 val LastTotalCalc=ref 0;
 val StartTime=ref 0;
 exception ProcPredTrue;
 in

 fun InitCounts()
    =let
      val curtod = tod()
     in
      NodeCount:=0;
      ArcCount:=0;
      NextStopTime:=
       (if !CompTimeLimit=0 then !CPN'maxint else !CompTimeLimit+curtod);
      LastTotalCalc:=curtod;
      StartTime:=curtod
     end(*let*);

 fun IncrAndTest(CPN'newnodes,CPN'newarcs,CPN'procnl,CPN'curtod)
    =((*update counters*)
      NodeCount:= !NodeCount + CPN'newnodes;
      ArcCount:= !ArcCount + CPN'newarcs;
      
      TotalTime:= !TotalTime + (CPN'curtod - !LastTotalCalc);
      LastTotalCalc:=CPN'curtod;
      
      (*test counters and return result*)
      ((map (fn CPN'n
              => if !ProcNodePred CPN'n then raise ProcPredTrue else ())
           CPN'procnl;
        false)
       handle ProcPredTrue => true) orelse
      ((!NodeLimit<>0)andalso(!NodeLimit<= !NodeCount))orelse
      ((!ArcLimit<>0)andalso(!ArcLimit<= !ArcCount))orelse
      (!NextStopTime<=tod()));
  

 fun StandardReport()
    = let
       val curtod=tod();
       val nodecountstr=makestring(!NodeCount)^
                        (if !NodeLimit<>0
                         then "/"^(makestring(!NodeLimit))
                         else "");
       val totalnodecountstr=makestring(CPN'AvlTree.AvlNo CPN'OGNodes);
       val arccountstr=makestring(!ArcCount)^
                        (if !ArcLimit<>0
                         then "/"^(makestring(!ArcLimit))
                         else "");
       val totalarccountstr=makestring(CPN'AvlTree.AvlNo CPN'OGArcs);
       val thistimestr=makestring(curtod- !StartTime)^
                        (if !CompTimeLimit<>0
                         then "/"^(makestring(!CompTimeLimit))
                          else "");
       val totaltimestr=makestring(!TotalTime) 
      in
       DSUI_SetStatusBarMessage("OG Generation:"^
               "#Nodes ="^nodecountstr^"/"^
                                totalnodecountstr^" "^
               "#Arcs  ="^arccountstr^"/"^
                                totalarccountstr^" "^
               "#Secs  ="^thistimestr^"/"^
                                totaltimestr)
      end(*let*);

 fun GetTotalTime()
    = makestring(!TotalTime);

 fun GetTotalTimeSecs()
    = !TotalTime;
end;

fun SetParams{NodeLimit=CPN'nl,ArcLimit=CPN'al,
              CompTimeLimit=CPN'ctl,ProcNodePred=CPN'pnp}
   = (NodeLimit:=CPN'nl;
      ArcLimit:=CPN'al;
      CompTimeLimit:=CPN'ctl;
      ProcNodePred:= CPN'pnp);

end(*struct*);

structure CPN'OGBranching = struct
 val TransInstLimit=ref 0;
 val BindingLimit=ref 0;
 val Pred  = ref (fn(CPN'n:CPN'OGrec)=>true);
 fun SetParams{Pred=CPN'pf,TransInstLimit=CPN'til,BindingLimit=CPN'bl}
    = (TransInstLimit:= CPN'til;
       BindingLimit:= CPN'bl;
       Pred:= CPN'pf);

end(*struct*);

structure CPN'OGInspection = struct
 val Freq=ref 0;
 val StandardReport = CPN'OGStopCrit.StandardReport;
 val Action=ref StandardReport;
 local
  val NextInspectTime=ref 0
 in
  fun InitCounts()
     = NextInspectTime:=
       (if !Freq=0 then !CPN'maxint else !Freq+tod());
 
  fun Test CPN'curtod
    = if !NextInspectTime<=CPN'curtod
      then (!Action();
            NextInspectTime:= !Freq+CPN'curtod)
      else ()
 end(*local*);
 fun SetParams{Frequency=CPN'fs,Action=CPN'a}
    = (Freq:=CPN'fs; Action:=CPN'a);

end(*struct*);

fun NoOfSecs () = CPN'OGStopCrit.GetTotalTimeSecs ();

val EntireGraph = [0];

val NoLimit = 0;

CPN'OGStopCrit.SetParams{
     NodeLimit= 0,
     ArcLimit= 0,
     CompTimeLimit= 300,
     ProcNodePred= fn _ => false};

CPN'OGInspection.SetParams{
     Frequency= 60,
     Action= CPN'OGInspection.StandardReport};

CPN'OGBranching.SetParams{
     Pred= fn _ => true,
     TransInstLimit= 0,
     BindingLimit= 0};
