(*
 * Module:       Generation of string representation functions
 *
 * Description:  Design/CPN OG Tool
 *
 * Created by:   Lars Kristensen (kris@daimi.aau.dk)   
 *
 * Date:         29/09/95   
 *
 * Design/CPN (tm)  Copyright (c) Meta Software Corp 1989--1995
 *)

(* $Source: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OGStringRepGen.sml,v $

$Log: OGStringRepGen.sml,v $
Revision 1.1  1995/09/29 14:57:29  kris
Initial revision
 

*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OGStringRepGen.sml,v 1.1 1995/09/29 14:57:29 kris Exp $";


local
    
    open CPN'CodeGenUtils;
    
    fun newlgen_stPI()
	=( patterncount:=0;
	  partfuncount:= 0;
	  code_temp:="";
	  code_storage:= ["fun st_PI0 _ = raise IllegalId;\n"];
	  
	  map
          (fn {pllist = pllist, pgname = pgname, instlist = instlist,...}
	   => CPN'AvlTree.AvlApp
	       (fn {plname="",...} => []
		 | {plname=plname, plmlno=plmlno,...}
		   => map
		       (fn {instno=instno,...} (* pattern match on specific ins
						tno *)
			=> let               (* in order to detect illegal id
					      s   *)
			       val CPN'pi=pgname^"'"^plname^" "^instno
			   in
			       newpattern("st_PI","(PI."^ CPN'pi^")= (!OGSet.stringreppiref)(\""^pgname^"\",\""^plname^"\",\""^instno^"\")")
			   end(*let*))
		       instlist,
		       pllist))
          (!CPN'NetCapture.Net);
	  
	  newpattern("st_PI","(PI.unnamedPI)=\"*** PI without a name ***\"");
	  
	  if !patterncount>0
	      then (* finish the last function in code_temp *)
		  (code_storage:= !code_storage^^
		   [!code_temp^
		    "|st_PI"^(makestring(!partfuncount+1))^" CPN'x=st_PI"^
		    (makestring(!partfuncount))^" CPN'x;\n"];
		   code_temp:="")
	  else
	      ();
	      
	      code_storage:= !code_storage^^
	      ["val st_PI =st_PI"^
	       (makestring(!partfuncount+(if !patterncount=0 then 0 else 1)))^";\n",
	       "fun mkst_PI CPN'pi=\"PI.\"^(st_PI CPN'pi);\n"]);
	

  fun lgen_stTI()
    =( patterncount:=0;
       partfuncount:=0;
       code_temp:="";
       code_storage:= ["fun st_TI0 _ = raise IllegalId;\n"];

       map
          (fn {trlist = trlist, pgname = pgname, instlist = instlist,...}
            => map
                 (fn {trname = "", ...} => []
                   | {trname = trname, ...}
                    => map
                        (fn {instno=instno,...} (* pattern match on specific ins
tno *)
                           => let               (* in order to detect illegal id
s   *)
                                val CPN'ti=pgname^"'"^trname^" "^instno
                              in
			       newpattern("st_TI","(TI."^ CPN'ti^")= (!OGSet.stringreptiref)(\""^pgname^"\",\""^trname^"\",\""^instno^"\")")
                              end(*let*))
                        instlist)
                 trlist)
          (!CPN'NetCapture.Net);
          
       newpattern("st_TI","(TI.unnamedTI)=\"*** TI without a name ***\"");
       
       if !patterncount>0
       then (* finish the last function in code_temp *)
        (code_storage:= !code_storage^^
                        [!code_temp^
                         "|st_TI"^(makestring(!partfuncount+1))^" CPN'x=st_TI"^
                        (makestring(!partfuncount))^" CPN'x;\n"];
         code_temp:="")
       else
        ();
        
       code_storage:= !code_storage^^
         ["val st_TI=st_TI"^
         (makestring(!partfuncount+(if !patterncount=0 then 0 else 1)))^";\n",
          "fun mkst_TI CPN'ti=\"TI.\"^(st_TI CPN'ti);\n"]);

 fun lgen_st_BE()
    =( patterncount:=0;
       partfuncount:=0;
       code_temp:="";
       code_storage:= ["fun CPN'st_BE0 _ _ = \"*** BE without a name ***\";\n"];

       map
          (fn {trlist = trlist, pgname = pgname,...}
            => map
                 (fn {trname = "", ...} => ()
                   | {trname = trname, trmlno = trmlno, ...}
                    => newpattern_2param("CPN'st_BE",
                                    "CPN'mkst (Bind."^pgname^"'"^trname^
                                    "(CPN'instno,CPN'b))=\n"^
					 "(if CPN'mkst then "^"\"Bind.\"^"^"(CPN'OGBindGen.skiptransno(st_TI(TI."^pgname^"'"^trname^" CPN'instno)))^\n"^"\"(\"^(makestring CPN'instno)^"^"\",\"^(CPN'st_binding'"^(CPN'OGBTconvGen.findmlnoforbinding trmlno)^" CPN'b)^\")\""^
                                    " else !OGSet.stringrepberef(\""^pgname^"\",\""^trname^"\",makestring(CPN'instno),(CPN'st_binding'"^
                                    (CPN'OGBTconvGen.findmlnoforbinding trmlno)^
                                    " CPN'b)))"))
                 trlist)
          (!CPN'NetCapture.Net);
                 
       if !patterncount>0
       then (* finish the last function in code_temp *)
        (code_storage:= !code_storage^^
                        [!code_temp^
                         "|CPN'st_BE"^(makestring(!partfuncount+1))^" CPN'm CPN'x=CPN'st_BE"^
                        (makestring(!partfuncount))^" CPN'm CPN'x;\n"];
         code_temp:="")
       else
        ();
        
       code_storage:= !code_storage^^
         ["val mkst_BE =CPN'st_BE"^
         (makestring(!partfuncount+(if !patterncount=0 then 0 else 1)))^" true;\n",
         "val st_BE =CPN'st_BE"^
         (makestring(!partfuncount+(if !patterncount=0 then 0 else 1)))^" false;"]);      

in
  fun Mark_gen_st()
    = (code_temp:="structure st_Mark=struct\n"^
                  "val format=ref(fn(\"\",\"\")=>\"\");\n";
                         
       map
         (fn {pllist = pllist, pgname = pgname, instlist = CPN'hdinst::_,...}
            => CPN'AvlTree.AvlApp
                 (fn {plname="",...} => ()
                   | {plname=plname, pldbid=pldbid,...}
		            => let
		                 val CPN'p=pgname^"'"^plname;
		                 val CPN'plikey=Cpndb.makestringdbid(pldbid)^" "^(Cpndb.makestringdbid(#instdbid CPN'hdinst));
		                 val CPN'colset= (#colset(CPN'AvlTree.AvlLookup(CPN'OGIdsGen.PlaceInsts,CPN'plikey)));
		                 val CPN'csistimed= CPN'env.IsTimed CPN'colset;
		                 val CPN'primecol= CPN'env.GetPrimename CPN'colset
		                in
		                 code_temp:= !code_temp^
		                              "fun "^CPN'p^"(CPN'i:Inst)CPN'n=\n"^"!OGSet.stringrepmarkref(\""^pgname^"\",\""^plname^"\",makestring(CPN'i),mkst_"^(if CPN'csistimed then "t"
 else "")^"ms'"^CPN'primecol^
                                              "(Mark."^CPN'p^" CPN'i CPN'n))"
		                 end(*let*),
		                 pllist))
          (!CPN'NetCapture.Net);       
        
       [!code_temp^"end(*struct*);\n"])       

    fun gen_stTI () =
	(lgen_stTI();
	 !code_storage);

    fun gen_stPI () =
	(newlgen_stPI();
	 !code_storage)

    fun gen_st_BE() =
	(lgen_st_BE();
	 !code_storage);

end;
