(*
 * Module:       Structure for recording the enabling data for timed transitions
 *
 * Description:  Design/CPN OG Tool
 *
 * Created by:   Lars Kristensen (kris@daimi.aau.dk)   
 *
 * Date:         29/09/95   
 *
 * Design/CPN (tm)  Copyright (c) Meta Software Corp 1989--1995
 *)

(* $Source: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OGTimeEnabDataGen.sml,v $

$Log: OGTimeEnabDataGen.sml,v $
Revision 1.1.4.2  1999/10/07 10:25:10  mailund
renamed i,j,t -> CPN'i,CPN'j,CPN't

Revision 1.1.4.1  1999/01/14 16:46:34  mailund
time port

Revision 1.1  1995/09/29 14:58:26  kris
Initial revision
 

*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OGTimeEnabDataGen.sml,v 1.1.4.2 1999/10/07 10:25:10 mailund Exp $";

usestring
["structure CPN'OGTimeEnabData=struct\n"^

"datatype limitmemory=CPN'not_reached|CPN'reached_at of"^

   (if (!CPN'env.Timedef)<>CPN'env.timenotdef then " TIME" else " IntInf.int")^";\n"^

"val timecands = ref ([]:(CPN'TransInst list));\n"^
"val bindcands = ref ([]:((CPN'TransInst*CPN'Binding) list));\n"^
"val count = ref 0;\n"^

 (if (!CPN'env.Timedef)<>CPN'env.timenotdef 
      then "val curr_min : TIME option ref = ref NONE;\n"
  else "val curr_min : IntInf.int option ref = ref NONE;\n")^

"val trans_inst_limit_occ_at = ref CPN'not_reached;\n"^
"val bind_limit_occ=ref false;\n"^

"fun any_limit()=((!trans_inst_limit_occ_at<>CPN'not_reached)andalso\n"^
                      "(let val CPN'reached_at CPN'limittime= !trans_inst_limit_occ_at\n"^

                       "in (case (!curr_min) of\n"^
		       "         SOME CPN't => (CPN'limittime=CPN't)\n"^
		       "       | NONE => false) end))\n"^

                  "orelse(!bind_limit_occ);\n"^

"fun init() = (timecands:=[];bindcands:=[];count:=0;bind_limit_occ:=false;\n"^

              "curr_min:=NONE;"^

	      "trans_inst_limit_occ_at:=CPN'not_reached);\n"^


 "fun option'lt (CPN'i , NONE) = true\n"^
 "  | option'lt (CPN'i, SOME CPN'j) = IntInf.< (CPN'i, CPN'j)\n"^
 "fun option'lte (CPN'i , NONE) = true\n"^
 "  | option'lte (CPN'i, SOME CPN'j) = IntInf.<= (CPN'i, CPN'j)\n"^


"fun add_timecand(CPN'cand,CPN't)\n"^

  "= (if (not (option'lte (CPN't,(!curr_min))))\n"^

    "then\n"^ (*ignore this candidate with high enab time *)
      "()\n"^
    "else\n"^

       "if option'lt (CPN't,(!curr_min))\n"^

       "then (*new candidate with strictly lower time enabling *)\n"^
       "(timecands:=[CPN'cand]; count:=1;\n"^

       "curr_min:=SOME (CPN't);\n"^

       "trans_inst_limit_occ_at:=CPN'not_reached)\n"^
       "else\n"^(*one more with same enabling time - check if max is reached *)
          "if (!CPN'OGBranching.TransInstLimit<>0)andalso\n"^
             "(!count = (!CPN'OGBranching.TransInstLimit))\n"^
          "then trans_inst_limit_occ_at:=CPN'reached_at CPN't\n"^
          "else (timecands:=CPN'cand::(!timecands);CPN'inc count));\n"^

"local\n"^
 "val no_bindings = ref 0;\n"^
 "exception BindStop\n"^
"in\n"^
 "fun add_BE(_,[])=()\n"^
 "|add_BE(CPN'cand,CPN'bl)\n"^
  "=(no_bindings:=(if !CPN'OGBranching.BindingLimit=0\n"^
                 "then ~1\n"^
                 "else !CPN'OGBranching.BindingLimit);\n"^
    "map(fn CPN'b\n"^
         "=> if !no_bindings=0\n"^
            "then (bind_limit_occ:=true;raise BindStop)\n"^
            "else\n"^
                 "(bindcands:=(CPN'cand,CPN'b)::(!bindcands);\n"^
                  "CPN'dec no_bindings))\n"^
           "CPN'bl\n"^
       "handle BindStop=>[];())\n"^       
"end(*local*);\n"^


"fun get_cands() = (!bindcands)\n"^

"end(*struct*);\n",

if (!CPN'env.Timedef)<>CPN'env.timenotdef 
    then "();" 
else "fun time()=IntInf.fromInt (0);"];

