(*
 * Module:
 *
 * Description:  Design/CPN OG Tool
 *
 * Created by:   Lars Kristensen (kris@daimi.aau.dk)   
 *
 * Date:         29/09/95   
 *
 * Design/CPN (tm)  Copyright (c) Meta Software Corp 1989--1995
 *)

(* $Source: /users/cpn/designcpnR/src/current/oga/OGMLfiles/Attic/OGTimeSweep.sml,v $

$Log: OGTimeSweep.sml,v $
Revision 1.1.2.6  2000/08/06 15:07:35  kris
Time efficiency improvements

Revision 1.1.2.5  2000/08/05 13:39:35  kris
Implemented support for use in batch state space generation

Revision 1.1.2.4  2000/08/03 12:27:27  kris
Removed the use of max time

Revision 1.1.2.3  2000/07/30 16:02:39  kris
Reimplement to support new timed generation with internal references

Revision 1.1.2.2  2000/07/30 12:23:46  kris
Switch to use of special version of DeleteOccGraph which does not insert current similator state. Modified the data dumped into the log files

Revision 1.1.2.1  2000/07/20 21:36:29  kris
Initial Revision


*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/oga/OGMLfiles/Attic/OGTimeSweep.sml,v 1.1.2.6 2000/08/06 15:07:35 kris Exp $";


(* --- delete the current occurrence graph --- *)
structure CPN'OGGlobTablesReset =
struct
    
local

open CPN'CodeGenUtils;
  
 fun gen_recsearchtree(recname,_)
    = code_storage:= ("CPN'RecSearchTree.Reset(CPN'"^recname^"'recs);\n")
                       ::(!code_storage)
in

fun gen () =
  
 (
 code_storage:=[];
  
 CPN'AvlTree.AvlApp(
  fn (csdesc as {name,...})
    => code_storage:= ("CPN'"^name^"'mss.Reset ();\n")
                       ::(!code_storage),
  CPN'env.colortabel);

 CPN'AvlTree.AvlAppKey(gen_recsearchtree, CPN'OGIdsGen.DataRecFields);

  ["fun SweepDeleteOccGraph () = (\n"]^^
 (!code_storage)^^[
 
      if (!CPN'env.Timedef)<>CPN'env.timenotdef
      then ("CPN'TimeListsTree.Reset CPN'TimeListTree;\n")

      else "();\n",
 
	 "CPN'RecSearchTree.Reset CPN'State'recs;\n",
	 "CPN'AvlTree.AvlReset CPN'OGArcs;\n",
	 "CPN'AvlTree.AvlReset CPN'OGNodes;\n",
	 "CPN'BindSearchTree.Reset CPN'Binds;\n",
	 "CPN'OGNodeSel.NextNodeNum:=0;\n",
	 "CPN'OGNodeSel.NextArcNum:=0;\n",
	 "CPN'OGNodeSel.InitList();\n",
(*	 "OGSimStatetoOG();\n",*)
		   "());\n"]

  )

end(*local*)

end(*struct*);

(* --- generate the delete function --- *)
usestring [(implode (CPN'OGGlobTablesReset.gen ()))];

(* --- main structure for making a simple implementation of the time sweep algorithm --- *)
structure OGTimeSweep = 
struct

    (* --- insert a state rec in the data structure --- *)
    exception InsertExn;

    fun Insert (ref staterec) = 
	let
	    val newsrcnoderef = 
		ref (
		     CPN'OGnode {state = ref staterec,
				 no = 0,
				 succlist = ref [],
				 predlist = ref [],
				 calcstat = ref UnProc});
		
	    (* --- set the simulator state to the canonical form ---
	     *)
	    val _ = CPN'OGToSimData.copyStateRec (!newsrcnoderef); 

	    (* --- store new marking in the trees --- *)
	    val staterec' = CPN'store_state(newsrcnoderef)

	    (* --- store the resulting state in the state space --- *)
	    val (insertstaterec as
		 ref(CPN'OGState(innerrec as {owner=ref(noderef),...})),wasthere)
		= CPN'RecSearchTree.FastInsert
		(CPN'State'recs, staterec', !CPN'OEEncode.CPN'EncodeMarkFun staterec');

	in
	    if not wasthere (* we have a new state and hence we create new node *)
		then
		    let
			
			val newnodenum = CPN'OGNodeSel.NewNodeNum();
			    
			val destnoderef (* the bare record for the new node *)
			    = ref( CPN'OGnode {state=insertstaterec,
					       no=newnodenum,
					       succlist=ref [],
					       predlist=ref [], (* will be assigned below *)
					       calcstat=ref UnProc})
			    
		    in
			#owner innerrec:=destnoderef;
			
			CPN'AvlTree.AvlInsert CPN'OGNodes(mkst_Node newnodenum,destnoderef);
			destnoderef (*newnodenum*)
		    end
	    else
		(DSUI_UserAckMessage("Node :"^(makestring (CPN'OGUtils.GetNodeNo (!noderef))));
		 raise InsertExn)
		(* --- we throw away the arcs at this point --- *)
	end;

    fun CreationTime (ref (CPN'OGState {creationtime = 
					creationtime,...})) = 
        creationtime;

    fun CreationTime' (ref (CPN'OGnode {state = ref (CPN'OGState {creationtime = 
								  creationtime,...}),...})) = 
        creationtime;

(*    fun CreationTime CPN'n = #creationtime(CPN'OGUtils.GetStateRec CPN'n);*)

    fun CopyAndRestore () = 
	let
	    (* --- find the minumum creation time in the priority queue --- *)
	    val _ = DSUI_SetStatusBarMessage ("Finding Minimum ...");
	    val (CPN'mincreation,CPN'n) = TimePQ.min(CPN'OGNodeSel.pq);
		
	    val _ = DSUI_SetStatusBarMessage ("Saving States ...");

	    (* --- empty the priority queue 
                   - we recreate it since we renumber states 
                   - and since currently we can only to delete everything --- *)

	    (* --- place holder for the state recs we want to keep --- *)
	    (*val CPN'savestate = ref ([] : CPN'OGrec ref list);*)

	    (* --- save all the states we still need --- *)
	    val CPN'savestate = (CPN'AvlTree.AvlFold 
			     (fn (ref(CPN'OGnode{calcstat,no,state,...}),CPN'savestate)
			      => (if (IntInf.>=(CreationTime state,CPN'mincreation))
				      then (state::CPN'savestate)
				  else CPN'savestate),[],CPN'OGNodes))

	    (* --- delete everything in the search trees and priority queue --- *)
	    val _ = DSUI_SetStatusBarMessage ("Deleting States ...");

	    (* --- special version of delete which does not insert current state --- *)
	    val _ = SweepDeleteOccGraph ();

	    val _ = DSUI_SetStatusBarMessage ("Reinserting states ...");

	    (* --- restore the search trees - could be combined with insert in queue -- *)
	    val CPN'newnodes = (List.foldl (fn (CPN'n,count) => 
					   let
					       val CPN'newnode = Insert CPN'n;
					       val _ = TimePQ.insert(CPN'OGNodeSel.pq,CreationTime' CPN'newnode,
								     CPN'newnode)
					   in
					       count+1
					   end) 0 (CPN'savestate))
(*
	    val _ = DSUI_SetStatusBarMessage ("Restoring queues [2] ...");
*)
	    (* --- restore the priority queue - could be made more efficient -- *)
(*	    val _ = (map (fn CPN'n => 
			  let
			      val _ = () (* DSUI_SetStatusBarMessage (makestring CPN'n)*)
			  in
			      TimePQ.insert(CPN'OGNodeSel.pq,CreationTime' CPN'n,CPN'n)
			  end)
		     (List.rev CPN'newnodes))
	*)	     
	in
	    CPN'newnodes
	end

    (* --- a simple test implementation do n generations --- *)
    (* --- stop options are used to trigger sweep --- *)


    fun Generate i writelogfun = 
	let

(*	    val queuedat = TextIO.openOut "/tmp/queue.dat"
	    val deletedat = TextIO.openOut "/tmp/delete.dat"
	    val sweepdat = TextIO.openOut "/tmp/sweep.dat"*)

	    fun LocalGenerate (maxnodes,globtotal) 0 _ _ = 
		((*TextIO.closeOut queuedat;
		 TextIO.closeOut deletedat;
		 TextIO.closeOut sweepdat;*)
		 (Int.max(maxnodes,NoOfNodes ()),globtotal+NoOfNodes()))
	      | LocalGenerate (maxnodes,globtotal) i deleted lastmin = 
		(ResumeCalculateOccGraph ();		  
		 (Int.max(maxnodes,NoOfNodes ()),globtotal+NoOfNodes()))
		handle StopCritSatisfied => 
		    (let
			 val currnodes = NoOfNodes ();
			 val realtime = NoOfSecs ();
			 val (mintime,_) = (TimePQ.min(CPN'OGNodeSel.pq));
			     
			 val nextnodes = (if (mintime = lastmin)
					     then (writelogfun("GC [Postponed]\n");
						   currnodes)
					 else
					     (CopyAndRestore ()));
					     
			 val newdeleted = deleted + currnodes - nextnodes;
			 val total = deleted + currnodes;

			 val newmaxnodes = Int.max(maxnodes,currnodes)
			 val newtotalglob = newdeleted

			 val (mintime,_) = (TimePQ.min(CPN'OGNodeSel.pq));
			 val _ = writelogfun("GC [Nodes = "^(makestring total)^"/"^
					     (makestring currnodes)^"/"^
					     (makestring nextnodes)^
					     " Min Time = "^(IntInf.toString mintime)^"]\n");
 
(*			 val (maxtime,_) = (TimePQ.max(CPN'OGNodeSel.pq));*)
(*			 val sweeptime = IntInf.-(maxtime,mintime);*)
     
			(* val _ = TextIO.output
			     (queuedat,(makestring realtime)^" "^(makestring currnodes)^"\n");
			 val _ = TextIO.flushOut queuedat;

			 val _ = TextIO.output
			     (queuedat,(makestring (realtime+1))^" "^(makestring nextnodes)^"\n");
			 val _ = TextIO.flushOut queuedat;

			 val _ = TextIO.output
			     (deletedat,(makestring realtime)^" "^(makestring total)^"\n");
			 val _ = TextIO.flushOut deletedat;*)

(*			 val _ = TextIO.output
			     (sweepdat,(makestring realtime)^" "^(IntInf.toString sweeptime)^"\n");
			 val _ = TextIO.flushOut sweepdat;*)

			 val _ = DSUI_SetStatusBarMessage 
			     ("Generation: "^(makestring i)^
			      " Time ["^(IntInf.toString mintime)^
			      (*":"^(IntInf.toString maxtime)^*)"]"^
			      " Status: "^makestring currnodes^" -> "^
			      (makestring nextnodes)^" | "^
			      " Deleted: "^(makestring newdeleted))
			     
		     in
			 LocalGenerate (newmaxnodes,newtotalglob) (i-1) newdeleted mintime
		     end)

	    val CPN'CurState = !(CPN'OGCreateStateRec());
	    val _ = CPN'OGNodeSel.BuildList()

	in
	     (let 
		  val (maxnodes,globtotal) = LocalGenerate (0,0) i 0 (IntInf.fromInt 0);
	      in
		  CPN'OGToSimData.copyStateRec CPN'CurState;
		  DeleteOccGraph();
		  (maxnodes,globtotal)
	      end)
		  handle CPN'x => ((*TextIO.closeOut queuedat;
				   TextIO.closeOut deletedat;
				   TextIO.closeOut sweepdat;*)
				   CPN'OGToSimData.copyStateRec CPN'CurState;
				   DeleteOccGraph();
				   raise CPN'x)
		
	end
end


