(* 
 * Module:          Priority Queue
 *
 * Description:     Priority Queue over BB-Trees
 *
 * Created by:      Torben Bisgaard Haagh
 *
 * Date:            Spring 97
 *)


(* $Source: /users/cpn/designcpnR/src/current/oga/OGMLfiles/Attic/OGpq-tree.sml,v $

$Log: OGpq-tree.sml,v $
Revision 1.1.2.1  2000/07/20 21:36:28  kris
Initial Revision


*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/oga/OGMLfiles/Attic/OGpq-tree.sml,v 1.1.2.1 2000/07/20 21:36:28 kris Exp $";

(* creating a priority queue witn integer key and string element:
 * structure IntPQ = MakePQ(type key = int; val priority = Integer.<);
 * val pq: string IntPQ.pq = IntPQ.create()
 *)

functor MakeTreePQ(type key; val priority: key * key -> bool) : PRIORITYQUEUE = struct

local
    open CPN'Tree

    fun tinsert (TreeNil, CPN'k',CPN'e') = singleton (CPN'k',CPN'e')
      | tinsert (TreeNode{value=(CPN'k,CPN'e),left,right,...}, CPN'k', CPN'e') =
	if priority(CPN'k,CPN'k') then
	    balance((CPN'k,CPN'e), left, tinsert(right,CPN'k',CPN'e'))
	else if priority(CPN'k',CPN'k) then
	    balance((CPN'k,CPN'e), tinsert(left,CPN'k',CPN'e'), right)
	else if size left < size right then
	    balance((CPN'k,CPN'e), tinsert(left,CPN'k',CPN'e'), right)
	else
	    balance((CPN'k,CPN'e), left, tinsert(right,CPN'k',CPN'e'));

in
    type key = key;
    val priority = priority;
	
    type 'a pq = (key * 'a) tree ref

    exception EmptyPQ;

    fun is_empty (ref TreeNil) = true
      | is_empty _ = false

    fun create() = ref TreeNil

    fun min(pq: 'a pq) = CPN'Tree.min(!pq)

    fun max(pq: 'a pq) = CPN'Tree.max(!pq)

    fun app(pq: 'a pq) f = CPN'Tree.app f (!pq);

    fun insert (pq as ref tree, CPN'k, CPN'e) =
	pq:= tinsert(tree,CPN'k,CPN'e)
	
    fun deletemin (pq as ref tree) = let
	val (value,tree) = delmin tree
    in
	pq:= tree;
	value
    end
	
    fun deleteto (ref TreeNil,_) = nil
      | deleteto (pq as ref tree, until) =
	if priority(until,#1(min(pq))) then nil
	else deletemin(pq)::deleteto(pq,until)

    fun wipe pq = pq:= TreeNil

    fun size (pq: 'a pq) = CPN'Tree.size(!pq)
end
end

(* functor MakePQ = MakeTreePQ;   KHM: not allowed in SML'97 *)
