(*
 * Module:       Implements the switch to the OG Tool
 *
 * Description:  Design/CPN OG Tool
 *
 * Created by:   Lars Kristensen (kris@daimi.aau.dk)   
 *
 * Date:         29/09/95   
 *
 * Design/CPN (tm)  Copyright (c) Meta Software Corp 1989--1995
 *)

(* $Source: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OGswitch.sml,v $

$Log: OGswitch.sml,v $
Revision 1.13.2.8  2000/08/02 21:38:41  kris
added generation of sim store state function

Revision 1.13.2.7  2000/07/31 19:31:41  kris
Added support for user defined hash function in OG part

Revision 1.13.2.6  2000/07/30 16:03:20  kris
Added load of the new timed generation

Revision 1.13.2.5  2000/07/20 21:37:52  kris
OGCanonFromSimDataGen no longer loaded from ImageFiles

Revision 1.13.2.4  2000/07/13 13:44:08  kris
Added load of files for canonical states

Revision 1.13.2.3  2000/05/12 19:32:28  kris
OGswitch.sml

Revision 1.13.2.2  2000/05/10 06:59:31  kris
OGDelete.sml added supporting delete of state space

Revision 1.13.2.1  2000/02/22 09:06:18  sts
Added filtering function on successors.

Revision 1.13  1998/03/04 15:48:35  kris
Added generation of function for mapping external to internal representation of binding elements and synchronize occ and oeos wrt inclusion of OGCalcSucc.sml

Revision 1.12  1998/01/03 13:14:17  kris
Modification to make 1 tool work on state recs

Revision 1.11  1998/01/02 15:52:42  kris
Updated status bar messages to depend on Occ and 1

Revision 1.10  1998/01/02 10:37:08  kris
1 tool now uses OETransferStateGen instaed of OGTransferStateGen

Revision 1.9  1996/12/12 14:42:24  cpn
Cleaned up. The 1 tool now also uses OGCalcSucc.sml

Revision 1.8  1996/08/27 12:09:06  cpn
Mac Port v3.0.2

Revision 1.7  1996/06/27 14:30:20  cpn
Added inclusion of OGinterface for 1

Revision 1.6  1996/06/06 10:09:58  cpn
Integrated OE/OS Tool modifications (1)

Revision 1.5  1996/05/14 12:24:10  cpn
OGTimeNodeSel are now loaded when timed code has been generated

Revision 1.4  1996/05/01 15:36:03  cpn
Added inclusion of Defaults.sml (defaults for node/arc descr) and OGTime.sml (additiona aux functions for time)

Revision 1.3  1996/04/12 09:06:36  cpn
Addditnal information about progess during switch

Revision 1.2  1996/03/18 19:58:49  cpn
Added use of Report files

 Revision 1.1  1995/09/29  15:00:35  kris
 Initial revision
 

*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OGswitch.sml,v 1.13.2.8 2000/08/02 21:38:41 kris Exp $";

(******* Disable check to speed up and save stack space ******)
CPN'CompDep.CheckOff();


DSUI_SetStatusBarMessage("Entering OE/OS Graph...");



(******* net capture ******)

DSUI_SetStatusBarMessage("[1:10] Reading Net Information...");

CPN'NetCapture.exe trans_seq;
CPN'NetCapture.check_names();         

(*** net dependent 1 ***)

usestring(CPN'OGIdsGen.gen_TI());   
usestring(CPN'OGIdsGen.gen_PI());   
usestring(CPN'OGBTconvGen.gen_binding());
usestring(CPN'OGBTconvGen.gen_conv());
usestring(CPN'OGBTconvGen.gen_encode());
usestring(CPN'OGBTconvGen.gen_st_binding());      
usestring(CPN'OGBasicRefsGen.gen());     


(*** net independent middle ***)

DSUI_SetStatusBarMessage("[2:10] Building Data Structures...");



use (ogpath^"OESetMark.sml");
use (ogpath^"OEUtils.sml");


structure CPN'RecSearchTree
         = CPN'OGRECSTORAGE (struct
                              type OGrectype=CPN'OGrec;

                              val IsRecEq=CPN'OEUtils.CPN'OEDataRecIsEq;

                              val NoRec=CPN'NoRec
                             end);
                             
structure CPN'BindSearchTree
         = CPN'OGBINDSTORAGE (struct
                               type OGbindtype=CPN'Binding;
                               val emptybind=CPN'BTT000000:OGbindtype;
                               fun bindeq(CPN'b1,CPN'b2) = CPN'b1=CPN'b2
			      end);
                                     
usestring(CPN'OGGlobTablesGen.gen());   

use (ogpath^"OGStopLimitCrit.sml");   
use (ogpath^"OGEnabData.sml");
use (ogpath^"OGTimeEnabDataGen.sml");
      

(*** net dependent 2 ***)


DSUI_SetStatusBarMessage("[3:10] Defining Basic OE/OS Functions...");


use (ogpath^"OGUtils.sml"); 

if (!NewOGGeneration)
    then
	(if (!CPN'env.Timedef)=CPN'env.timenotdef
	     then
		 use (ogpath^"OGNewNodeSel.sml")
	 else
	     use (ogpath^"OGNewTimeNodeSel.sml"))
else
	(if (!CPN'env.Timedef)=CPN'env.timenotdef
	     then
		 use (ogpath^"OGNodeSel.sml")
	 else
	     use (ogpath^"OGTimeNodeSel.sml"));

    
usestring(CPN'OGMarkGen.gen());
usestring(CPN'OEMarkGen.gen());

use (ogpath^"OGSetMarkGen.sml");
usestring(CPN'OGSetMarkGen.gen ());

usestring(CPN'OGEncodeGen.gen());

usestring(CPN'OGToSimDataGen.gen());
usestring(CPN'OGFromSimDataGen.gen());

usestring(CPN'OEFromSimDataGen.gen());

use (ogpath^"OGCanonical.sml");
use (ogpath^"OGCanonFromSimDataGen.sml");
usestring [(CPN'OGCanonFromSimDataGen.gen ())];
usestring [(CPN'OGCanonFromSimDataGen.gensim ())];

use (ogpath^"OEEncode.sml");


usestring(CPN'OETransferStateGen.gen());

usestring(CPN'OGCreateStateRec.gen());
usestring(CPN'OGTransEnabOccGen.gen()); 

(*** net independent post : the last OG functions ***)

use (ogpath^"OGInOutPath.sml");

usestring(CPN'OGBindGen.gen_BindStruct());
usestring(CPN'OGBindGen.gen_ArcToBE());
use (ogpath^"OGBindMap.sml");
(* generate function *)
usestring(CPN'OGBindGenNew.gen_BEToArc ());

usestring(CPN'OGBindGen.gen_BEToTI());


use (ogpath^"OESetBind.sml");
use (ogpath^"OESet.sml");
use (ogpath^"OEMarkDefaultGen.sml");
usestring(MarkEquivDefaultGen()); 

use (ogpath^"OEBindUtils.sml");


DSUI_SetStatusBarMessage("[4:10] Defining Print Functions...");
use (ogpath^"OGPrintFuns.sml");
DSUI_SetStatusBarMessage("[5:10] Defining SCC Functions...");
use (ogpath^"SCC.sml"); 
DSUI_SetStatusBarMessage("[6:10] Defining Search Functions...");
use (ogpath^"OGSearchFuns.sml");
use (ogpath^"OGSearchReachable.sml");

use (ogpath^"OGTime.sml");

DSUI_SetStatusBarMessage("[7:10] Defining Drawing Functions...");
use (ogpath^"OGDrawUtils.sml");
use (ogpath^"OGDrawFuns.sml");
use (ogpath^"OGDrawSuppl.sml");
use (ogpath^"OGSetAttrOptions.sml");
usestring(CPN'OGIdsGen.gen_stTI());   
usestring(CPN'OGIdsGen.gen_stPI());   
usestring(CPN'OGMarkGen.gen_st()); 
usestring(CPN'OGBindGen.gen_st_BE());

use (ogpath^"OGProfile.sml");

use (ogpath^"OGBranching.sml");
if (!NewOGGeneration)
    then
	if (!CPN'env.Timedef)=CPN'env.timenotdef
	    then use (ogpath^"OGNewCalcSucc.sml")
	else
	    use (ogpath^"OGNewTimeCalcSucc.sml")
else
    use (ogpath^"OGCalcSucc.sml"); 

use (ogpath^"OGDelete.sml");

use (ogpath^"OGDefaults.sml");
DSUI_SetStatusBarMessage("[8:10] Defining Query Functions...");

use (ogpath^"OEProps.sml");



DSUI_SetStatusBarMessage("[9:10] Defining Utility Functions...");

use (ogpath^"OEUtilities/OEListBasics.sml");
use (ogpath^"OEUtilities/OEMulBasics.sml");
use (ogpath^"OEUtilities/OEConvertMuls.sml");
use (ogpath^"OEUtilities/OEResAndComp.sml");
use (ogpath^"OEUtilities/OEResUtils.sml");
use (ogpath^"OEUtilities/OEAllPerms.sml");
use (ogpath^"OEUtilities/OEPerm.sml");
use (ogpath^"OEUtilities/OEResToCanon.sml");
use (ogpath^"OEUtilities/OEApplyAndTest.sml");
use (ogpath^"OEUtilities/OESizes.sml");
use (ogpath^"OEUtilities/OESpecInterface.sml");
use (ogpath^"OEUtilities/OEInterface.sml");




DSUI_SetStatusBarMessage("[9:10] Defining Report Functions...");
use (ogpath^"OGNewSaveReport.sml");

use (ogpath^"OGMenu.sml");


DSUI_SetStatusBarMessage("[10:10] Copying Current Sim State to the OE/OS...");


CPN'OGNodeSel.NextNodeNum:=0;
OGSimStatetoOG();


DSUI_SetStatusBarMessage("OE/OS Graph Entered");


CPN'CompDep.CheckOn();


