(*
 * Module:       Report of the home properties functions
 *
 * Description:  Design/CPN OG Tool
 *
 * Created by:   Lars Kristensen (kris@daimi.aau.dk)   
 *
 * Date:         29/09/95   
 *
 * Design/CPN (tm)  Copyright (c) Meta Software Corp 1989--1995
 *)

(* $Source: /users/cpn/designcpnR/src/current/oga/OGMLfiles/ReportFiles/OGHome.sml,v $

 $Log: OGHome.sml,v $
 Revision 1.3  1996/05/14 12:41:04  cpn
 *** empty log message ***

  Revision 1.2  1996/02/04  13:49:16  kris
  Fix a bug in the output when more than 5 home states
 #
  Revision 1.1  1995/11/24  12:44:30  kris
  Initial revision
  

 *)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/oga/OGMLfiles/ReportFiles/OGHome.sml,v 1.3 1996/05/14 12:41:04 cpn Exp $";

structure CPN'OGHome =
    struct

	fun HomeMarkings () = 
	    let
		val CPN'hml = (ListHomeMarkings ());
		(* if we only have 5 or less home markings sort them *)
		val CPN'nohm = (length CPN'hml);
		    
		val CPN'all = (CPN'nohm = (NoOfNodes ()));
		val CPN'lt5 = (length CPN'hml <= 5);
		val CPN'none = (CPN'nohm = 0);
		    
		fun CPN'Print [] CPN'n = "" (* None will be printed *)
		  | CPN'Print (CPN'el1::(CPN'el2::CPN'rest)) 5 = 
		    mkst_Node(CPN'el1)^",...]"
		  | CPN'Print (CPN'el1::(CPN'el2::CPN'rest)) CPN'n = 
		    mkst_Node(CPN'el1)^","^(CPN'Print (CPN'el2::CPN'rest) (CPN'n+1))
		  | CPN'Print (CPN'el1::CPN'rest) CPN'n = 
		    mkst_Node(CPN'el1)^(CPN'Print CPN'rest (CPN'n+1))	

		fun CPN'SortAndPrint CPN'lst = 
		    let
			(* Simple bobble-sort - since we only have 5 elements *)
			fun CPN'sort CPN'lst = 
			    let
				fun CPN'insert [] CPN'el = CPN'el::[]
				  | CPN'insert ((CPN'el1:int)::CPN'l) (CPN'el2:int) =
				    if (CPN'el1 < CPN'el2)
					then CPN'el1::(CPN'insert CPN'l CPN'el2)
				    else
					(CPN'el2::(CPN'el1::CPN'l))
					
				fun CPN'sort' [] CPN'nl = CPN'nl
				  | CPN'sort' (CPN'el::CPN'tl) CPN'nl =
				    CPN'sort' CPN'tl (CPN'insert CPN'nl CPN'el) 
			    in
				CPN'sort' CPN'lst [] 
			    end
			
			
		    in
			"["^(CPN'Print (CPN'sort CPN'lst) 1)^"]"
		    end
		
		fun PrintHomeMarkings CPN'l =
		    if CPN'all 
			then "All"
		    else
			(if CPN'none
			     then
				 "None"
			 else
			     (if CPN'lt5
				  then
				      (CPN'SortAndPrint CPN'l)
			      else
				  makestring(length(CPN'l))^" ["^CPN'Print CPN'l 1))
	    in
		"\n\n Home Properties"
		^"\n------------------------------------------------------------------------\n"
		^"  Home Markings:  "
		^(if ((!CPN'env.Timedef <> CPN'env.timenotdef) andalso (with_time ()))
		      then
			  (if (InitialHomeMarking ())
			       then
				   "Initial Marking is a home marking"
			   else
			       "Initial Makring is not a home marking")
		  else (PrintHomeMarkings (CPN'hml)))^"\n"
	    end
	
    end (* struct CPN'OGHome *);

(* Setup reference in the main module *)
OGSaveReport.HomeMarkings := CPN'OGHome.HomeMarkings;
OGSaveReport.HomeMarkingsGen := true;
