(*
 * Module:       OG generation function calcSucc
 *
 * Description:  Design/CPN OG Tool
 *
 * Created by:   Lars Kristensen (kris@daimi.aau.dk)   
 *
 * Date:         29/09/95   
 *
 * Design/CPN (tm)  Copyright (c) Meta Software Corp 1989--1995
 *)


(* $Source: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OECalcSucc.sml,v $

$Log: OECalcSucc.sml,v $
Revision 1.1.4.2  1999/10/07 10:25:07  mailund
renamed i,j,t -> CPN'i,CPN'j,CPN't

Revision 1.1.4.1  1999/01/14 16:46:31  mailund
time port

Revision 1.1  1996/11/27 17:40:15  cpn
Initial revision
 

*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OECalcSucc.sml,v 1.1.4.2 1999/10/07 10:25:07 mailund Exp $";


(* this is the main OG generation function GenOG *)

local 

 val newnodelist = ref([]:Node list); (* temp store for numbers for new nodes *)
  
 val newarccount = ref 0;			 (* temp counter for new arcs *)
 
 val AlreadyProcesed = CPN'AvlTree.AvlNew(): unit CPN'AvlTree.avltree;
 val CPN'curtod = ref 0;

 fun CalcSucc' (CPN'n:Node)
    = let
       
       val NewCalcStat=ref FullProc; (* temp store for the future cal status of CPN'n *)
       
       fun createnewarc(srcnoderef,destnoderef,(CPN'tri,CPN'b))
          = let
             val newarcnum = CPN'OGNodeSel.NewArcNum();
		              
             val newarc
                = ref (CPN'OGarc {src=srcnoderef,
                                  dest=destnoderef,
                                  no=newarcnum,
                                  inscr=(CPN'tri,CPN'BindSearchTree.Insert
                                         (CPN'Binds,CPN'b,CPN'OGEncodeBinds CPN'b))})
            in
             CPN'AvlTree.AvlInsert CPN'OGArcs(mkst_Arc newarcnum,newarc);
              
             CPN'inc newarccount;
                           
             CPN'OGUtils.GetNodePredlist(!destnoderef):=
                                   newarc::(!(CPN'OGUtils.GetNodePredlist(!destnoderef)));
                           
             CPN'OGUtils.GetNodeSucclist(!srcnoderef):=
                                   newarc::(!(CPN'OGUtils.GetNodeSucclist(!srcnoderef)))                            
                          
            end(*let*);

       fun processBindElem (CPN'tri,CPN'b) (* execute CPN'tri with CPN'b and store *)
	   = (                              (* the result in the OG structures *)
             
	      CPN'OGToSimData.copy CPN'n; (* prepare the simulator data records *)
	      CPN'OGExecute(CPN'tri,CPN'b); (* execute CPN'ttri with CPN'b *)
	      (* DSUI_UserAckMessage ("Processing BE"); *)

	      let
		  val srcnoderef = CPN'OGUtils.NodeToNodeRef CPN'n; 
		  (* store the state and set the temporary marking record *)
		  val staterec = CPN'OEUtils.CPN'OESetTempMarkRec (CPN'OEstore_state(srcnoderef,CPN'tri)); 
		  val (insertstaterec,wasthere) = CPN'RecSearchTree.Member(CPN'State'recs, staterec, ((!CPN'OEEncode.CPN'EncodeMarkFun) ~1));

	      in
		  if not wasthere (* we have a new state and hence we create new node *)
		      then
			  let
			      val staterec = CPN'OGstore_state(srcnoderef,CPN'tri);
			      val (insertstaterec as 
				   ref(CPN'OGState(innerrec as {owner=ref(noderef),...})),wasthere) = CPN'RecSearchTree.Insert(CPN'State'recs, staterec, ((!CPN'OEEncode.CPN'EncodeMarkFun) ~1));			      
			      val newnodenum = CPN'OGNodeSel.NewNodeNum();
			      val destnoderef (* the bare record for the new node *)
				  = ref( CPN'OGnode {state=insertstaterec,
						     no=newnodenum,
						     succlist=ref [],
						     predlist=ref [], (* will be assigned below *)
						     calcstat=ref UnProc})
			  in
			      #owner innerrec:=destnoderef;
			      createnewarc(srcnoderef,destnoderef,(CPN'tri,CPN'b));
			      CPN'AvlTree.AvlInsert CPN'OGNodes(mkst_Node newnodenum,destnoderef);
			      newnodelist:= newnodenum::(!newnodelist)
			  end(*let*)
		  else (* we have reached an existing state and *)
		      (* hence we may only have to create an arc -- if it does not exist *)
		      let 
			  val destnoderef = (case insertstaterec of  
					     ref(CPN'OGState(innerrec as {owner=ref(noderef),...})) => noderef)
		      in
			  if CPN'OGListUtils.predlist(fn ref(CPN'OGarc{dest=CPN'dnr,inscr=(CPN'tri2,ref CPN'b2),...})
						      => (CPN'dnr=destnoderef)andalso (CPN'OEBindUtils.CPN'OEBindingIsEq((#1 CPN'tri),(#2 CPN'tri),CPN'b,(#1 CPN'tri2),(#2 CPN'tri2),CPN'b2)))

							  (!(CPN'OGUtils.GetNodeSucclist(!srcnoderef)))
			      then (* the arc was already there -- only when dev. prev. lim nodes *)
				  ()
			  else (* create a new arc *)
			      createnewarc(srcnoderef,destnoderef,(CPN'tri,CPN'b))
               end(*let*)
	      end(*let*));
	   
       fun local_st_TI(CPN'trmlno:int,CPN'instno:int)
	   = (makestring CPN'trmlno)^","^(makestring CPN'instno);

      in
       
         if (!(CPN'OGUtils.GetNodeCalcStat(CPN'OGUtils.NodeToNodeRec CPN'n))<>FullProc)
             andalso
             ((!CPN'OGBranching.TransInstLimit=0)orelse
             (let
                val transinsts = CPN'AvlTree.AvlNew(): unit CPN'AvlTree.avltree
              in
               map (fn ref(CPN'OGarc{inscr=(CPN'tri,_),...})
                      => CPN'AvlTree.AvlInsert transinsts (local_st_TI CPN'tri,())
                          handle CPN'AvlTree.ExcAvlInsert => ())
                   (!(CPN'OGUtils.GetNodeSucclist(CPN'OGUtils.NodeToNodeRec CPN'n)));
               (CPN'AvlTree.AvlNo transinsts)<(!CPN'OGBranching.TransInstLimit)
              end(*let*)))
         then (* let us calculate the successors for this non-fullproc node CPN'n, 
                 which does not fulfill the transition instance branching criteria *)
          (
	   (* DSUI_UserAckMessage("In CalcSucc'"); *)
           newnodelist:=[]; newarccount:=0; (* initialization *)
           
           CPN'OGEnabData.init();  (* initialization for immediate trans*)
           CPN'OGTimeEnabData.init();  (* initialization for timed trans*)
 
           CPN'OGToSimData.copy CPN'n;  (* prepare the simulator data records *)
           
           CPN'ValuePrint:=false; (* disable communication with C part for reg. sim. *)

           NewCalcStat:=FullProc; (* new calc status for CPN'n -- may be set to PartProc below *)
           
           CPN'OGCalcEnab() (* calculate the enabled binding elements in CPN'n *)
           handle CPN'OGEnabData.TransInstLimitReached
                 => NewCalcStat:=PartProc;
                 
           if ((!CPN'env.Timedef)<>CPN'env.timenotdef)andalso
              (with_time())andalso
              (CPN'OGEnabData.get_cands()<>[])andalso

	      (case (!CPN'OGTimeEnabData.curr_min) of
		   SOME CPN't => IntInf.>(CPN't,(time()))
		 | NONE => true)	(* NONE is larger than anything *)

           then (* only immediate trans should be executed at current model time *)
             CPN'OGTimeEnabData.init()
           else
             CPN'OGCalcEnabTimeBinds();
           
           CPN'ValuePrint:=true;  (* enable communication with C part for reg. sim. -- to be nice :-) *)
 
           if (!CPN'OGEnabData.bindlimit_reached)orelse
             (CPN'OGTimeEnabData.any_limit()) 
           then NewCalcStat:=PartProc
           else ();
           
           map (* generate successors arcs for each stored un-timed enabled binding element *)
            processBindElem (CPN'OGEnabData.get_cands());
 
           map (* generate successors arcs for each stored timed enabled binding element *)
            processBindElem (CPN'OGTimeEnabData.get_cands());
 
           CPN'OGUtils.GetNodeCalcStat(CPN'OGUtils.NodeToNodeRec CPN'n):= !NewCalcStat;
           
           CPN'AvlTree.AvlInsert AlreadyProcesed (mkst_Node CPN'n,())
             handle CPN'AvlTree.ExcAvlInsert => ();
             
           CPN'OGNodeSel.AppendList(CPN'OGListUtils.mapfilter( (*add the new nodes to the selection list*)
                    ref(fn CPN'outn
                          => (((CPN'AvlTree.AvlLookup(AlreadyProcesed,mkst_Node CPN'outn);
                               false) handle CPN'AvlTree.ExcAvlLookup => true)andalso
                               (!(CPN'OGUtils.GetNodeCalcStat(CPN'OGUtils.NodeToNodeRec CPN'outn))<>FullProc),
                              CPN'outn)),
                    rev (OutNodes CPN'n)));

           (* Freeze time *)
	   CPN'curtod := tod();
             
           (if CPN'OGStopCrit.IncrAndTest(length(!newnodelist),!newarccount,[CPN'n],!CPN'curtod)
           then (* a stop criteria has been satisfied *)
            raise CPN'OGNodeSel.StopCritSatisfied
           else (* continue *)
            ());

           CPN'OGInspection.Test (!CPN'curtod)
          )
         else
          (); 

         CalcSucc'(CPN'OGNodeSel.SelectNextNode()) (* select and treat next node *)
       
      end
in
 
 fun GenOG(CPN'nl:Node list)
    = (
       CPN'OGStopCrit.InitCounts();
       CPN'OGInspection.InitCounts();
       
       if CPN'nl=[]
       then 
        CPN'OGNodeSel.BuildList()
       else
        CPN'OGNodeSel.AssignList CPN'nl;
       
       CPN'AvlTree.AvlReset AlreadyProcesed;

	
       let
          val CPN'CurState = !(CPN'OGCreateStateRec())
         in 
          (((CalcSucc'(CPN'OGNodeSel.SelectNextNode());
              CPN'OGToSimData.copyStateRec CPN'CurState)
            handle CPN'OGNodeSel.NoSelPossible (* we are done *)
                   => (CPN'OGToSimData.copyStateRec CPN'CurState;
                        CPN'OGInspection.StandardReport()))
            handle CPN'OGNodeSel.StopCritSatisfied
                   => (CPN'OGToSimData.copyStateRec CPN'CurState;
                        CPN'OGInspection.StandardReport();
                        raise CPN'OGNodeSel.StopCritSatisfied))
         end)
                      
 fun ResumeGenOG()
    = (
       CPN'OGStopCrit.InitCounts();
       CPN'OGInspection.InitCounts();
       
 	let
          val CPN'CurState = !(CPN'OGCreateStateRec())
         in 
          (((CalcSucc'(CPN'OGNodeSel.SelectNextNode());
              CPN'OGToSimData.copyStateRec CPN'CurState)
            handle CPN'OGNodeSel.NoSelPossible (* we are done *)
                   => (CPN'OGToSimData.copyStateRec CPN'CurState;
                        CPN'OGInspection.StandardReport()))
            handle CPN'OGNodeSel.StopCritSatisfied
                   => (CPN'OGToSimData.copyStateRec CPN'CurState;
                        CPN'OGInspection.StandardReport();
                        raise CPN'OGNodeSel.StopCritSatisfied))
         end)

 fun CalculateOEGraph () = GenOG [];
              
end (*local*);
