(*
 * Module:       OG generation function calcSucc
 *
 * Description:  Design/CPN OG Tool
 *
 * Created by:   Lars Kristensen (kris@daimi.aau.dk)   
 *
 * Date:         29/09/95   
 *
 * Design/CPN (tm)  Copyright (c) Meta Software Corp 1989--1995
 *)

(* $Source: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OGCalcSucc.sml,v $

$Log: OGCalcSucc.sml,v $
Revision 1.7.2.6  2000/08/07 13:37:50  kris
Modification to reflect that branching options now works directly on state recs

Revision 1.7.2.5  2000/07/13 13:43:08  kris
Added support for canonical states in the OG part

Revision 1.7.2.4  2000/02/22 09:06:17  sts
Added filtering function on successors.

Revision 1.7.2.3  1999/10/07 10:25:09  mailund
renamed i,j,t -> CPN'i,CPN'j,CPN't

Revision 1.7.2.2  1999/01/14 16:46:32  mailund
time port

Revision 1.7.2.1  1998/10/12 12:46:14  kris
Start of 1 branch

Revision 1.7  1998/03/04 15:49:18  kris
Added handling of exception raised during OG generation

Revision 1.6  1998/01/03 13:34:59  kris
Fix of 971209b0039. Problem with timed occurence graphs was caused by duplicates in the list of enabled binding elements

Revision 1.5  1998/01/03 13:14:16  kris
Modification to make OEOS tool work on state recs

Revision 1.4  1996/12/12 14:41:11  cpn
Incorporated the support for OEOS

Revision 1.3  1996/11/09 14:17:40  sorenchr
fixed problem with variables in output pattern of codesegments.

 Revision 1.2  1996/02/04  13:56:33  kris
 Modified call to predlist which is now curried

 Revision 1.1  1995/09/29  14:31:22  kris
 Initial revision
 

*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OGCalcSucc.sml,v 1.7.2.6 2000/08/07 13:37:50 kris Exp $";

(* this is the main OG generation function GenOG *)

local 

 val newnodelist = ref([]:Node list); (* temp store for numbers for new nodes *)
  
 val newarccount = ref 0;			 (* temp counter for new arcs *)
 
 val AlreadyProcesed = CPN'AvlTree.AvlNew(): unit CPN'AvlTree.avltree;
 val CPN'curtod = ref 0;

 (* exception used to catch exception raised during generation *)
 exception CPN'OGExcDuringGen (*  of Node *) ;

 fun CalcSucc' (CPN'n:Node)
    = let
       
       val NewCalcStat=ref FullProc; (* temp store for the future cal status of CPN'n *)
       
       fun createnewarc(srcnoderef,destnoderef,(CPN'tri,CPN'b))
          = let
             val newarcnum = CPN'OGNodeSel.NewArcNum();
		              
             val newarc
                = ref (CPN'OGarc {src=srcnoderef,
                                  dest=destnoderef,
                                  no=newarcnum,
                                  inscr=(CPN'tri,CPN'BindSearchTree.Insert
                                         (CPN'Binds,CPN'b,CPN'OGEncodeBinds CPN'b))})
            in
             CPN'AvlTree.AvlInsert CPN'OGArcs(mkst_Arc newarcnum,newarc);
              
             CPN'inc newarccount;
                           
             CPN'OGUtils.GetNodePredlist(!destnoderef):=
                                   newarc::(!(CPN'OGUtils.GetNodePredlist(!destnoderef)));
                           
             CPN'OGUtils.GetNodeSucclist(!srcnoderef):=
                                   newarc::(!(CPN'OGUtils.GetNodeSucclist(!srcnoderef)))                            
                          
            end(*let*);

       fun processBindElem (CPN'tri,CPN'b) (* execute CPN'tri with CPN'b and store *)
          = (                              (* the result in the OG structures *)
             
             CPN'OGToSimData.copy CPN'n; (* prepare the simulator data records *)
             
             let
		 (* execute CPN'ttri with CPN'b, and record the new binding - this is to handle output form codeseg in the right way.... *)
		 
              val CPN'b = (CPN'OGExecute(CPN'tri,CPN'b) 
			   handle CPN'e => 

			       (DSUI_UserAckMessage ("ML exception "^(exnName CPN'e)^

						     " raised during processing of binding element: "^
						     (st_BE (CPN'IntBEToExtBE (CPN'tri,CPN'b)))^" in node: "^
						     (makestring CPN'n));
					    raise CPN'OGExcDuringGen));
		  
              val srcnoderef = CPN'OGUtils.NodeToNodeRef CPN'n; 
              

              (* val staterec = CPN'OGstore_state(srcnoderef,CPN'tri);*) (* store the state *)
                                                                    (* and return res. *)
                                                                    (* staterec *)
              
	      val staterec = 
		  (case (OGSetCanonical.GetCanonFun ()) of
		       NONE => CPN'OGstore_state(srcnoderef,CPN'tri)
		     | SOME(CPN'canonfun) => 
			   (let
				(* --- extract from simulator without storing --- *)
				val staterec = CPN'OEstore_state(srcnoderef,CPN'tri);
                                
				(* --- compute the canonical form --- *)
				val staterec' = CPN'canonfun staterec;
				val newsrcnoderef = 
				    ref (
					 CPN'OGnode {state = ref staterec',
						     no = 0,
						     succlist = ref [],
						     predlist = ref [],
						     calcstat = ref UnProc});
				    
				(* --- set the simulator state to the canonical form --- *)
				val _ = CPN'OGToSimData.copyStateRec (!newsrcnoderef); 
                        in
                            (* --- store new marking in the trees --- *)
                            CPN'store_state(newsrcnoderef)
                        end));

	      (* --- store the resulting state in the state space --- *)
              val (insertstaterec as
                   ref(CPN'OGState(innerrec as {owner=ref(noderef),...})),wasthere)
                 = CPN'RecSearchTree.Insert
                    (CPN'State'recs, staterec, CPN'OGEncode.State staterec);
 

             in

              if not wasthere (* we have a new state and hence we create new node *)
              then
		  let

                val newnodenum = CPN'OGNodeSel.NewNodeNum();
                
                val destnoderef (* the bare record for the new node *)
                   = ref( CPN'OGnode {state=insertstaterec,
                                      no=newnodenum,
                                      succlist=ref [],
                                      predlist=ref [], (* will be assigned below *)
                                      calcstat=ref UnProc})
                
               in
                #owner innerrec:=destnoderef;
            
                createnewarc(srcnoderef,destnoderef,(CPN'tri,CPN'b));
                  
                CPN'AvlTree.AvlInsert CPN'OGNodes(mkst_Node newnodenum,destnoderef);
           
                newnodelist:= newnodenum::(!newnodelist)
                         
               end(*let*)
        
              else (* we have reached an existing state and *)
                   (* hence we may only have to create an arc -- if it does not exist *)
         
               let 

                val destnoderef=noderef

               in

                if CPN'OGListUtils.predlist (
                    fn ref(CPN'OGarc{dest=CPN'dnr,inscr=(CPN'tri2,ref CPN'b2),...})
                      => (CPN'dnr=destnoderef)andalso
                         (CPN'tri2=CPN'tri)andalso(CPN'b2=CPN'b))
                    (!(CPN'OGUtils.GetNodeSucclist(!srcnoderef)))

                then (* the arc was already there -- only when dev. prev. lim nodes *)
                  ()
                else (* create a new arc *)
                  createnewarc(srcnoderef,destnoderef,(CPN'tri,CPN'b))
                
               end(*let*)
             
             end(*let*));
             
          fun local_st_TI(CPN'trmlno:int,CPN'instno:int)
             = (makestring CPN'trmlno)^","^(makestring CPN'instno);

      in
       
         if (!(CPN'OGUtils.GetNodeCalcStat(CPN'OGUtils.NodeToNodeRec CPN'n))<>FullProc)
             andalso
             ((!CPN'OGBranching.TransInstLimit=0)orelse
             (let
                val transinsts = CPN'AvlTree.AvlNew(): unit CPN'AvlTree.avltree
              in
               map (fn ref(CPN'OGarc{inscr=(CPN'tri,_),...})
                      => CPN'AvlTree.AvlInsert transinsts (local_st_TI CPN'tri,())
                          handle CPN'AvlTree.ExcAvlInsert => ())
                   (!(CPN'OGUtils.GetNodeSucclist(CPN'OGUtils.NodeToNodeRec CPN'n)));
               (CPN'AvlTree.AvlNo transinsts)<(!CPN'OGBranching.TransInstLimit)
              end(*let*)))
         then (* let us calculate the successors for this non-fullproc node CPN'n, 
                 which does not fulfill the transition instance branching criteria *)
          (
           newnodelist:=[]; newarccount:=0; (* initialization *)
           
           CPN'OGEnabData.init();  (* initialization for immediate trans*)
           CPN'OGTimeEnabData.init();  (* initialization for timed trans*)
 
           CPN'OGToSimData.copy CPN'n;  (* prepare the simulator data records *)
           
           CPN'ValuePrint:=false; (* disable communication with C part for reg. sim. *)

           NewCalcStat:=FullProc; (* new calc status for CPN'n -- may be set to PartProc below *)
           
           ((CPN'OGCalcEnab() (* calculate the enabled binding elements in CPN'n *)
	     handle CPN'OGEnabData.TransInstLimitReached
	     => NewCalcStat:=PartProc)

	    handle CPN'e => (DSUI_UserAckMessage ("ML exception "^(exnName CPN'e)^" raised during calculation of enabling in node: "^(makestring CPN'n));
			     raise CPN'OGExcDuringGen));


           if ((!CPN'env.Timedef)<>CPN'env.timenotdef)andalso
              (with_time())andalso
              (CPN'OGEnabData.get_cands()<>[])andalso

	      (case (!CPN'OGTimeEnabData.curr_min) of
		   SOME CPN't => IntInf.>(CPN't,(time()))
		 | NONE => true)	(* NONE is larger than anything *)

           then (* only immediate trans should be executed at current model time *)
             CPN'OGTimeEnabData.init()
           else
             CPN'OGCalcEnabTimeBinds();
           
           CPN'ValuePrint:=true;  (* enable communication with C part for reg. sim. -- to be nice :-) *)
 
           if (!CPN'OGEnabData.bindlimit_reached)orelse
             (CPN'OGTimeEnabData.any_limit()) 
           then NewCalcStat:=PartProc
           else ();
           
	       (* generate succesors for arcs stored as timed and
		  untimed enabled binding elements *)
	       map processBindElem (CPN'ReduceBEs (CPN'n,(remdupl ((CPN'OGEnabData.get_cands())^^
					     (CPN'OGTimeEnabData.get_cands())))));
 
           CPN'OGUtils.GetNodeCalcStat(CPN'OGUtils.NodeToNodeRec CPN'n):= !NewCalcStat;
           
           CPN'AvlTree.AvlInsert AlreadyProcesed (mkst_Node CPN'n,())
             handle CPN'AvlTree.ExcAvlInsert => ();
             
           CPN'OGNodeSel.AppendList(CPN'OGListUtils.mapfilter( (*add the new nodes to the selection list*)
                    ref(fn CPN'outn
                          => (((CPN'AvlTree.AvlLookup(AlreadyProcesed,mkst_Node CPN'outn);
                               false) handle CPN'AvlTree.ExcAvlLookup => true)andalso
                               (!(CPN'OGUtils.GetNodeCalcStat(CPN'OGUtils.NodeToNodeRec CPN'outn))<>FullProc),
                              CPN'outn)),
                    rev (OutNodes CPN'n)));

           (* Freeze time *)
	   CPN'curtod := tod();
             
           (if CPN'OGStopCrit.IncrAndTest(length(!newnodelist),!newarccount,[CPN'OGState (CPN'OGUtils.GetStateRec CPN'n)],!CPN'curtod)
           then (* a stop criteria has been satisfied *)
            raise CPN'OGNodeSel.StopCritSatisfied
           else (* continue *)
            ());

           CPN'OGInspection.Test (!CPN'curtod)
          )
         else
          (); 

         CalcSucc'(CPN'OGNodeSel.SelectNextNode()) (* select and treat next node *)
       
      end
in
 
 fun GenOG(CPN'nl:Node list)
    = (
       CPN'OGStopCrit.InitCounts();
       CPN'OGInspection.InitCounts();
       
       if CPN'nl=[]
       then 
        CPN'OGNodeSel.BuildList()
       else
        CPN'OGNodeSel.AssignList CPN'nl;
       
       CPN'AvlTree.AvlReset AlreadyProcesed;

	
       let
          val CPN'CurState = !(CPN'OGCreateStateRec())
         in 
          (((CalcSucc'(CPN'OGNodeSel.SelectNextNode());
              CPN'OGToSimData.copyStateRec CPN'CurState)
            handle CPN'OGNodeSel.NoSelPossible (* we are done *)
                   => (CPN'OGToSimData.copyStateRec CPN'CurState;
                        CPN'OGInspection.StandardReport()))
            handle CPN'OGNodeSel.StopCritSatisfied
                   => (CPN'OGToSimData.copyStateRec CPN'CurState;
                        CPN'OGInspection.StandardReport();
                        raise CPN'OGNodeSel.StopCritSatisfied))
	  handle CPN'OGExcDuringGen (* CPN'n *)
	  => (CPN'OGToSimData.copyStateRec CPN'CurState;
	      DSUI_SetStatusBarMessage("Generation Terminated."))

         end)
                      
 fun ResumeGenOG()
    = (
       CPN'OGStopCrit.InitCounts();
       CPN'OGInspection.InitCounts();
       
       let
	   val CPN'CurState = !(CPN'OGCreateStateRec())
       in 
	   (((CalcSucc'(CPN'OGNodeSel.SelectNextNode());
              CPN'OGToSimData.copyStateRec CPN'CurState)
	     handle CPN'OGNodeSel.NoSelPossible (* we are done *)
	     => (CPN'OGToSimData.copyStateRec CPN'CurState;
		 CPN'OGInspection.StandardReport()))
            handle CPN'OGNodeSel.StopCritSatisfied
	    => (CPN'OGToSimData.copyStateRec CPN'CurState;
		CPN'OGInspection.StandardReport();
		raise CPN'OGNodeSel.StopCritSatisfied)
	    handle CPN'OGExcDuringGen
	    => (CPN'OGToSimData.copyStateRec CPN'CurState;
		DSUI_SetStatusBarMessage("Generation Terminated.")
		))

       end)


 fun CalculateOccGraph () = GenOG [];

end (*local*);
