(*
 * Module:       Auxiliary function used by OGDrawFuns
 *
 * Description:  Design/CPN OG Tool
 *
 * Created by:   Lars Kristensen (kris@daimi.aau.dk)   
 *
 * Date:         29/09/95   
 *
 * Design/CPN (tm)  Copyright (c) Meta Software Corp 1989--1995
 *)

(* $Source: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OGDrawUtils.sml,v $

$Log: OGDrawUtils.sml,v $
Revision 1.5.4.2  2000/08/05 14:06:21  kris
Modify the node descriptor to not include assigned port as default

Revision 1.5.4.1  1999/01/04 09:16:29  mailund
char vs. string

Revision 1.5  1996/12/12 14:41:48  cpn
Moved ArcFilterGen and the functions dependency from mac to EML

Revision 1.4  1996/11/27 17:47:00  cpn
Mac Port

Revision 1.3  1996/04/15 08:58:42  cpn
Implemented the function ArcFilterGen for generation of the default arc ndescriptor function

Revision 1.2  1996/02/04 13:59:19  kris
Modified default settings and moved string rep. options to match the dialog

 Revision 1.1  1995/09/29  14:38:58  kris
 Initial revision
 

*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OGDrawUtils.sml,v 1.5.4.2 2000/08/05 14:06:21 kris Exp $";

(* this structure contains auxiliary function to use for in the OGDrawFuns structure *)
(* which defines the behavior of the OG Analyzer *)

(* Line patterns - shuould be part of OA *)
val None = 0;
val Black = 1;
val White = 20;

(* Fill patterns - should be part of OA *)

val Solid = 0;
val Dashed = 1;
val LongDashed = 2;
val Dotted = 3;
val DottedDashed = 4;


structure CPN'OGDrawUtils
= struct

(* constants for use with DSWtAttr_SetPetriNodeType : *)

val OGdescrrgntype = 12;


(* references variables to contains the settings set by SetInscrParams; *)
(* to be used in InscrExtNode and InscrNode : *)

val nodenumref = ref true;
val extpgnameref = ref false;
val extpgnumref = ref true;
val unprocref = ref "?";
val partprocref = ref "+";
val predsuccref = ref "=";
val deadlockref = ref "DEAD";

(* references variables to contains the settings set by SetDrawParams; *)

val nodesizeref = ref {width= 35, height= 35};

val nodeattrref = ref {fill= 0, line= 0, thick= 1};

val nodetextref = ref {size= 10, font= Courier, style= PlainText, just= Centered};
val nodesuccoffsetref = ref {horz= 0, vert= 80};
val nodepredoffsetref = ref {horz= 0, vert= ~80};
val succsuccoffsetref = ref {horz= 50, vert= 10};
val predpredoffsetref = ref {horz= ~50, vert= ~10};
val nodedescrsizeref = ref {width= 250, height= 80};
val arcdescrsizeref = ref {width= 100, height= 40};
val nodedescrtextref = ref {size= 10, font= Courier, style= PlainText, just= LeftJustification};
val arcdescrtextref = ref {size= 10, font= Courier, style= PlainText, just= LeftJustification};

val nodedescrattrref = ref {fill= 20, line= 1, thick= 1};
val arcdescrattrref = ref {fill= 20, line= 1, thick= 1};
val arcattrref = ref {fill= 1, line= 0, thick= 1};

val max_fanoutref = ref 10;
val max_faninref = ref 10;

(* InscrNode inscribes a node CPN'n  *)
(* The inscription is governed by the parameters set by the function *)
(* SetInscrParam *)
fun InscrNode CPN'n =
 let
  val noderec = CPN'OGUtils.NodeToNodeRec CPN'n; 
 in
  CPN'Edit.out_val(case !(CPN'OGUtils.GetNodeCalcStat noderec) of
                   UnProc=> "0" |
                   PartProc=> "1"|
                   FullProc=> "2");
  CPN'Edit.out_val (
         makestring(length (!(CPN'OGUtils.GetNodePredlist noderec))));
  CPN'Edit.out_val (         
        makestring(length (!(CPN'OGUtils.GetNodeSucclist noderec)))) end(*let*);

(* creates an internal node in the position given, and inserts it in     *)
(* the data structures of the OG; the caller must inscribe it by calling *)
(* InscrNode; returns the modstruct of the new graphical node.        *)
fun CreateNode{x=CPN'x,y=CPN'y,page=CPN'gp,ognode=CPN'n}
   = let
  	  val noderec = CPN'OGUtils.NodeToNodeRec CPN'n;
      val CPN'gn
         = DSStr_CreateNode
            {page=CPN'gp,x=CPN'x,y=CPN'y,
             w= #width(!nodesizeref),h= #height(!nodesizeref),shape=RNDRECT}
     in
      DSWtAttr_ObjectVisuals{obj=CPN'gn,
                             line= #line(!nodeattrref),thick= #thick(!nodeattrref),
                             fill= #fill(!nodeattrref),vis= true};
      CPN'MakeOGNode{no=CPN'n,id=CPN'gn, stat = (case !(CPN'OGUtils.GetNodeCalcStat noderec) of
                   UnProc=> 0 |
                   PartProc=> 1 |
                   FullProc=> 2 ), pre = length (!(CPN'OGUtils.GetNodePredlist noderec)),
                   post = length (!(CPN'OGUtils.GetNodeSucclist noderec)) };
      DSText_SetAttr{obj = CPN'gn,font = #font(!nodetextref),size = #size(!nodetextref), style= #style(!nodetextref), just = #just(!nodetextref)}; 
      CPN'gn
     end(*let*);

(*** addition 23/2/95 - jbj ***)
fun CreateNodeCheck{x=CPN'x,y=CPN'y,page=CPN'gp,ognode=CPN'n}
   = (CPN'GetOGNode{no=CPN'n, id=CPN'gp}
       handle CPN'GetOGNodeExc =>
         CreateNode{x=CPN'x,y=CPN'y,page=CPN'gp,ognode=CPN'n});
(*** end: addition 23/2/95 - jbj ***)

local
 fun CreateAnyConn (CPN'arcdata,CPN'a)=
  let
  val CPN'ga = DSStr_CreateConn CPN'arcdata
 in
  DSWtAttr_ObjectVisuals{obj=CPN'ga,
                         line= #line(!arcattrref),thick= #thick(!arcattrref),
                         fill= #fill(!arcattrref),vis= true};
  if (#node1 CPN'arcdata) = (#node2 CPN'arcdata)
  then
   let
    val {x=CPN'x,y=CPN'y} = DSRdAttr_GetObjectCenter (#node1 CPN'arcdata);
   in
    DSWtAttr_SetConnPoints{conn=CPN'ga,
        points=[0,0,CPN'x,CPN'y + #height(!nodesizeref),CPN'x + #width(!nodesizeref),CPN'y,0,0]}
   end
  else
   ();
  CPN'ga
 end(*let*)
in

(* Creates a new internal OG arc based on CPN'arcdata {node1,node2,page} *)
(* and the arc is inserted in the OG data structures *)
(* Returns the modstruct of the new arc *)
fun CreateConn (CPN'arcdata,CPN'a)=
 let
  val CPN'ga = CreateAnyConn (CPN'arcdata,CPN'a)
 in
  CPN'MakeOGArc{no = CPN'a, id = CPN'ga};
  CPN'ga
 end(*let*)

end(*local*);

(* find the second component given the first component and a list of *)
(* integer pairs *)
 fun findsecond(CPN'0,(CPN'1,CPN'2)::CPN'tl)
    = if CPN'0=CPN'1 then CPN'2 else findsecond(CPN'0,CPN'tl)
   | findsecond(_,nil)=0;

(* fun makegncopy CPN'gn
    = let
       val {w=CPN'w,h=CPN'h}=DSRdAttr_GetObjectSize CPN'gn;
       val {x=CPN'x,y=CPN'y}=DSRdAttr_GetObjectCenter CPN'gn       
      in
       DSStr_CreateNode{page=DSStr_GetParent CPN'gn,x=CPN'x,
            y=CPN'y,w=CPN'w,h=CPN'h,shape=RECTANGLE}
      end;
*)

 fun makegacopy CPN'ga (* makes a copy of a graphic arc and returns the new modstruct *)
    = let
       val {node1=CPN'n1,node2=CPN'n2}
          = DSRdAttr_GetConnEnds{conn=CPN'ga,rgn=false};
       val CPN'pl=DSRdAttr_ConnPoints CPN'ga;
       val CPN'gacopy = DSStr_CreateConn
            {page=DSStr_GetParent CPN'ga,node1=CPN'n1,node2=CPN'n2}      
      in
       DSWtAttr_SetConnPoints {conn=CPN'gacopy,points=CPN'pl};
        CPN'gacopy
      end(*let*);

  
  fun CreateNodeInscrReg {obj=CPN'gn,text=CPN't,x=CPN'x,y=CPN'y, page= CPN'gp} =
      let
       val CPN'gr=DSStr_CreateNode{x=CPN'x,y=CPN'y,page=CPN'gp,
                   w= #width(!nodedescrsizeref),h= #height(!nodedescrsizeref),shape=RECTANGLE}
      in
       DSStr_MakeNodeIntoRgn{parent=CPN'gn,obj=CPN'gr};
       DSWtAttr_ObjectVisuals{obj=CPN'gr,
                              line= #line(!nodedescrattrref),thick= #thick(!nodedescrattrref),
                              fill= #fill(!nodedescrattrref),vis= true};
       DSText_Put{obj=CPN'gr,text=CPN't};
       DSText_SetAttr{obj=CPN'gr,size= #size(!nodedescrtextref),font= #font(!nodedescrtextref),
           style= #style(!nodedescrtextref),just= #just(!nodedescrtextref)};
        CPN'gr
      end(*let*);
 
  fun CreateArcInscrReg {obj=CPN'gn,text=CPN't,x=CPN'x,y=CPN'y, page= CPN'gp} =
      let
       val CPN'gr=DSStr_CreateNode{x=CPN'x,y=CPN'y,page=CPN'gp,
                   w= #width(!arcdescrsizeref),h= #height(!arcdescrsizeref),shape=RECTANGLE}
      in
       DSStr_MakeNodeIntoRgn{parent=CPN'gn,obj=CPN'gr};
       DSWtAttr_ObjectVisuals{obj=CPN'gr,
                              line= #line(!arcdescrattrref),thick= #thick(!arcdescrattrref),
                              fill= #fill(!arcdescrattrref),vis= true};
       DSText_Put{obj=CPN'gr,text=CPN't};
       DSText_SetAttr{obj=CPN'gr,size= #size(!arcdescrtextref),font= #font(!arcdescrtextref),
           style= #style(!arcdescrtextref),just= #just(!arcdescrtextref)};
        CPN'gr
      end(*let*);
     
  local
      
   fun getnextinlist' (CPN'hd::CPN'tl,CPN'e)
      = if CPN'hd=CPN'e
        then 
         if length CPN'tl=0
         then ~1
         else hd CPN'tl
        else
         getnextinlist' (CPN'tl,CPN'e)
      | getnextinlist' _ = 0
 in
  (* gets the element following CPN'e in the list CPN'l in a cyclic manner *)
  fun getnextinlist (CPN'l as (CPN'hd::_),CPN'e)
     = let
        val CPN'en=getnextinlist'(CPN'l,CPN'e)
       in
        if CPN'en= ~1
        then CPN'hd (*CPN'e was last elem, so wrap around*)
        else CPN'en
       end(*let*)
    | getnextinlist _ = 0
 end(*local*);

 
 local 
 
 (* The local function below should is actually OA-utils and should be placed elsewhere. *)
  fun FontnoToSymbol 0 = "System"
    | FontnoToSymbol 1 = "ApplFont"
    | FontnoToSymbol 2 = "NewYork"
    | FontnoToSymbol 3 = "Geneva"
    | FontnoToSymbol 4 = "Monaco"
    | FontnoToSymbol 5 = "Venice"
    | FontnoToSymbol 6 = "London"
    | FontnoToSymbol 7 = "Athens"
    | FontnoToSymbol 8 = "SanFran"
    | FontnoToSymbol 9 = "Toronto"
    | FontnoToSymbol 10 = "NewYork"
    | FontnoToSymbol 11 = "Cairo"
    | FontnoToSymbol 12 = "LosAngeles"
    | FontnoToSymbol 20 = "Times"
    | FontnoToSymbol 21 = "Helvetica"
    | FontnoToSymbol 22 = "Courier"
    | FontnoToSymbol 23 = "Symbol"
    | FontnoToSymbol 24 = "Terminal"
    | FontnoToSymbol CPN'n = makestring CPN'n;

   fun StylenoToSym CPN'n =
    let
     fun CPN'pp [] = ""
       | CPN'pp [CPN'h] = CPN'h
       | CPN'pp (CPN'h::CPN't) = CPN'h^" + "^(CPN'pp CPN't);

     fun LStylenoToSym CPN'n CPN'st
       = if CPN'n>=Extended
       then LStylenoToSym (CPN'n-Extended) ("Extended"::CPN'st)
       else
      	if CPN'n>=Condense
      	then LStylenoToSym (CPN'n-Condense) ("Condense"::CPN'st)
      	else
       	if CPN'n>=Shadow
       	then LStylenoToSym (CPN'n-Shadow) ("Shadow"::CPN'st)
       else
        if CPN'n>=Outline
        then LStylenoToSym (CPN'n-Outline) ("Outline"::CPN'st)
        else
         if CPN'n>=Underline
         then LStylenoToSym (CPN'n-Underline) ("Underline"::CPN'st)
         else
          if CPN'n>=Italic
          then LStylenoToSym (CPN'n-Italic) ("Italic"::CPN'st)
          else
           if CPN'n>=Bold
           then LStylenoToSym (CPN'n-Bold) ("Bold"::CPN'st)
           else (* CPN'n = 0 *)
            if CPN'st=[] then "PlainText"
            else CPN'pp CPN'st;
     in
      LStylenoToSym CPN'n []
     end;


	fun JustnoToSymbol 1 = "LeftJustification"
	 | JustnoToSymbol 2 = "Centered"
	  | JustnoToSymbol 0 = "RightJustification"; (* actually is should be ~1 *)


	fun LinePatnotoSymbol 0 = "Solid"
	  | LinePatnotoSymbol 1 = "Dashed"
	  | LinePatnotoSymbol 2 = "LongDashed"
	  | LinePatnotoSymbol 3 = "Dotted"
	  | LinePatnotoSymbol 4 = "DottedDashed"
	  | LinePatnotoSymbol _ = "Solid";

	fun FillPatnotoSymbol 0 = "None"
	  | FillPatnotoSymbol 1 = "Black"
	  | FillPatnotoSymbol 20 = "White"
	  | FillPatnotoSymbol _ = "None";

   in 

	fun NodeFilterGen ()
	    = let
		  val CPN'store = ref [] : string list ref;
		  fun CPN'app CPN'st = CPN'store := CPN'st:: (!CPN'store);
	      in
		  CPN'app ("OGSet.NodeDescriptorOptions("

			   ^"\n"

			   ^"      fn (CPN'n:Node) =>"

			   ^"\n"
			   ^"      (st_Node CPN'n)^\"\\n\"");

		  map
		  (fn {pllist = pllist, pgname = pgname, instlist = instlist,...}
		   => CPN'AvlTree.AvlApp
		       (fn {plname="",...} => ()
			 | {plname=plname, pldbid=pldbid,...}
			   => (map 
			       (fn {instno=instno,instdbid=instdbid,...} =>
				let
				    val CPN'plikey=Cpndb.makestringdbid(pldbid)^" "^
					(Cpndb.makestringdbid(instdbid));
				    val CPN'IsAssignedPort = 
					((#plitype(CPN'AvlTree.AvlLookup
						   (CPN'OGIdsGen.PlaceInsts,CPN'plikey))) =  CPN'OGIdsGen.assignedport)
				in
				    if CPN'IsAssignedPort
					then ()
				    else
					CPN'app ("^"

						 ^"\n"

						 ^"      (st_Mark."
						 ^pgname^"'"
						 ^plname^" "^instno^" CPN'n)")
				end)
			       instlist;()),
			       pllist))
		  (!CPN'NetCapture.Net);       
		  CPN'app (");");
		  (implode (rev (!CPN'store)))
	      end;

	fun ArcFilterGen () 
	    = 

	    "\n\nOGSet.ArcDescriptorOptions(fn (CPN'a:Arc) =>\n"
	    ^" (st_Arc CPN'a)^\"\\n\"^\n"
	    ^"(st_BE(ArcToBE CPN'a)));"



       fun AttrOptionsGen(CPN'b1,CPN'b2,CPN'b3,CPN'b4,CPN'b5,CPN'b6,CPN'b7, CPN'b8, CPN'b9, CPN'b10, CPN'b11)
	   = let
		 val CPN'text_node = DSStr_CreateNode{page=DSStr_GetCurPage(),
						      x = 0, y = 0, w = 500, 
						      h = 500, shape = RECTANGLE};
		     
		 val text_store = 
		     (if CPN'b1
			  then 


			      "OGSet.NodeAttributes{\nSize = {width = "
			      ^makestring(#width (!nodesizeref))
			      ^", height = "
			      ^makestring(#height (!nodesizeref))
			      ^"},\nGraphics = {fill = "
			      ^FillPatnotoSymbol(#fill (!nodeattrref))
			      ^", line = "
			      ^LinePatnotoSymbol(#line (!nodeattrref))
			      ^", thick = "
			      ^makestring(#thick (!nodeattrref))
			      ^"},\nText = {size = "
			      ^makestring(#size (!nodetextref))
			      ^", font = "
			      ^FontnoToSymbol(#font (!nodetextref))
			      ^", style = "
			      ^StylenoToSym(#style (!nodetextref))
			      ^", just = "
			      ^JustnoToSymbol(#just (!nodetextref))^"}};"




		      else "")^
		      (if CPN'b2
			   then



			       "\n\nOGSet.ArcAttributes{\nGraphics = {fill = "
			       ^FillPatnotoSymbol(#fill (!arcattrref))
			       ^", line = "
			       ^LinePatnotoSymbol(#line (!arcattrref))
			       ^", thick = "
			       ^makestring(#thick (!arcattrref))^"}};"



		       else
			   "")^
		      (if CPN'b3
			   then 



			       "\n\nOGSet.NodeDescriptorAttributes{\n"
			       ^"Size = {width = "
			       ^makestring(#width (!nodedescrsizeref))
			       ^", height = "
			       ^makestring(#height (!nodedescrsizeref))
			       ^"},\nGraphics = {fill = "
			       ^FillPatnotoSymbol(#fill (!nodedescrattrref))
			       ^", line = "
			       ^LinePatnotoSymbol(#line (!nodedescrattrref))
			       ^", thick = "
			       ^makestring(#thick (!nodedescrattrref))
			       ^"},\nText = {size = "
			       ^makestring(#size (!nodedescrtextref))
			       ^", font = "
			       ^FontnoToSymbol(#font (!nodedescrtextref))
			       ^", style = "
			       ^StylenoToSym(#style (!nodedescrtextref))
			       ^", just = "
			       ^JustnoToSymbol(#just (!nodedescrtextref))^"}};"



		       else
			   "")^
		       (if CPN'b4
			    then



				"\n\nOGSet.ArcDescriptorAttributes{\n"
				^"Size = {width = "
				^makestring(#width (!arcdescrsizeref))
				^",height = "
				^makestring(#height (!arcdescrsizeref))
				^"},\nGraphics = {fill = "
				^FillPatnotoSymbol(#fill (!arcdescrattrref))
				^", line = "
				^LinePatnotoSymbol(#line (!arcdescrattrref))
				^", thick = "
				^makestring(#thick (!arcdescrattrref))
				^"},\nText = {size = "
				^makestring(#size (!arcdescrtextref))
				^", font = "
				^FontnoToSymbol(#font (!arcdescrtextref))
				^", style = "
				^StylenoToSym(#style (!arcdescrtextref))
				^", just = "
				^JustnoToSymbol(#just (!arcdescrtextref))^"}};"



			else
			    "")^
			    (if CPN'b11
				 then




				     "\n\nOGSet.StringRepOptions'Node(\n"
				     ^"fn (CPN'node) => CPN'node);\n\n"
				     ^"OGSet.StringRepOptions'Arc(\n"
				     ^"fn (CPN'arc, CPN'source, CPN'dest) => CPN'arc^\":\"^CPN'source^\"->\"^CPN'dest);"
				     ^"\n\nOGSet.StringRepOptions'PI(\n"
				     ^"fn (CPN'page, CPN'place, CPN'inst) => CPN'page^\"'\"^CPN'place^\" \"^CPN'inst);"
				     ^"\n\nOGSet.StringRepOptions'TI(\n"
				     ^"fn (CPN'page, CPN'trans, CPN'inst) => CPN'page^\"'\"^CPN'trans^\" \"^CPN'inst);"
				     ^"\n\nOGSet.StringRepOptions'BE(\n"
				     ^"fn (CPN'TI, CPN'bind) => CPN'TI^\": \"^CPN'bind);"
				     ^"\n\nOGSet.StringRepOptions'Mark(\n"
				     ^"fn (CPN'PI, CPN'mark) => CPN'PI^\": \"^CPN'mark^\"\\n\");"



				     else
					 "")^
			(if CPN'b5 

			     then "\n\n"^NodeFilterGen()

			 else
			     "")^
			(if CPN'b6
			     then ArcFilterGen ()
			 else
			     "")^
			 (if CPN'b7
			      then



				  "\n\nOGSet.SuccPredOptions{\n"
				  ^"FirstSucc = {horz = "
				  ^makestring(#horz (!nodesuccoffsetref))
				  ^", vert = "
				  ^makestring(#vert (!nodesuccoffsetref))
				  ^"},\nNextSucc = {horz = "
				  ^makestring(#horz (!succsuccoffsetref))
				  ^", vert = "
				  ^makestring(#vert (!succsuccoffsetref))
				  ^"},\nMaxSucc = "
				  ^makestring(!max_fanoutref)^",\n"
				  ^"FirstPred = {horz = "
				  ^makestring(#horz (!nodepredoffsetref))
				  ^", vert = "
				  ^makestring(#vert (!nodepredoffsetref))
				  ^"},\nNextPred = {horz = "
				  ^makestring(#horz (!predpredoffsetref))
				  ^", vert = "
				  ^makestring(#vert (!predpredoffsetref))
				  ^"},\nMaxPred = "
				  ^makestring(!max_faninref)^"};"



			  else
			      "")^
                          (if CPN'b8 
			       then



				   "\n\nOGSet.StopOptions{\n"
				   ^"Nodes = "
				   ^makestring(!CPN'OGStopCrit.NodeLimit)
				   ^",\nArcs = "
				   ^makestring(!CPN'OGStopCrit.ArcLimit)
				   ^",\nSecs = "
				   ^makestring(!CPN'OGStopCrit.CompTimeLimit)
				   ^",\nPredicate = fn _ => false};"



			   else
			       "")^
			   (if CPN'b9
				then 



				    "\n\nOGSet.BranchingOptions{\n"
				    ^"TransInsts = "
				    ^makestring(!CPN'OGBranching.TransInstLimit)
				    ^",\nBindings = "
				    ^makestring(!CPN'OGBranching.BindingLimit)
				    ^",\nPredicate = fn _ => true};"



			    else

				"")^"\n\n"^

			    (if CPN'b10
				 then



				     "OGSet.InspectionOptions{\n"
				     ^"Frequency = "
				     ^makestring(!CPN'OGInspection.Freq)
				     ^",\n"
				     ^"Action = !OGSet.StandardInspectionReport};"



			     else
				 "")
		     
	     in
		 DSText_SetAttr{obj=CPN'text_node,font=Courier,size=12,
				style=PlainText,just=LeftJustification};
		 DSText_Put{obj=CPN'text_node,text=text_store};
		 DSStr_SetCurObject(CPN'text_node)
	     end;

 end(*local*); 

end (*struct*);    
