(*
 * Module:       Main structure for generating a Report on the CP-net
 *
 * Description:  Design/CPN OG Tool
 *
 * Created by:   Lars Kristensen (kris@daimi.aau.dk)   
 *
 * Date:         29/09/95   
 *
 * Design/CPN (tm)  Copyright (c) Meta Software Corp 1989--1995
 *)

(* $Source: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OGNewSaveReport.sml,v $

$Log: OGNewSaveReport.sml,v $
Revision 1.3  1996/08/27 12:10:34  cpn
Mac Port v3.0.2

Revision 1.2  1996/05/14 12:29:11  cpn
Added TimedOG and SccGen functions

Revision 1.1  1995/11/24 12:40:27  kris
Initial revision
 

*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OGNewSaveReport.sml,v 1.3 1996/08/27 12:10:34 cpn Exp $";

structure OGSaveReport =
struct
    (* Statistics *)
    val Statistics = ref (fn () => "");
    val StatisticsGen = ref false;


    fun GetStatistics CPN'b =
	(if CPN'b 
	     then
		 (if not (!StatisticsGen)
		      then
			  (DSUI_SetStatusBarMessage("Defining Report Functions - 1:8");
			   use (ogpath^"ReportFiles/OGStats.sml");
			   DSUI_SetStatusBarMessage("Processing Statistics...");
			   (!Statistics)())
		  else
		      (!Statistics)())
	 else
	     "")

	     
    (* Boundedness Properties *)
    val IntegerBounds = ref (fn () => "");
    val IntegerBoundsGen = ref false;


    fun GetIntegerBounds CPN'b =
	(if CPN'b 
	     then
		 (if not (!IntegerBoundsGen)
		      then
			  (DSUI_SetStatusBarMessage("Defining Report Functions - 2:8");
			   use (ogpath^"ReportFiles/OGIntegerBoundsGen.sml");

			   use (ogpath^"ReportFiles/OGIntegerBounds.sml");
			   "\n\n Boundedness Properties\n------------------------------------------------------------------------\n"^(DSUI_SetStatusBarMessage("Processing Integer Bounds..."); (!IntegerBounds())))
		  else
		      "\n\n Boundedness Properties\n------------------------------------------------------------------------\n"^((DSUI_SetStatusBarMessage("Processing Integer Bounds..."); (!IntegerBounds))()))
	 else
	     "")

    val MultiSetBounds = ref (fn () => "");
    val MultiSetBoundsGen = ref false;

    fun GetMultiSetBounds (CPN'b, CPN'bi) = 
	(if CPN'b 
	     then
		 (if not (!MultiSetBoundsGen)
		      then
			  (DSUI_SetStatusBarMessage("Defining Report Functions - 3:8");
			   use (ogpath^"ReportFiles/OGMultiSetBoundsGen.sml");
			   
			   use (ogpath^"ReportFiles/OGMultiSetBounds.sml");
			   (if (not CPN'bi) then 
			   "\n\n Boundedness Properties\n------------------------------------------------------------------------\n\n"
			    else
				"")^(DSUI_SetStatusBarMessage("Processing Multi-set Bounds..."); (!MultiSetBounds())))
		  else
		      (if (not CPN'bi) then 
			   "\n\n Boundedness Properties\n------------------------------------------------------------------------\n\n"
		       else
			    "")^
			   (DSUI_SetStatusBarMessage("Processing Multi-set Bounds..."); (!MultiSetBounds)()))
	 else
	     "")

    (* Home Properties *)
    val HomeMarkings = ref (fn () => "");
    val HomeMarkingsGen = ref false


    fun GetHomeMarkings CPN'b = 
	(if CPN'b 
	     then
		 (if not (!HomeMarkingsGen)
		      then
			  (DSUI_SetStatusBarMessage("Defining Report Functions - 4:8");
			   use (ogpath^"ReportFiles/OGHome.sml");
			   (DSUI_SetStatusBarMessage("Processing Home Properties..."); (!HomeMarkings)()))
		  else
		      (DSUI_SetStatusBarMessage("Processing Home Properties..."); (!HomeMarkings)()))
	 else
	     "")

    (* Liveness Properties *)
    val DeadMarkings = ref (fn () => "");
    val DeadMarkingsGen = ref false


    fun GetDeadMarkings CPN'b =
	(if CPN'b 
	     then
		 (if not (!DeadMarkingsGen)
		      then
			  (DSUI_SetStatusBarMessage("Defining Report Functions - 5:8");
			   use (ogpath^"ReportFiles/OGDead.sml");
			   "\n\n Liveness Properties\n------------------------------------------------------------------------\n"^(DSUI_SetStatusBarMessage("Processing Liveness Properties..."); (!DeadMarkings)()))
		  else
			   "\n\n Liveness Properties\n------------------------------------------------------------------------\n"^(DSUI_SetStatusBarMessage("Processing Liveness Properties..."); (!DeadMarkings)()))
	 else
	     "")

    val DeadTIs = ref (fn () => "");
    val DeadTIsGen = ref false


    fun GetDeadTIs (CPN'b,CPN'pre) = 
	(if CPN'b 
	     then
		 (if not (!DeadTIsGen)
		      then
			  (DSUI_SetStatusBarMessage("Defining Report Functions - 6:8");
			   use (ogpath^"ReportFiles/OGDeadTIs.sml");
			   (if CPN'pre then "" else "\n\n Liveness Properties\n------------------------------------------------------------------------\n")^(DSUI_SetStatusBarMessage("Processing Liveness Properties..."); (!DeadTIs)()))
		  else
		      (if CPN'pre then "" else "\n\n Liveness Properties\n------------------------------------------------------------------------\n")^(DSUI_SetStatusBarMessage("Processing Liveness Properties..."); (!DeadTIs)()))
	 else
	     "")

    val LiveTIs = ref (fn () => "");
    val LiveTIsGen = ref false


    fun GetLiveTIs (CPN'b,CPN'pre) = 
	(if CPN'b 
	     then
		 (if not (!LiveTIsGen)
		      then
			  (DSUI_SetStatusBarMessage("Defining Report Functions - 7:8");
			   use (ogpath^"ReportFiles/OGLiveTIs.sml");
			   (if CPN'pre then "" else "\n\n Liveness Properties\n------------------------------------------------------------------------\n")^(DSUI_SetStatusBarMessage("Processing Liveness Properties..."); (!LiveTIs())))
		  else
		      (if CPN'pre then "" else "\n\n Liveness Properties\n------------------------------------------------------------------------\n")^(DSUI_SetStatusBarMessage("Processing Liveness Properties..."); (!LiveTIs)()))
	 else
	     "")

    (* Fairness Properties *)	
    val FairnessGen = ref false;
    val FairnessProperties = ref (fn () => "");


    fun GetFairnessProperties CPN'b = 
	(if CPN'b 
	     then
		 (if not (!FairnessGen)
		      then
			  (DSUI_SetStatusBarMessage("Defining Report Functions - 8:8");
			   use (ogpath^"ReportFiles/OGFairness.sml");
			   (DSUI_SetStatusBarMessage("Processing Fairness Properties..."); (!FairnessProperties)()))
		  else
		      (DSUI_SetStatusBarMessage("Processing Fairness Properties..."); (!FairnessProperties)()))
	 else
	     "")

    (* Functions which generates report according to the checked boxes *)
    (* in the Save Report dialog *)

    fun Report (CPN'b1,CPN'b2,CPN'b3,CPN'b4,CPN'b5,CPN'b6,CPN'b7,CPN'b8) = 
	((GetStatistics(CPN'b1))^(GetIntegerBounds(CPN'b2))^
	 (GetMultiSetBounds(CPN'b3,CPN'b2))^(GetHomeMarkings(CPN'b4))^
	 (GetDeadMarkings(CPN'b5))^(GetDeadTIs(CPN'b6,CPN'b5))^
	 (GetLiveTIs(CPN'b7,(CPN'b5 orelse CPN'b6)))^
	 (GetFairnessProperties(CPN'b8)))
	     
    (* Function to dump a string on a file *)
    fun DumpReport (CPN'path,CPN'report) =
        let val OUT = open_out(CPN'path)
        in
            (output(OUT,CPN'report);
             close_out(OUT))
        end

    fun FullOGAndScc () = if ((EntireGraphCalculated ()) andalso (SccGraphCalculated ())) then "1" else "0";

    fun FullOG () = if (EntireGraphCalculated ()) then "1" else "0";

    fun TimedOG () = if ((!CPN'env.Timedef <> CPN'env.timenotdef) andalso (with_time ())) then "1" else "0";
    fun SccGen () = if SccGraphCalculated () then "1" else "0";

    (* Main function invoked from the Menu module *)
    fun SaveReport (CPN'b1,CPN'b2,CPN'b3,CPN'b4,CPN'b5,CPN'b6,CPN'b7,CPN'b8) = 
	

	(DumpReport(DSUI_GetFileName{path = "",prompt = "Save the report as"},
		    Report(CPN'b1,CPN'b2,CPN'b3,CPN'b4,CPN'b5,CPN'b6,CPN'b7,CPN'b8));

	 DSUI_SetStatusBarMessage("Done.")

	 handle _  => (DSUI_SetStatusBarMessage("An Error Occurred..."); ()))



end (* struct CPN'OGReportGen *);

