(*
 * Module:       OG generation function calcSucc
 *
 * Description:  Design/CPN OG Tool
 *
 * Created by:   Lars Kristensen (kris@daimi.aau.dk)   
 *
 * Date:         29/09/95   
 *
 * Design/CPN (tm)  Copyright (c) Meta Software Corp 1989--1995
 *)

(* $Source: /users/cpn/designcpnR/src/current/oga/OGMLfiles/Attic/OGNewTimeCalcSucc.sml,v $

$Log: OGNewTimeCalcSucc.sml,v $
Revision 1.1.2.3  2000/08/06 15:09:13  kris
added profiling on time for calculation enabling

Revision 1.1.2.2  2000/07/31 19:31:06  kris
Added support for user defined hash coding and optional storage of arcs

Revision 1.1.2.1  2000/07/30 16:01:43  kris
initial revision


*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/oga/OGMLfiles/Attic/OGNewTimeCalcSucc.sml,v 1.1.2.3 2000/08/06 15:09:13 kris Exp $";

(* this is the main OG generation function GenOG *)

(* --- flag for switching on/off the storage of arcs --- *)
val CPN'storearcs = ref true;

local 

 val newnodecount = ref 0 (* temp store for numbers for new nodes *)
  
 val newarccount = ref 0; (* temp counter for new arcs *)
 
 (* val AlreadyProcesed = CPN'AvlTree.AvlNew(): unit CPN'AvlTree.avltree;*)
 val CPN'curtod = ref 0;

 (* exception used to catch exception raised during generation *)
 exception CPN'OGExcDuringGen (*  of Node *) ;

 fun CalcSucc' (CPN'n : CPN'OGnoderec ref)
    = let
       
       val NewCalcStat=ref FullProc; (* temp store for the future cal status of CPN'n *)
       
       fun createnewarc(srcnoderef,destnoderef,(CPN'tri,CPN'b))
          = let
             val newarcnum = CPN'OGNodeSel.NewArcNum();
		              
             val newarc
                = ref (CPN'OGarc {src=srcnoderef,
                                  dest=destnoderef,
                                  no=newarcnum,
                                  inscr=(CPN'tri,CPN'BindSearchTree.Insert
                                         (CPN'Binds,CPN'b,CPN'OGEncodeBinds CPN'b))})
            in
             CPN'AvlTree.AvlInsert CPN'OGArcs(mkst_Arc newarcnum,newarc);
              
             CPN'inc newarccount;
                           
             CPN'OGUtils.GetNodePredlist(!destnoderef):=
                                   newarc::(!(CPN'OGUtils.GetNodePredlist(!destnoderef)));
                           
             CPN'OGUtils.GetNodeSucclist(!srcnoderef):=
                                   newarc::(!(CPN'OGUtils.GetNodeSucclist(!srcnoderef)))                            
                          
            end(*let*);

       fun processBindElem (CPN'tri,CPN'b) (* execute CPN'tri with CPN'b and store *)
          = (                              (* the result in the OG structures *)
             
             CPN'OGToSimData.copyStateRec (!CPN'n); (* prepare the simulator data records *)
             
             let
		 (* execute CPN'ttri with CPN'b, and record the new binding - this is to handle output form codeseg in the right way.... *)
		 
              val CPN'b = (CPN'OGExecute(CPN'tri,CPN'b) 
			   handle CPN'e => 

			       (DSUI_UserAckMessage ("ML exception "^(exnName CPN'e)^

						     " raised during processing of binding element: "^
						     (st_BE (CPN'IntBEToExtBE (CPN'tri,CPN'b)))^" in node: "^
						     (makestring (CPN'OGUtils.GetNodeNo (!CPN'n))));
					    raise CPN'OGExcDuringGen));
		  
              val srcnoderef = CPN'n; 
              

              (* val staterec = CPN'OGstore_state(srcnoderef,CPN'tri);*) (* store the state *)
                                                                    (* and return res. *)
                                                                    (* staterec *)
              
	      val staterec = 
		  (case (OGSetCanonical.GetCanonFun ()) of
		       NONE => CPN'OGstore_state(srcnoderef,CPN'tri)
		     | SOME(CPN'canonfun) => 
			   (let
				(* --- extract from simulator without storing --- *)
				val staterec = CPN'OEstore_state(srcnoderef,CPN'tri);
                                
				(* --- compute the canonical form --- *)
				val staterec' = CPN'canonfun staterec;
				val newsrcnoderef = 
				    ref (
					 CPN'OGnode {state = ref staterec',
						     no = 0,
						     succlist = ref [],
						     predlist = ref [],
						     calcstat = ref UnProc});
				    
				(* --- set the simulator state to the canonical form --- *)
				val _ = CPN'OGToSimData.copyStateRec (!newsrcnoderef); 
                        in
                            (* --- store new marking in the trees --- *)
                            CPN'store_state(newsrcnoderef)
                        end));

	      (* --- store the resulting state in the state space --- *)
              val (insertstaterec as
                   ref(CPN'OGState(innerrec as {owner=ref(noderef),...})),wasthere)
                 = CPN'RecSearchTree.Insert
                    (CPN'State'recs, staterec, (!CPN'OEEncode.CPN'EncodeMarkFun) staterec);
 

             in

              if not wasthere (* we have a new state and hence we create new node *)
              then
		  let

                val newnodenum = CPN'OGNodeSel.NewNodeNum();
                
                val destnoderef (* the bare record for the new node *)
                   = ref( CPN'OGnode {state=insertstaterec,
                                      no=newnodenum,
                                      succlist=ref [],
                                      predlist=ref [], (* will be assigned below *)
                                      calcstat=ref UnProc})
                
               in
                #owner innerrec:=destnoderef;
		   
		(if (!CPN'storearcs)
		     then
			 createnewarc(srcnoderef,destnoderef,(CPN'tri,CPN'b))
		 else
		     ());

                CPN'AvlTree.AvlInsert CPN'OGNodes(mkst_Node newnodenum,destnoderef);
		CPN'OGNodeSel.AppendList [destnoderef];

                newnodecount:= (!newnodecount) + 1
                         
               end(*let*)
        
              else (* we have reached an existing state and *)
                   (* hence we may only have to create an arc -- if it does not exist *)
         
               let 

                val destnoderef=noderef

               in

                if CPN'OGListUtils.predlist (
                    fn ref(CPN'OGarc{dest=CPN'dnr,inscr=(CPN'tri2,ref CPN'b2),...})
                      => (CPN'dnr=destnoderef)andalso
                         (CPN'tri2=CPN'tri)andalso(CPN'b2=CPN'b))
                    (!(CPN'OGUtils.GetNodeSucclist(!srcnoderef)))

                then (* the arc was already there -- only when dev. prev. lim nodes *)
                  ()
                else (* create a new arc *)
		(if (!CPN'storearcs)
		     then
			 createnewarc(srcnoderef,destnoderef,(CPN'tri,CPN'b))
		 else
		     ())
               end(*let*)
             
             end(*let*));
             
          fun local_st_TI(CPN'trmlno:int,CPN'instno:int)
             = (makestring CPN'trmlno)^","^(makestring CPN'instno);

      in
       
         if (!(CPN'OGUtils.GetNodeCalcStat((!CPN'n)))<>FullProc)
             andalso
             ((!CPN'OGBranching.TransInstLimit=0)orelse
             (let
                val transinsts = CPN'AvlTree.AvlNew(): unit CPN'AvlTree.avltree
              in
               map (fn ref(CPN'OGarc{inscr=(CPN'tri,_),...})
                      => CPN'AvlTree.AvlInsert transinsts (local_st_TI CPN'tri,())
                          handle CPN'AvlTree.ExcAvlInsert => ())
                   (!(CPN'OGUtils.GetNodeSucclist(!CPN'n)));
               (CPN'AvlTree.AvlNo transinsts)<(!CPN'OGBranching.TransInstLimit)
              end(*let*)))
         then (* let us calculate the successors for this non-fullproc node CPN'n, 
                 which does not fulfill the transition instance branching criteria *)
          (
           newnodecount:=0; newarccount:=0; (* initialization *)
           OGTimeProfile.BeginObs 5;

           CPN'OGEnabData.init();  (* initialization for immediate trans*)
           CPN'OGTimeEnabData.init();  (* initialization for timed trans*)
 
           CPN'OGToSimData.copyStateRec (!CPN'n);  (* prepare the simulator data records *)
           
           CPN'ValuePrint:=false; (* disable communication with C part for reg. sim. *)

           NewCalcStat:=FullProc; (* new calc status for CPN'n -- may be set to PartProc below *)
           
           ((CPN'OGCalcEnab() (* calculate the enabled binding elements in CPN'n *)
	     handle CPN'OGEnabData.TransInstLimitReached
	     => NewCalcStat:=PartProc)

	    handle CPN'e => (DSUI_UserAckMessage ("ML exception "^(exnName CPN'e)^" raised during calculation of enabling in node: "^(makestring (CPN'OGUtils.GetNodeNo (!CPN'n))));
			     raise CPN'OGExcDuringGen));


           if ((!CPN'env.Timedef)<>CPN'env.timenotdef)andalso
              (with_time())andalso
              (CPN'OGEnabData.get_cands()<>[])andalso

	      (case (!CPN'OGTimeEnabData.curr_min) of
		   SOME CPN't => IntInf.>(CPN't,(time()))
		 | NONE => true)	(* NONE is larger than anything *)

           then (* only immediate trans should be executed at current model time *)
             CPN'OGTimeEnabData.init()
           else
             CPN'OGCalcEnabTimeBinds();
           
           CPN'ValuePrint:=true;  (* enable communication with C part for reg. sim. -- to be nice :-) *)
 
           if (!CPN'OGEnabData.bindlimit_reached)orelse
             (CPN'OGTimeEnabData.any_limit()) 
           then NewCalcStat:=PartProc
           else ();
           OGTimeProfile.EndObs 5;
	       (* generate succesors for arcs stored as timed and
		  untimed enabled binding elements *)
	       map processBindElem (CPN'ReduceBEs (CPN'OGUtils.GetNodeNo (!CPN'n),(remdupl ((CPN'OGEnabData.get_cands())^^
					     (CPN'OGTimeEnabData.get_cands())))));
 
           CPN'OGUtils.GetNodeCalcStat(!CPN'n):= !NewCalcStat;
           
(*
           CPN'AvlTree.AvlInsert AlreadyProcesed (mkst_Node CPN'n,())
             handle CPN'AvlTree.ExcAvlInsert => ();
             
           CPN'OGNodeSel.AppendList(CPN'OGListUtils.mapfilter( (*add the new nodes to the selection list*)
                    ref(fn CPN'outn
                          => (((CPN'AvlTree.AvlLookup(AlreadyProcesed,mkst_Node CPN'outn);
                               false) handle CPN'AvlTree.ExcAvlLookup => true)andalso
                               (!(CPN'OGUtils.GetNodeCalcStat(CPN'OGUtils.NodeToNodeRec CPN'outn))<>FullProc),
                              CPN'outn)),
                    rev (OutNodes CPN'n)));
*)
           (* Freeze time *)
	   CPN'curtod := tod();
             
           (if CPN'OGStopCrit.IncrAndTest((!newnodecount),!newarccount,[CPN'OGUtils.NodeRecToStateRec (!CPN'n)],!CPN'curtod)
           then (* a stop criteria has been satisfied *)
            raise CPN'OGNodeSel.StopCritSatisfied
           else (* continue *)
            ());

           CPN'OGInspection.Test (!CPN'curtod)
          )
         else
          (); 

         CalcSucc'(CPN'OGNodeSel.SelectNextNode()) (* select and treat next node *)
       
      end
in
 
 fun GenOG (CPN'nl:Node list)
    = (
       CPN'OGStopCrit.InitCounts();
       CPN'OGInspection.InitCounts();
       
       if CPN'nl=[]
       then 
        CPN'OGNodeSel.BuildList()
       else
        CPN'OGNodeSel.AssignList (map CPN'OGUtils.NodeToNodeRef CPN'nl); (* INEFFICIENT *)
       
(*       CPN'AvlTree.AvlReset AlreadyProcesed;*)

	
       let
          val CPN'CurState = !(CPN'OGCreateStateRec())
         in 
          (((CalcSucc'(CPN'OGNodeSel.SelectNextNode());
              CPN'OGToSimData.copyStateRec CPN'CurState)
            handle CPN'OGNodeSel.NoSelPossible (* we are done *)
                   => (CPN'OGToSimData.copyStateRec CPN'CurState;
                        CPN'OGInspection.StandardReport()))
            handle CPN'OGNodeSel.StopCritSatisfied
                   => (CPN'OGToSimData.copyStateRec CPN'CurState;
                        CPN'OGInspection.StandardReport();
                        raise CPN'OGNodeSel.StopCritSatisfied))
	  handle CPN'OGExcDuringGen (* CPN'n *)
	  => (CPN'OGToSimData.copyStateRec CPN'CurState;
	      DSUI_SetStatusBarMessage("Generation Terminated."))

         end)
                      
 fun ResumeGenOG()
    = (
       CPN'OGStopCrit.InitCounts();
       CPN'OGInspection.InitCounts();
       
       let
	   val CPN'CurState = !(CPN'OGCreateStateRec())
       in 
	   (((CalcSucc'(CPN'OGNodeSel.SelectNextNode());
              CPN'OGToSimData.copyStateRec CPN'CurState)
	     handle CPN'OGNodeSel.NoSelPossible (* we are done *)
	     => (CPN'OGToSimData.copyStateRec CPN'CurState;
		 CPN'OGInspection.StandardReport()))
            handle CPN'OGNodeSel.StopCritSatisfied
	    => (CPN'OGToSimData.copyStateRec CPN'CurState;
		CPN'OGInspection.StandardReport();
		raise CPN'OGNodeSel.StopCritSatisfied)
	    handle CPN'OGExcDuringGen
	    => (CPN'OGToSimData.copyStateRec CPN'CurState;
		DSUI_SetStatusBarMessage("Generation Terminated.")
		))

       end)


 fun CalculateOccGraph () = GenOG [];
 fun ResumeCalculateOccGraph () = ResumeGenOG ();

end (*local*);
