(*
 * Module:       Mechanisms for selecting the next node - tailored for time
 *
 * Description:  Design/CPN OG Tool
 *
 * Created by:   Lars Kristensen (kris@daimi.aau.dk)   
 *
 * Date:         29/09/95   
 *
 * Design/CPN (tm)  Copyright (c) Meta Software Corp 1989--1995
 *)

(* $Source: /users/cpn/designcpnR/src/current/oga/OGMLfiles/Attic/OGNewTimeNodeSel.sml,v $

$Log: OGNewTimeNodeSel.sml,v $
Revision 1.1.2.3  2000/08/06 15:06:41  kris
Implemented depth-first mode, made brahcing function work directly on state recs, and cleaned-up

Revision 1.1.2.2  2000/08/03 12:24:37  kris
Removed a ;

Revision 1.1.2.1  2000/07/30 16:01:43  kris
initial revision


*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/oga/OGMLfiles/Attic/OGNewTimeNodeSel.sml,v 1.1.2.3 2000/08/06 15:06:41 kris Exp $";

(* this structure hold the mechanims for selecting the next node *)
(* for processing in the OG generation                           *)

(* --- addition for using a priority queue based on creation time --- *)
(* --- a tree based implementation might be better --- *)
use (ogpath^"OGtree.sml");
use (ogpath^"OGpq-sig.sml");
use (ogpath^"OGpq-tree.sml");

structure TimePQ = MakeTreePQ(struct
				  type key = TIME
				  val priority = IntInf.<
			      end);
structure CPN'OGNodeSel =
    struct
	
	exception StopCritSatisfied;
	exception NoSelPossible;

	(* --- generation order: breadth (using priority) or depth first --- *)
	datatype CPN'GenOrder = OG'DepthFirst | OG'BreadthFirst;
	    
	val SS'genogorder = ref OG'BreadthFirst;
    
	fun SetGenOrder SS'val = (SS'genogorder:= SS'val);
	    
	fun GetGenOrder () = (!SS'genogorder);

	fun IsDepthFirst () = ((!SS'genogorder) = OG'DepthFirst);

	val ToProcess = ref ([] : (CPN'OGnoderec ref) list);
	val pq: (CPN'OGnoderec ref) TimePQ.pq = TimePQ.create()

	val NextNodeNum = ref 0;
	fun NewNodeNum () = (CPN'inc NextNodeNum;!NextNodeNum);
	    
	val NextArcNum = ref 0;
	fun NewArcNum () = (CPN'inc NextArcNum;!NextArcNum);
	    
	fun CreationTime (ref (CPN'OGnode 
			       {state = ref (CPN'OGState 
					     {creationtime = creationtime,...}),...})) = 
	    creationtime;
	    

	fun StdSelNextNode () (* : Node*) = 
	    (if (IsDepthFirst ())
		 then
		     (case (!ToProcess) of
			  [] => raise NoSelPossible
			| _ => 
			      (let
				   val nextnode = List.hd(!ToProcess);
				   val _ = (ToProcess := List.tl(!ToProcess))
			       in
				   if !CPN'OGBranching.Pred (CPN'OGUtils.NodeRecToStateRec (!nextnode)) 
				       then
					   nextnode
				   else 
				       (CPN'OGUtils.GetNodeCalcStat
					(!nextnode):=PartProc;
					StdSelNextNode())
			       end))
	     else
		 (case TimePQ.is_empty(pq) of
		      true => raise NoSelPossible
		    | _ => 
			  (let
			       val (key,nextnode) = TimePQ.deletemin pq
			   in
			       if !CPN'OGBranching.Pred (CPN'OGUtils.NodeRecToStateRec (!nextnode)) 
				   then
				       nextnode
			       else 
				   (CPN'OGUtils.GetNodeCalcStat
				    (!nextnode):=PartProc;
				    StdSelNextNode())
			   end)))
		  
	val OnlyThisList=ref false;

	val SelectNextNode = StdSelNextNode;
  
	fun AssignList CPN'nl = 
	    let
		val CPN'newnl = (CPN'OGListUtils.mapfilter
				 (
				  ref(fn CPN'n
				      => ((CPN'AvlTree.AvlLookup (CPN'OGNodes,mkst_Node (CPN'OGUtils.
											 GetNodeNo (!CPN'n)));
					   true) handle CPN'AvlTree.ExcAvlLookup => false,
					  CPN'n)),
				  CPN'nl))
	    in
		(if (IsDepthFirst ())
		     then
			 (ToProcess := CPN'newnl)
		 else
		     (TimePQ.wipe(pq);
		      (app (fn CPN'n => TimePQ.insert(pq,CreationTime CPN'n,CPN'n)) CPN'newnl)))
	    end;
	    
	fun AppendList [] = () 
	  | AppendList CPN'nl = 
	    if (!OnlyThisList)
		then ()
	    else
		(if (IsDepthFirst ())
		     then
			 (ToProcess := CPN'nl^^(!ToProcess))
		 else
		     (app (fn CPN'n => TimePQ.insert(pq,(CreationTime CPN'n),CPN'n)) CPN'nl))
		     
	fun BuildList () = 
	    (CPN'AvlTree.AvlApp(
			  fn ref(CPN'OGnode{calcstat,no,...})
			  => if !calcstat<>FullProc
				 then
				     if !CPN'OGBranching.Pred (CPN'OGState (CPN'OGUtils.GetStateRec no)) (* VERY INEFFICIENT *)
					 then 
					     let
						 val CPN'n = CPN'OGUtils.NodeToNodeRef no
					     in
						 (if (IsDepthFirst ())
						      then
							  (ToProcess := CPN'n::(!ToProcess))
						  else
						      TimePQ.insert(pq,CreationTime CPN'n,CPN'n))
					     end
				     else calcstat:= PartProc
			     else
				 (), CPN'OGNodes));
	    
	fun InitList () = 
	    (if (IsDepthFirst ())
		 then
		     (ToProcess := [])
	     else
		 (TimePQ.wipe(pq)));
	    
	fun StartSelOnlyOne() = OnlyThisList:=true;
	    
	fun RestoreStdSel() =  OnlyThisList:=false;
	    
end(*struct*); 
