(*
 * Module:       User accesible node and arc search functions
 *
 * Description:  Design/CPN OG Tool
 *
 * Created by:   Lars Kristensen (kris@daimi.aau.dk)   
 *
 * Date:         29/09/95   
 *
 * Design/CPN (tm)  Copyright (c) Meta Software Corp 1989--1995
 *)

(* $Source: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OGSearchFuns.sml,v $

$Log: OGSearchFuns.sml,v $
Revision 1.2  1996/02/04 14:02:06  kris
SearchNodes now operates on Node list instated of an int list

 Revision 1.1  1995/09/29  14:49:41  kris
 Initial revision
 

*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OGSearchFuns.sml,v 1.2 1996/02/04 14:02:06 kris Exp $";

(* this files define the user accesible node and arc search functions and *)
(* their derivates plus one of the standard query functions               *)

local
 exception stopsearch;
 val CPN'found=ref 0
in
 fun SearchNodes(CPN'nodelist:Node list, CPN'pred, CPN'limit, CPN'eval, CPN'start, CPN'comb)
    = let
       val CPN'result=ref CPN'start;
       fun searchone (CPN'n:Node)
          = if CPN'pred CPN'n
            then
             (CPN'result:=CPN'comb(CPN'eval(CPN'n),!CPN'result);
              CPN'inc CPN'found;
              if !CPN'found=CPN'limit
              then raise stopsearch
              else ())
            else ()
      in
       CPN'result:=CPN'start;
       CPN'found:=0;
       (if CPN'nodelist=[0]
        then CPN'AvlTree.AvlApp(fn ref(CPN'OGnode{no=CPN'n,...})
                                  => searchone CPN'n, CPN'OGNodes)
        else (map searchone CPN'nodelist;())) handle stopsearch => ();
        !CPN'result
      end(*let*)
end(*local*);

fun PredNodes (CPN'area,CPN'pred,CPN'limit)
   = SearchNodes(CPN'area,CPN'pred,CPN'limit,id,[],op ::);
fun EvalNodes(CPN'area,CPN'eval)
   = SearchNodes(CPN'area,fn _=>true,0,CPN'eval,[],op ::);
fun SearchAllNodes(CPN'pred,CPN'eval,CPN'start,CPN'comb)
   = SearchNodes([0],CPN'pred,0,CPN'eval,CPN'start,CPN'comb);
fun PredAllNodes CPN'pred
   = PredNodes([0],CPN'pred,0);
fun EvalAllNodes CPN'eval
   = EvalNodes([0],CPN'eval);
 
local
 exception stopsearch;
 val CPN'found=ref 0
in
 fun SearchArcs (CPN'arclist: Arc list,CPN'pred,CPN'limit,CPN'eval,CPN'start,CPN'comb)
    = let
       val CPN'result=ref CPN'start;
       fun processarc (CPN'a:Arc)
          = if CPN'pred CPN'a
            then
             (CPN'result:=CPN'comb(CPN'eval CPN'a,!CPN'result);
              CPN'inc CPN'found;
              if !CPN'found=CPN'limit
              then raise stopsearch
              else ())
            else ()
      in
       CPN'result:=CPN'start;
       CPN'found:=0;
       (if CPN'arclist=[0]
        then CPN'AvlTree.AvlApp(
              fn ref(CPN'OGarc
                     {no = CPN'a,...})
               => processarc CPN'a,
               CPN'OGArcs)
         else (map processarc CPN'arclist;())) handle stopsearch => ();
       !CPN'result
      end
end;


fun PredArcs (CPN'area,CPN'pred,CPN'limit) = SearchArcs(CPN'area,CPN'pred,CPN'limit,id,[],op ::);

fun EvalArcs(CPN'area,CPN'eval) = SearchArcs(CPN'area,fn _=>true,0,CPN'eval,[],op ::);

fun SearchAllArcs(CPN'pred,CPN'eval,CPN'start,CPN'comb)
   = SearchArcs([0],CPN'pred,0,CPN'eval,CPN'start,CPN'comb);

fun PredAllArcs CPN'pred
   = PredArcs ([0],CPN'pred,0);

fun EvalAllArcs CPN'eval
   = EvalArcs([0],CPN'eval);
