(*
 * Module:       Timed Equivalence
 *
 * Description:  Generation of canonical form
 *
 * Created by:   Lars Michael Kristensen (kris@daimi.au.dk)   
 *
 * Date:         12/07/2000   
 *
*)

(* $Source: /users/cpn/designcpnR/src/current/oga/OGMLfiles/Attic/OGTimeEquivCanon.sml,v $

$Log: OGTimeEquivCanon.sml,v $
Revision 1.1.2.6  2000/08/03 12:25:54  kris
Added support for occurrence time equivalence and renamed the canonical functions

Revision 1.1.2.5  2000/08/02 21:37:19  kris
First step towards making canon more efficient by avoiding multiple calculation of occurrence time

Revision 1.1.2.4  2000/07/31 19:32:50  kris
Improved canonical function by not normalising assigned ports

Revision 1.1.2.3  2000/07/29 14:13:11  kris
Fixed a problem when places has different timed colour sets

Revision 1.1.2.2  2000/07/14 13:46:55  kris
Creation time now set to zero

Revision 1.1.2.1  2000/07/13 13:41:28  kris
Initial revision


*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/oga/OGMLfiles/Attic/OGTimeEquivCanon.sml,v 1.1.2.6 2000/08/03 12:25:54 kris Exp $";

(* --- generation of EquivMark and NodeDescriptor --- *)
structure CPN'TimedEquivGen =
    struct
    local
        open CPN'NetCapture;
        open CPN'CodeGenUtils;


    in
        fun gen_CanonMarkEquiv () =
            (
             code_temp := "CPN'newstaterec \n end);\n";
             map
             (fn {pllist = pllist, pgname = pgname, instlist = instlist,...}
              => CPN'AvlTree.AvlApp
                  (fn {plname="",...} => ()
                    | {plname=plname, pldbid=pldbid,...}
                      => ( map
                          (fn {instno=instno,instdbid=instdbid,...}
                           => let
                               val CPN'key = pgname^"'"^plname^instno;
                               val CPN'PI = pgname^"'"^plname^" "^instno;
                               val CPN'plikey=Cpndb.makestringdbid(pldbid)^" "^
				   (Cpndb.makestringdbid(instdbid));
                               val CPN'colset = 
				   (#colset(CPN'AvlTree.AvlLookup
					    (CPN'OGIdsGen.PlaceInsts,CPN'plikey)));
                               val CPN'IsTimed = CPN'env.IsTimed CPN'colset;
			       val CPN'IsAssignedPort = 
				   ((#plitype(CPN'AvlTree.AvlLookup
					    (CPN'OGIdsGen.PlaceInsts,CPN'plikey))) =  CPN'OGIdsGen.assignedport)
                              in
				  if CPN'IsAssignedPort
				      then ()
				  else
				      (code_temp := 
				       (case CPN'IsTimed of
					    true => 
						"(OESetMark."^CPN'PI^" CPN'newstaterec (subtractfun ("^
						"OEMark."^CPN'PI^" CPN'staterec)));\n"
					  | false => 
						"(OESetMark."^CPN'PI^" CPN'newstaterec ("^
						"OEMark."^CPN'PI^" CPN'staterec));\n")^(!code_temp)
					    )
                              end(*let*))
                          instlist; 
                          ()),pllist))
             (!CPN'NetCapture.Net); 
             code_temp :=  
	     "fun CanonMarkEquiv subtimefun timeminfun (CPN'staterec "^ 
	     "as CPN'OGState {owner = owner, creationtime = creationtime,...}) = \n "^
	     "(let \n "^
	     " val CPN'sub = subtimefun CPN'staterec;\n"^
	     " val CPN'min = timeminfun CPN'staterec;\n"^
	     "  fun subtractfun CPN'ms = TimedOE.tms_subtracttime CPN'sub CPN'min CPN'ms \n"^
	     "  val CPN'newstaterec = OGCreateEmptyState owner (IntInf.fromInt 0) \n "^
	     "in \n"^(!code_temp)^
	     (* CT - move everything down by creation time, and make zero the minimal time value *)
	     "val CanonMarkEquivCT = CanonMarkEquiv TimedOE.IntFindCreationTime (fn CPN'x => (IntInf.fromInt 0));\n"^
	     (* OT - move everything down by creation time, and make (ot-ct) the minimal time value *)
	     "val CanonMarkEquivOT = CanonMarkEquiv TimedOE.IntFindCreationTime TimedOE.IntFindTimeDelay;\n"^
	     (* WOT - move everything down by occurrence time, and make zero the minimal time value *)
	     "val CanonMarkEquivWOT = CanonMarkEquiv TimedOE.IntFindOccurrenceTime (fn CPN'x => (IntInf.fromInt 0));\n";
             ([!code_temp]))

    end
end;
