(*
 * Module:       Mechanisms for selecting the next node - tailored for time
 *
 * Description:  Design/CPN OG Tool
 *
 * Created by:   Lars Kristensen (kris@daimi.aau.dk)   
 *
 * Date:         29/09/95   
 *
 * Design/CPN (tm)  Copyright (c) Meta Software Corp 1989--1995
 *)

(* $Source: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OGTimeNodeSel.sml,v $

$Log: OGTimeNodeSel.sml,v $
Revision 1.1.4.3  2000/07/20 21:35:36  kris
Modified implementation to use a priority queue from the new simulator

Revision 1.1.4.2  2000/05/11 13:24:17  kris
Added InitList function

Revision 1.1.4.1  1999/01/14 16:46:34  mailund
time port

Revision 1.1  1996/11/27 17:30:55  cpn
Initial revision
 

*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/oga/OGMLfiles/OGTimeNodeSel.sml,v 1.1.4.3 2000/07/20 21:35:36 kris Exp $";

(* this structure hold the mechanims for selecting the next node *)
(* for processing in the OG generation                           *)

(* --- addition for using a priority queue based on creation time --- *)
(* --- a tree based implementation might be better --- *)
use (ogpath^"OGtree.sml");
use (ogpath^"OGpq-sig.sml");
use (ogpath^"OGpq-tree.sml");

structure TimePQ = MakeTreePQ(struct
				     type key = TIME;
				     val priority = IntInf.<
				 end);
structure CPN'OGNodeSel =
struct

exception StopCritSatisfied;
exception NoSelPossible;

val NextNodeNum = ref 0;
fun NewNodeNum () = (CPN'inc NextNodeNum;!NextNodeNum);

val NextArcNum = ref 0;
fun NewArcNum () = (CPN'inc NextArcNum;!NextArcNum);


(* time addition, begin *)
    fun CreationTime CPN'n = #creationtime(CPN'OGUtils.GetStateRec CPN'n);

    fun CPN'TimeActive CPN'f
	= if with_time() then CPN'f 
	  else (fn CPN'n => CPN'n);
	
    fun CPN'sort_nodes [] = []
      | CPN'sort_nodes (CPN'n::CPN'r)
	= let
	      val CPN'cr_time = CreationTime CPN'n;
	      fun CPN'ins [] = [CPN'n]
		| CPN'ins (CPN'n1::CPN'r1)

		  = if IntInf.> ((CreationTime CPN'n1), CPN'cr_time)

			then (CPN'n::(CPN'n1::CPN'r1))
		    else (CPN'n1::(CPN'ins CPN'r1))
	  in
	      CPN'ins (CPN'sort_nodes CPN'r)
	  end;
	   
    fun CPN'insert_list [] CPN'lst _ = CPN'lst
      | CPN'insert_list (CPN'n1::CPN'r1) CPN'lst CPN'cr_time

	=  if IntInf.> ((CreationTime CPN'n1), CPN'cr_time)

	       then CPN'lst^^(CPN'n1::CPN'r1)
	   else (CPN'n1::(CPN'insert_list CPN'r1 CPN'lst CPN'cr_time));
	       
(* time addition, end *)

 val pq: Node TimePQ.pq = TimePQ.create()
(*  val ToProcess = ref ([]:Node list);*)

 fun StdSelNextNode () : Node = 
     (case TimePQ.is_empty(pq) of
	  true => raise NoSelPossible
	| _ => 
	      (let
		   val (key,nextnode) = TimePQ.deletemin pq
	       in
		   if !CPN'OGBranching.Pred nextnode
		       then
			   nextnode
		   else 
		       (CPN'OGUtils.GetNodeCalcStat
			(CPN'OGUtils.NodeToNodeRec nextnode):=PartProc;
			StdSelNextNode())
	       end)); 
		  

(*
 fun StdSelNextNode():Node
   = if !ToProcess=[] 
      then
         raise NoSelPossible
      else
         let
          val nexthead::nexttail = (!ToProcess)
         in
          ToProcess:= nexttail;
          if !CPN'OGBranching.Pred nexthead
          then
           nexthead
          else (CPN'OGUtils.GetNodeCalcStat(CPN'OGUtils.NodeToNodeRec nexthead):=PartProc;
            StdSelNextNode())
         end(*let*); 
  *)

 val OnlyThisList=ref false;

  val SelectNextNode = StdSelNextNode;
  
  fun AssignList CPN'nl = 
      let
	  val CPN'newnl = (CPN'OGListUtils.mapfilter
			   (
			    ref(fn CPN'n
				=> ((CPN'AvlTree.AvlLookup (CPN'OGNodes,mkst_Node CPN'n);
				     true) handle CPN'AvlTree.ExcAvlLookup => false,
				    CPN'n)),
			    CPN'nl))
      in
	  (app (fn CPN'n => TimePQ.insert(pq,CreationTime CPN'n,CPN'n)) CPN'newnl)
      end;

(*
  fun AssignList CPN'nl (* assigns list of nodes to be processed - omits illegal nodes *)
     = ToProcess := 
        CPN'TimeActive CPN'sort_nodes
                    (CPN'OGListUtils.mapfilter(
                    ref(fn CPN'n
                          => ((CPN'AvlTree.AvlLookup (CPN'OGNodes,mkst_Node CPN'n);
                               true) handle CPN'AvlTree.ExcAvlLookup => false,
                              CPN'n)),
                    CPN'nl));
*)  

  fun AppendList [] = () 
    | AppendList CPN'nl = 
      if (!OnlyThisList)
	  then ()
      else
	  (app (fn CPN'n => TimePQ.insert(pq,(CreationTime CPN'n),CPN'n)) CPN'nl)
(*

  fun AppendList [] = () (* appends list to list of nodes to be processed *)
    | AppendList CPN'nl (* appends list to list of nodes to be processed *)
     = if !OnlyThisList
       then ()
       else 
	   ToProcess:= 
	   (if with_time ()
            then 
		CPN'insert_list (!ToProcess) CPN'nl (CreationTime(hd CPN'nl))
	    else (!ToProcess)^^CPN'nl);

*)               

  fun BuildList () = 
      (CPN'AvlTree.AvlApp(
			  fn ref(CPN'OGnode{calcstat,no,...})
			  => if !calcstat<>FullProc
				 then
				     if !CPN'OGBranching.Pred no
					 then 
					     TimePQ.insert(pq,CreationTime no,no)
				     else calcstat:= PartProc
			     else
				 (), CPN'OGNodes));

(*
  fun BuildList() (* assigns list of all non-full processed nodes to the list to be processed *)
     = (CPN'AvlTree.AvlApp(
        fn ref(CPN'OGnode{calcstat,no,...})
          => if !calcstat<>FullProc
             then
              if !CPN'OGBranching.Pred no
              then ToProcess:= no::(!ToProcess)
              else calcstat:= PartProc
             else
              (),
         CPN'OGNodes);
        ToProcess:=  CPN'TimeActive CPN'sort_nodes (rev(!ToProcess)))
*)

  fun InitList () = (TimePQ.wipe(pq));
(*  fun InitList () = (ToProcess:=[])*)
      
  fun StartSelOnlyOne() = OnlyThisList:=true;

  fun RestoreStdSel() =  OnlyThisList:=false;

end(*struct*); 
