(*
 * Module:       Design/CPN Performance Analysis Library
 *
 * Description:  Accessing measures of statistical variables
 *               in data collections, integer time.
 *
 * Created by:   Bo Lindstrm (blind@daimi.au.dk)   
 *               Lisa Wells   (wells@daimi.au.dk)
 *
 * Date:         January 1999  
 *
 * Design/CPN   Copyright (c) University of Aarhus 1998
 *)

(* 

$Source: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfAccessDCIntTime.sml,v $

$Log: PerfAccessDCIntTime.sml,v $
Revision 1.3.2.1  1999/05/20 06:43:23  wells
Port to sml97

Revision 1.3  1999/02/04 08:57:31  wells
AvrgR corrected to Avgr

Revision 1.2  1999/02/03 12:20:28  wells
CVS Source, Log and Header added


*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfAccessDCIntTime.sml,v 1.3.2.1 1999/05/20 06:43:23 wells Exp $";

(* Integer Time 

  Abbreviations:
   I  = untimed int  statvar.
   R  = untimed real statvar.
   II = int timed statvar with int values.
   RI = int timed statvar with real values.
   DC = Data Collection
*)

structure PerfAccessDC = struct

(* Return value at time 'curTime' by applying 'curFunc' to TSV with 'name' in list 'l' *)
fun getTMeasure CPN'curTime CPN'curFunc CPN'name [] = raise  PerfGenAccessDC.Perf_SVNotFoundInDC CPN'name
  | getTMeasure CPN'curTime CPN'curFunc CPN'name (CPN'curRecord::CPN'rest) = 
    if ( PerfGenAccessDC.recordMatchOne CPN'name CPN'curRecord) then
	CPN'curFunc ((Perf'Env.getSV CPN'curRecord), CPN'curTime)
    else
	getTMeasure CPN'curTime CPN'curFunc CPN'name CPN'rest;


fun accessMeasureT (CPN'name:string) CPN'curTime CPN'curFunc CPN'l =
    if ( PerfGenAccessDC.findName CPN'name CPN'l) then
	getTMeasure CPN'curTime CPN'curFunc CPN'name CPN'l
    else raise  PerfGenAccessDC.Perf_SVNotFoundInDC CPN'name;

(* Access WITHOUT time parameter *)
fun accessMeasureNew (CPN'name:string) CPN'Func1 CPN'List1 CPN'Func2 CPN'List2 =
    if ( PerfGenAccessDC.findName CPN'name CPN'List1) then
	PerfGenAccessDC.getMeasure CPN'Func1 CPN'name CPN'List1
    else 
	if ( PerfGenAccessDC.findName CPN'name CPN'List2) then
	    PerfGenAccessDC.getMeasure CPN'Func2 CPN'name CPN'List2
	else 
	    raise  PerfGenAccessDC.Perf_SVNotFoundInDC CPN'name;

(* Access with time parameter *)
fun accessMeasureTimedNew (CPN'name:string) CPN'curTime CPN'Func1 CPN'List1 CPN'Func2 CPN'List2 =
    if (PerfGenAccessDC.findName CPN'name CPN'List1) then
	getTMeasure CPN'curTime CPN'Func1 CPN'name CPN'List1
    else 
	if (PerfGenAccessDC.findName CPN'name CPN'List2) then
	    getTMeasure CPN'curTime CPN'Func2 CPN'name CPN'List2
	else 
	    raise  PerfGenAccessDC.Perf_SVNotFoundInDC CPN'name;


(* --==>> Integer time specific auxilary functions <<==-- *)

fun accessMeasureIIT (CPN'name:string) (CPN'curTime:int) CPN'curFunc = 
    accessMeasureT CPN'name CPN'curTime CPN'curFunc (Perf'Env.getIIList Perf'Env.Records);

fun accessMeasureRIT (CPN'name:string) (CPN'curTime:int) CPN'curFunc = 
    accessMeasureT CPN'name CPN'curTime CPN'curFunc (Perf'Env.getRIList Perf'Env.Records);

fun accessMeasureII (CPN'name:string) CPN'curFunc = 
    PerfGenAccessDC.accessMeasure CPN'name CPN'curFunc (Perf'Env.getIIList Perf'Env.Records);

fun accessMeasureRI (CPN'name:string) CPN'curFunc = 
    PerfGenAccessDC.accessMeasure CPN'name CPN'curFunc (Perf'Env.getRIList Perf'Env.Records);

(* Find the data collection with the name CPN'name in one of the list
 * containing statvars of type  {I, R, II, RI} and return the value
 * returned by applying the corresponding function {CPN'Func*}.
 * If not found an exception is raised. 
 *)

fun accessDC_I_R_II_RI (CPN'name:string)  CPN'FuncI CPN'FuncR CPN'FuncII CPN'FuncRI=
    accessMeasureNew CPN'name 
    CPN'FuncI (Perf'Env.getIList  Perf'Env.Records)  
    CPN'FuncR (Perf'Env.getRList  Perf'Env.Records)
handle PerfGenAccessDC.Perf_SVNotFoundInDC _ => 
    accessMeasureNew CPN'name 
    CPN'FuncII (Perf'Env.getIIList  Perf'Env.Records)  
    CPN'FuncRI (Perf'Env.getRIList  Perf'Env.Records);

fun accessDC_I_R_II_RI_Timed (CPN'name:string) (CPN'curTime) CPN'FuncI CPN'FuncR CPN'FuncII CPN'FuncRI=
    accessMeasureNew CPN'name 
    CPN'FuncI (Perf'Env.getIList  Perf'Env.Records)  
    CPN'FuncR (Perf'Env.getRList  Perf'Env.Records)
handle PerfGenAccessDC.Perf_SVNotFoundInDC _ => 
    accessMeasureTimedNew CPN'name CPN'curTime 
    CPN'FuncII (Perf'Env.getIIList  Perf'Env.Records)  
    CPN'FuncRI (Perf'Env.getRIList  Perf'Env.Records);

fun accessDC_R_II_RI (CPN'name:string) (CPN'curTime:int) CPN'FuncR CPN'FuncII CPN'FuncRI=
    PerfGenAccessDC.accessMeasureR CPN'name CPN'FuncR 
handle PerfGenAccessDC.Perf_SVNotFoundInDC _ => 
    accessMeasureTimedNew CPN'name CPN'curTime
    CPN'FuncII (Perf'Env.getIIList  Perf'Env.Records)  
    CPN'FuncRI (Perf'Env.getRIList  Perf'Env.Records);

fun accessDC_I_II (CPN'name:string) CPN'FuncI CPN'FuncII=
    PerfGenAccessDC.accessMeasureI CPN'name CPN'FuncI 
handle PerfGenAccessDC.Perf_SVNotFoundInDC _ => 
    accessMeasureII CPN'name CPN'FuncII;

fun accessDC_I_II_Timed (CPN'name:string) CPN'curTime CPN'FuncI CPN'FuncII=
    PerfGenAccessDC.accessMeasureI CPN'name CPN'FuncI 
handle PerfGenAccessDC.Perf_SVNotFoundInDC _ => 
    accessMeasureIIT CPN'name CPN'curTime CPN'FuncII;

fun accessDC_R_RI (CPN'name:string) CPN'FuncR CPN'FuncRI=
    PerfGenAccessDC.accessMeasureR CPN'name CPN'FuncR 
handle PerfGenAccessDC.Perf_SVNotFoundInDC _ => 
    accessMeasureRI CPN'name CPN'FuncRI;

fun accessDC_R_RI_Timed (CPN'name:string) CPN'curTime CPN'FuncR CPN'FuncRI=
    PerfGenAccessDC.accessMeasureR CPN'name CPN'FuncR 
handle PerfGenAccessDC.Perf_SVNotFoundInDC _ => 
    accessMeasureRIT CPN'name CPN'curTime CPN'FuncRI;

fun accessDC_II_RI (CPN'name:string) CPN'FuncII CPN'FuncRI=
    accessMeasureII CPN'name CPN'FuncII 
handle PerfGenAccessDC.Perf_SVNotFoundInDC _ => 
    accessMeasureRI CPN'name CPN'FuncRI;

(* Only apply to timed lists; Used in Interval *)
fun accessTimedDCMeasureTimeParm (CPN'name:string) CPN'curTime CPN'curFuncI CPN'curFuncR = 
    accessMeasureTimedNew CPN'name CPN'curTime 
    CPN'curFuncI (Perf'Env.getIIList Perf'Env.Records)
    CPN'curFuncR (Perf'Env.getRIList Perf'Env.Records);
 
(**************************************************************************)
(* --==>> Access Statistics from Untimed and Integer Timed StatVar <<==-- *)
(**************************************************************************)

(* --==>> Avrg <<==-- *)
fun Avrg (CPN'name:string) = 
    accessDC_I_R_II_RI_Timed CPN'name (time()) CPN'ISV.Avrg CPN'RSV.Avrg CPN'IITSV.Avrg CPN'RITSV.Avrg;

(* --==>> Count <<==-- *)
fun Count (CPN'name:string)= 
    accessDC_I_R_II_RI CPN'name CPN'ISV.Count CPN'RSV.Count CPN'IITSV.Count CPN'RITSV.Count ;

(* --==>> First <<==-- *)
fun FirstI (CPN'name:string)= 
    accessDC_I_II CPN'name CPN'ISV.First CPN'IITSV.First;
fun FirstR (CPN'name:string)= 
    accessDC_R_RI CPN'name CPN'RSV.First CPN'RITSV.First;

(* --==>> Interval <<==-- *)
fun Interval (CPN'name:string) = 
    accessTimedDCMeasureTimeParm CPN'name (time()) CPN'IITSV.Interval CPN'RITSV.Interval;

(* --==>> LastTime <<==-- *)
fun LastTime (CPN'name:string) = 
     accessDC_II_RI CPN'name CPN'IITSV.LastTime CPN'RITSV.LastTime;

(* --==>> Max <<==-- *)
fun MaxI (CPN'name:string)= 
    accessDC_I_II CPN'name CPN'ISV.Max CPN'IITSV.Max;
fun MaxR (CPN'name:string)= 
    accessDC_R_RI CPN'name CPN'RSV.Max CPN'RITSV.Max;

(* --==>> Min <<==-- *)
fun MinI (CPN'name:string)= 
    accessDC_I_II CPN'name CPN'ISV.Min CPN'IITSV.Min;
fun MinR (CPN'name:string)= 
    accessDC_R_RI CPN'name CPN'RSV.Min CPN'RITSV.Min;

(* --==>> SS <<==-- *)
fun SSI (CPN'name:string) = 
    accessDC_I_II_Timed CPN'name (time()) CPN'ISV.SS CPN'IITSV.SS;
fun SSR (CPN'name:string) = 
    accessDC_R_RI_Timed CPN'name (time()) CPN'RSV.SS CPN'RITSV.SS;

(* --==>> SSD <<==-- *)
fun SSD (CPN'name:string) = 
    accessDC_I_R_II_RI_Timed CPN'name (time()) CPN'ISV.SSD CPN'RSV.SSD CPN'IITSV.SSD CPN'RITSV.SSD;

(* --==>> StD <<==-- *)
fun StD (CPN'name:string) = 
    accessDC_I_R_II_RI_Timed CPN'name (time()) CPN'ISV.StD CPN'RSV.StD CPN'IITSV.StD CPN'RITSV.StD;

(* --==>> StartTime <<==-- *)
fun StartTime (CPN'name:string)= 
     accessDC_II_RI CPN'name CPN'IITSV.StartTime CPN'RITSV.StartTime;

(* --==>> SUM <<==-- *)
fun SumI (CPN'name:string) = 
    accessDC_I_II_Timed CPN'name (time()) CPN'ISV.Sum CPN'IITSV.Sum;
fun SumR (CPN'name:string) = 
    accessDC_R_RI_Timed CPN'name (time()) CPN'RSV.Sum CPN'RITSV.Sum;

(* --==>> Value <<==-- *)
fun ValueI (CPN'name:string)= 
    accessDC_I_II CPN'name CPN'ISV.Value CPN'IITSV.Value;
fun ValueR (CPN'name:string)= 
    accessDC_R_RI CPN'name CPN'RSV.Value CPN'RITSV.Value;

(* --==>> Vari <<==-- *)
fun VariR (CPN'name:string) = 
    accessDC_I_R_II_RI_Timed CPN'name (time()) CPN'ISV.Vari CPN'RSV.Vari CPN'IITSV.Vari CPN'RITSV.Vari;

end (* struct *)

