(*
 * Module:       Design/CPN Performance Analysis Library
 *
 * Description:  Accessing measures of statistical variables
 *               in data collections, integer time.
 *
 * Created by:   Bo Lindstrm (blind@daimi.au.dk)   
 *               Lisa Wells   (wells@daimi.au.dk)
 *
 * Date:         January 1999  
 *
 * Design/CPN   Copyright (c) University of Aarhus 1998
 *)

(* 

$Source: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfAccessDCTimed.sml,v $

$Log: PerfAccessDCTimed.sml,v $
Revision 1.2.2.1  1999/05/20 06:43:24  wells
Port to sml97

Revision 1.3  1999/02/04 08:57:31  wells
AvrgR corrected to Avgr

Revision 1.2  1999/02/03 12:20:28  wells
CVS Source, Log and Header added


*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfAccessDCTimed.sml,v 1.2.2.1 1999/05/20 06:43:24 wells Exp $";

(* Integer Time 

  Abbreviations:
   IU  = int untimed   statvar.
   RU  = real untimed  statvar.
   IT = int timed statvar
   RT = int timed statvar 
   DC = Data Collection
*)

structure PerfAccessDC = struct

(* Return value at time 'curTime' by applying 'curFunc' to TSV with 'name' in list 'l' *)
fun getTMeasure CPN'curTime CPN'curFunc CPN'name [] = raise  PerfGenAccessDC.Perf_SVNotFoundInDC CPN'name
  | getTMeasure CPN'curTime CPN'curFunc CPN'name (CPN'curRecord::CPN'rest) = 
    if ( PerfGenAccessDC.recordMatchOne CPN'name CPN'curRecord) then
	CPN'curFunc ((Perf'Env.getSV CPN'curRecord), CPN'curTime)
    else
	getTMeasure CPN'curTime CPN'curFunc CPN'name CPN'rest;


fun accessMeasureT (CPN'name:string) CPN'curTime CPN'curFunc CPN'l =
    if ( PerfGenAccessDC.findName CPN'name CPN'l) then
	getTMeasure CPN'curTime CPN'curFunc CPN'name CPN'l
    else raise  PerfGenAccessDC.Perf_SVNotFoundInDC CPN'name;

(* Access WITHOUT time parameter *)
fun accessMeasureNew (CPN'name:string) CPN'Func1 CPN'List1 CPN'Func2 CPN'List2 =
    if ( PerfGenAccessDC.findName CPN'name CPN'List1) then
	PerfGenAccessDC.getMeasure CPN'Func1 CPN'name CPN'List1
    else 
	if ( PerfGenAccessDC.findName CPN'name CPN'List2) then
	    PerfGenAccessDC.getMeasure CPN'Func2 CPN'name CPN'List2
	else 
	    raise  PerfGenAccessDC.Perf_SVNotFoundInDC CPN'name;

(* Access with time parameter *)
fun accessMeasureTimedNew (CPN'name:string) CPN'curTime CPN'Func1 CPN'List1 CPN'Func2 CPN'List2 =
    if (PerfGenAccessDC.findName CPN'name CPN'List1) then
	getTMeasure CPN'curTime CPN'Func1 CPN'name CPN'List1
    else 
	if (PerfGenAccessDC.findName CPN'name CPN'List2) then
	    getTMeasure CPN'curTime CPN'Func2 CPN'name CPN'List2
	else 
	    raise  PerfGenAccessDC.Perf_SVNotFoundInDC CPN'name;


(* --==>> Infinite integer time specific auxilary functions <<==-- *)

fun accessMeasureIIT (CPN'name:string) (CPN'curTime:IntInf.int) CPN'curFunc = 
    accessMeasureT CPN'name CPN'curTime CPN'curFunc (Perf'Env.getTIList Perf'Env.DataCollectors);

fun accessMeasureRIT (CPN'name:string) (CPN'curTime:IntInf.int) CPN'curFunc = 
    accessMeasureT CPN'name CPN'curTime CPN'curFunc (Perf'Env.getTRList Perf'Env.DataCollectors);

fun accessMeasureII (CPN'name:string) CPN'curFunc = 
    PerfGenAccessDC.accessMeasure CPN'name CPN'curFunc (Perf'Env.getTIList Perf'Env.DataCollectors);

fun accessMeasureRI (CPN'name:string) CPN'curFunc = 
    PerfGenAccessDC.accessMeasure CPN'name CPN'curFunc (Perf'Env.getTRList Perf'Env.DataCollectors);

(* Find the data collection with the name CPN'name in one of the list
 * containing statvars of type  {I, R, II, RI} and return the value
 * returned by applying the corresponding function {CPN'Func*}.
 * If not found an exception is raised. 
 *)

fun accessDC_I_R_II_RI (CPN'name:string)  CPN'FuncI CPN'FuncR CPN'FuncII CPN'FuncRI=
    accessMeasureNew CPN'name 
    CPN'FuncI (Perf'Env.getUIList  Perf'Env.DataCollectors)  
    CPN'FuncR (Perf'Env.getURList  Perf'Env.DataCollectors)
handle PerfGenAccessDC.Perf_SVNotFoundInDC _ => 
    accessMeasureNew CPN'name 
    CPN'FuncII (Perf'Env.getTIList  Perf'Env.DataCollectors)  
    CPN'FuncRI (Perf'Env.getTRList  Perf'Env.DataCollectors);

fun accessDC_I_R_II_RI_Timed (CPN'name:string) (CPN'curTime) CPN'FuncI CPN'FuncR CPN'FuncII CPN'FuncRI=
    accessMeasureNew CPN'name 
    CPN'FuncI (Perf'Env.getUIList  Perf'Env.DataCollectors)  
    CPN'FuncR (Perf'Env.getURList  Perf'Env.DataCollectors)
handle PerfGenAccessDC.Perf_SVNotFoundInDC _ => 
    accessMeasureTimedNew CPN'name CPN'curTime 
    CPN'FuncII (Perf'Env.getTIList  Perf'Env.DataCollectors)  
    CPN'FuncRI (Perf'Env.getTRList  Perf'Env.DataCollectors);

fun accessDC_R_II_RI (CPN'name:string) (CPN'curTime:IntInf.int) CPN'FuncR CPN'FuncII CPN'FuncRI=
    PerfGenAccessDC.accessMeasureR CPN'name CPN'FuncR 
handle PerfGenAccessDC.Perf_SVNotFoundInDC _ => 
    accessMeasureTimedNew CPN'name CPN'curTime
    CPN'FuncII (Perf'Env.getTIList  Perf'Env.DataCollectors)  
    CPN'FuncRI (Perf'Env.getTRList  Perf'Env.DataCollectors);

fun accessDC_I_II (CPN'name:string) CPN'FuncI CPN'FuncII=
    PerfGenAccessDC.accessMeasureI CPN'name CPN'FuncI 
handle PerfGenAccessDC.Perf_SVNotFoundInDC _ => 
    accessMeasureII CPN'name CPN'FuncII;

fun accessDC_I_II_Timed (CPN'name:string) CPN'curTime CPN'FuncI CPN'FuncII=
    PerfGenAccessDC.accessMeasureI CPN'name CPN'FuncI 
handle PerfGenAccessDC.Perf_SVNotFoundInDC _ => 
    accessMeasureIIT CPN'name CPN'curTime CPN'FuncII;

fun accessDC_R_RI (CPN'name:string) CPN'FuncR CPN'FuncRI=
    PerfGenAccessDC.accessMeasureR CPN'name CPN'FuncR 
handle PerfGenAccessDC.Perf_SVNotFoundInDC _ => 
    accessMeasureRI CPN'name CPN'FuncRI;

fun accessDC_R_RI_Timed (CPN'name:string) CPN'curTime CPN'FuncR CPN'FuncRI=
    PerfGenAccessDC.accessMeasureR CPN'name CPN'FuncR 
handle PerfGenAccessDC.Perf_SVNotFoundInDC _ => 
    accessMeasureRIT CPN'name CPN'curTime CPN'FuncRI;

fun accessDC_II_RI (CPN'name:string) CPN'FuncII CPN'FuncRI=
    accessMeasureII CPN'name CPN'FuncII 
handle PerfGenAccessDC.Perf_SVNotFoundInDC _ => 
    accessMeasureRI CPN'name CPN'FuncRI;

(* Only apply to timed lists; Used in Interval *)
fun accessTimedDCMeasureTimeParm (CPN'name:string) CPN'curTime CPN'curFuncI CPN'curFuncR = 
    accessMeasureTimedNew CPN'name CPN'curTime 
    CPN'curFuncI (Perf'Env.getTIList Perf'Env.DataCollectors)
    CPN'curFuncR (Perf'Env.getTRList Perf'Env.DataCollectors);
 
(**************************************************************************)
(* --==>> Access Statistics from Untimed and Integer Timed StatVar <<==-- *)
(**************************************************************************)

(* --==>> Avrg <<==-- *)
fun Avrg (CPN'name:string) = 
    accessDC_I_R_II_RI_Timed CPN'name (time()) CPN'IUSV.Avrg CPN'RUSV.Avrg CPN'ITSV.Avrg CPN'RTSV.Avrg;

(* --==>> Count <<==-- *)
fun Count (CPN'name:string)= 
    accessDC_I_R_II_RI CPN'name CPN'IUSV.Count CPN'RUSV.Count CPN'ITSV.Count CPN'RTSV.Count ;

(* --==>> First <<==-- *)
fun FirstI (CPN'name:string)= 
    accessDC_I_II CPN'name CPN'IUSV.First CPN'ITSV.First;
fun FirstR (CPN'name:string)= 
    accessDC_R_RI CPN'name CPN'RUSV.First CPN'RTSV.First;

(* --==>> Interval <<==-- *)
fun Interval (CPN'name:string) = 
    accessTimedDCMeasureTimeParm CPN'name (time()) CPN'ITSV.Interval CPN'RTSV.Interval;

(* --==>> LastTime <<==-- *)
fun LastTime (CPN'name:string) = 
     accessDC_II_RI CPN'name CPN'ITSV.LastTime CPN'RTSV.LastTime;

(* --==>> Max <<==-- *)
fun MaxI (CPN'name:string)= 
    accessDC_I_II CPN'name CPN'IUSV.Max CPN'ITSV.Max;
fun MaxR (CPN'name:string)= 
    accessDC_R_RI CPN'name CPN'RUSV.Max CPN'RTSV.Max;

(* --==>> Min <<==-- *)
fun MinI (CPN'name:string)= 
    accessDC_I_II CPN'name CPN'IUSV.Min CPN'ITSV.Min;
fun MinR (CPN'name:string)= 
    accessDC_R_RI CPN'name CPN'RUSV.Min CPN'RTSV.Min;

(* --==>> SS <<==-- *)
fun SSI (CPN'name:string) = 
    accessDC_I_II_Timed CPN'name (time()) CPN'IUSV.SS CPN'ITSV.SS;
fun SSR (CPN'name:string) = 
    accessDC_R_RI_Timed CPN'name (time()) CPN'RUSV.SS CPN'RTSV.SS;

(* --==>> SSD <<==-- *)
fun SSD (CPN'name:string) = 
    accessDC_I_R_II_RI_Timed CPN'name (time()) CPN'IUSV.SSD CPN'RUSV.SSD CPN'ITSV.SSD CPN'RTSV.SSD;

(* --==>> StD <<==-- *)
fun StD (CPN'name:string) = 
    accessDC_I_R_II_RI_Timed CPN'name (time()) CPN'IUSV.StD CPN'RUSV.StD CPN'ITSV.StD CPN'RTSV.StD;

(* --==>> StartTime <<==-- *)
fun StartTime (CPN'name:string)= 
     accessDC_II_RI CPN'name CPN'ITSV.StartTime CPN'RTSV.StartTime;

(* --==>> SUM <<==-- *)
fun SumI (CPN'name:string) = 
    accessDC_I_II_Timed CPN'name (time()) CPN'IUSV.Sum CPN'ITSV.Sum;
fun SumR (CPN'name:string) = 
    accessDC_R_RI_Timed CPN'name (time()) CPN'RUSV.Sum CPN'RTSV.Sum;

(* --==>> Value <<==-- *)
fun ValueI (CPN'name:string)= 
    accessDC_I_II CPN'name CPN'IUSV.Value CPN'ITSV.Value;
fun ValueR (CPN'name:string)= 
    accessDC_R_RI CPN'name CPN'RUSV.Value CPN'RTSV.Value;

(* --==>> Vari <<==-- *)
fun VariR (CPN'name:string) = 
    accessDC_I_R_II_RI_Timed CPN'name (time()) CPN'IUSV.Vari CPN'RUSV.Vari CPN'ITSV.Vari CPN'RTSV.Vari;

end (* struct *)

