(*
 * Module:       Design/CPN Performance Analysis Library
 *
 * Description:  Accessing measures of statistical variables
 *               in data collections, time not defined.
 *
 * Created by:   Bo Lindstrm (blind@daimi.au.dk)   
 *               Lisa Wells   (wells@daimi.au.dk)
 *
 * Date:         January 1999  
 *
 * Design/CPN   Copyright (c) University of Aarhus 1998
 *)

(* 

$Source: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfAccessDCUntimed.sml,v $

$Log: PerfAccessDCUntimed.sml,v $
Revision 1.3.2.1  1999/05/20 06:43:25  wells
Port to sml97

Revision 1.3  1999/02/04 08:57:33  wells
AvrgR corrected to Avgr

Revision 1.2  1999/02/03 12:20:29  wells
CVS Source, Log and Header added


*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfAccessDCUntimed.sml,v 1.3.2.1 1999/05/20 06:43:25 wells Exp $";

(* Untimed CPN model

  Abbreviations:
   I  = untimed int  statvar.
   R  = untimed real statvar.
   DC = Data Collection
*)

structure PerfAccessDC = struct


(* --==>> Untimed specific auxilary functions <<==-- *)

(* Find the data collection with the name CPN'name in one of the list
 * containing statvars of type  {I, R} and return the value
 * returned by applying the corresponding function {CPN'Func*}.
 * If not found an exception is raised. 
 *)

fun accessDC_I_R (CPN'name:string)  CPN'FuncI CPN'FuncR =
    PerfGenAccessDC.accessMeasureI CPN'name CPN'FuncI
handle PerfGenAccessDC.Perf_SVNotFoundInDC _ => 
    PerfGenAccessDC.accessMeasureR CPN'name CPN'FuncR;


(**************************************************************************)
(* --==>> Access Statistics from Untimed StatVar <<==-- *)
(**************************************************************************)

(* --==>> Avrg <<==-- *)
fun Avrg (CPN'name:string) = 
    accessDC_I_R CPN'name CPN'IUSV.Avrg CPN'RUSV.Avrg;

(* --==>> Count <<==-- *)
fun Count (CPN'name:string)= 
    accessDC_I_R CPN'name CPN'IUSV.Count CPN'RUSV.Count;

(* --==>> First <<==-- *)
fun FirstI (CPN'name:string)= 
    PerfGenAccessDC.accessMeasureI CPN'name CPN'IUSV.First;
fun FirstR (CPN'name:string)= 
    PerfGenAccessDC.accessMeasureR CPN'name CPN'RUSV.First;

(* --==>> Max <<==-- *)
fun MaxI (CPN'name:string)= 
    PerfGenAccessDC.accessMeasureI CPN'name CPN'IUSV.Max;
fun MaxR (CPN'name:string)= 
    PerfGenAccessDC.accessMeasureR CPN'name CPN'RUSV.Max;

(* --==>> Min <<==-- *)
fun MinI (CPN'name:string)= 
    PerfGenAccessDC.accessMeasureI CPN'name CPN'IUSV.Min;
fun MinR (CPN'name:string)= 
    PerfGenAccessDC.accessMeasureR CPN'name CPN'RUSV.Min;

(* --==>> SS <<==-- *)
fun SSI (CPN'name:string) = 
    PerfGenAccessDC.accessMeasureI CPN'name  CPN'IUSV.SS;
fun SSR (CPN'name:string) = 
    PerfGenAccessDC.accessMeasureR CPN'name  CPN'RUSV.SS;

(* --==>> SSD <<==-- *)
fun SSD (CPN'name:string) = 
    accessDC_I_R CPN'name CPN'IUSV.SSD CPN'RUSV.SSD;

(* --==>> StD <<==-- *)
fun StD (CPN'name:string) = 
    accessDC_I_R CPN'name CPN'IUSV.StD CPN'RUSV.StD;

(* --==>> SUM <<==-- *)
fun SumI (CPN'name:string) = 
    PerfGenAccessDC.accessMeasureI CPN'name CPN'IUSV.Sum;
fun SumR (CPN'name:string) = 
    PerfGenAccessDC.accessMeasureR CPN'name CPN'RUSV.Sum;

(* --==>> Value <<==-- *)
fun ValueI (CPN'name:string)= 
    PerfGenAccessDC.accessMeasureI CPN'name CPN'IUSV.Value;
fun ValueR (CPN'name:string)= 
    PerfGenAccessDC.accessMeasureR CPN'name CPN'RUSV.Value;

(* --==>> Vari <<==-- *)
fun VariR (CPN'name:string) = 
    accessDC_I_R CPN'name CPN'IUSV.Vari CPN'RUSV.Vari;

end (* struct *)

