(*
 * Module:       Bind, AI, BI & TRI functions
 *
 * Description:  Design/CPN OG Tool
 *
 * Created by:   Lars Kristensen (kris@daimi.aau.dk)   
 *
 * Date:         29/09/95   
 *
 * Design/CPN (tm)  Copyright (c) Meta Software Corp 1989--1995
 *)

(* $Source: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfBind.sml,v $

$Log: PerfBind.sml,v $
Revision 1.4  1998/10/26 08:44:37  wells
structure names standardized, CPN' added to parameter names

Revision 1.3  1998/10/06 11:37:07  wells
Initial Version

Revision 1.3  1996/05/14 12:54:16  cpn
Removed use of st_TI since user can now redefine this function

Revision 1.2  1996/02/04 13:44:16  kris
Modified the call to OGSet.stringrepberef to use OGSet.stringreptiref

# Revision 1.1  1995/09/29  15:19:56  kris
# Initial revision
# 

*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfBind.sml,v 1.4 1998/10/26 08:44:37 wells Exp $";


(* generates the code for the Bind structure and the ArcToBE, ArcToTI, BEToTI and *)
(* st_BE/mkst_BE funs *)


structure Perf'BindGen =
struct

local

open CPN'CodeGenUtils;

val FirstConstructor = ref true;

 fun lgen_BindStruct()
    =( code_temp:="structure Bind=struct\n"^
                  "datatype Elem=";
                  
       FirstConstructor:=true;
        
       map
          (fn {trlist = trlist, pgname = pgname, ...}
            => map
                 (fn {trname = "", ...} => ()
                   | {trname = trname, trmlno = trmlno, ...}
                    => code_temp:= !code_temp^
                       (if !FirstConstructor
                        then (FirstConstructor:=false;"")
                        else "|")^
                       pgname^"'"^trname^" of (Inst*"^
                       (CPN'OGBTconvGen.genBEbindtypedef trmlno)^")\n")
                 trlist)
          (!CPN'NetCapture.Net);       
        
       code_temp:= !code_temp^(if !FirstConstructor then "" else "|")^
                   "unnamedBE end;\n");

 
  fun lgen_ArcToBE()
    =( patterncount:=0;
       partfuncount:=0;
       code_temp:="";
       code_storage:= ["fun CPN'ArcToBE0 _ = Bind.unnamedBE;\n"];

       map
          (fn {trlist = trlist, pgname = pgname,...}
            => map
                 (fn {trname = "", ...} => ()
                   | {trname = trname, trmlno = trmlno, ...}
                    => let
                         val internalid = "("^trmlno^",CPN'instno)";
                         val (intpatt,extpatt)
                            = CPN'OGBTconvGen.genBEbindpatterns trmlno 
                        in
                         newpattern("CPN'ArcToBE",
                                    "("^internalid^",ref("^(intpatt)^"))=\n"^
                                    "Bind."^pgname^"'"^trname^"(CPN'instno,"^extpatt^")")
                        end(*let*))
                 trlist)
          (!CPN'NetCapture.Net);
                 
       if !patterncount>0
       then (* finish the last function in code_temp *)
        (code_storage:= !code_storage^^
                        [!code_temp^
                         "|CPN'ArcToBE"^(makestring(!partfuncount+1))^" CPN'x=CPN'ArcToBE"^
                        (makestring(!partfuncount))^" CPN'x;\n"];
         code_temp:="")
       else
        ();
        
       code_storage:= !code_storage^^
         ["fun Perf_ArcToBE CPN'a=CPN'ArcToBE"^
         (makestring(!partfuncount+(if !patterncount=0 then 0 else 1)))^
         "(CPN'a);\n"]);

   fun lgen_BEToTI()
    =( patterncount:=0;
       partfuncount:=0;
       code_temp:="";
       code_storage:= ["fun CPN'BEToTI0 _ = TI.unnamedTI;\n"];

       map
          (fn {trlist = trlist, pgname = pgname,...}
            => map
                 (fn {trname = "", ...} => ()
                   | {trname = trname, trmlno = trmlno, ...}
                    => newpattern("CPN'BEToTI",
                                    "(Bind."^pgname^"'"^trname^
                                    "(CPN'instno,_))=\n"^
                                    "TI."^pgname^"'"^trname^" CPN'instno")
                       )
                 trlist)
          (!CPN'NetCapture.Net);
                 
       if !patterncount>0
       then (* finish the last function in code_temp *)
        (code_storage:= !code_storage^^
                        [!code_temp^
                         "|CPN'BEToTI"^(makestring(!partfuncount+1))^" CPN'x=CPN'BEToTI"^
                        (makestring(!partfuncount))^" CPN'x;\n"];
         code_temp:="")
       else
        ();
        
       code_storage:= !code_storage^^
         ["val Perf_BEToTI =CPN'BEToTI"^
         (makestring(!partfuncount+(if !patterncount=0 then 0 else 1)))^";\n",
         
         "val Perf_ArcToTI = Perf_BEToTI o Perf_ArcToBE;\n"]);
         
       
 fun lgen_st_BE()
    =( patterncount:=0;
       partfuncount:=0;
       code_temp:="";
       code_storage:= ["fun CPN'st_BE0 _ _ = \"*** BE without a name ***\";\n"];

       map
          (fn {trlist = trlist, pgname = pgname,...}
            => map
                 (fn {trname = "", ...} => ()
                   | {trname = trname, trmlno = trmlno, ...}
                    => newpattern_2param("CPN'st_BE",
                                    "CPN'mkst (Bind."^pgname^"'"^trname^
                                    "(CPN'instno,CPN'b))=\n"^
					 "(if CPN'mkst then "^"\"Bind.\"^"^"(CPN'OGBindGen.skiptransno(\""^pgname^"'"^trname^" \"^(makestring CPN'instno)))^\n"^"\"(\"^(makestring CPN'instno)^"^"\",\"^(CPN'st_binding'"^(CPN'OGBTconvGen.findmlnoforbinding trmlno)^" CPN'b)^\")\""^
                                    " else !OGSet.stringrepberef(!OGSet.stringreptiref(\""^pgname^"\",\""^trname^"\",makestring(CPN'instno)),(CPN'st_binding'"^
                                    (CPN'OGBTconvGen.findmlnoforbinding trmlno)^
                                    " CPN'b)))"))
                 trlist)
          (!CPN'NetCapture.Net);
                 
       if !patterncount>0
       then (* finish the last function in code_temp *)
        (code_storage:= !code_storage^^
                        [!code_temp^
                         "|CPN'st_BE"^(makestring(!partfuncount+1))^" CPN'm CPN'x=CPN'st_BE"^
                        (makestring(!partfuncount))^" CPN'm CPN'x;\n"];
         code_temp:="")
       else
        ();
        
       code_storage:= !code_storage^^
         ["val mkst_BE =CPN'st_BE"^
         (makestring(!partfuncount+(if !patterncount=0 then 0 else 1)))^" true;\n",
         "val st_BE =CPN'st_BE"^
         (makestring(!partfuncount+(if !patterncount=0 then 0 else 1)))^" false;"]);      
     
in    

fun gen_BindStruct() =
(lgen_BindStruct();
 [!code_temp]);

fun gen_ArcToBE() =
(lgen_ArcToBE();
 !code_storage);

fun gen_BEToTI() =
(lgen_BEToTI();
 !code_storage);
 
fun gen_st_BE() =
(lgen_st_BE();
 !code_storage);
 
local
 fun skiptospace(CPN'l as (" "::_))=CPN'l
   | skiptospace(CPN'hd::CPN'tl)=skiptospace CPN'tl
in
 fun skiptransno CPN'str = implode(rev(skiptospace(rev(explode CPN'str))))
end(*local*)
 
 
end(*local*)
end(*struct*);
 

