(*
 * Module:       Design/CPN Performance Analysis Library
 *
 * Description:  Creating and inserting new performance analysis
 *               records
 *
 * Created by:   Bo Lindstrm (blind@daimi.aau.dk)   
 *               Lisa Wells   (wells@daimi.aau.dk)
 *
 * Date:         March 1998   
 *
 * Design/CPN   Copyright (c) University of Aarhus 1998
 *)

(* 
 $Source: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfCreateRecords.sml,v $

 $Log: PerfCreateRecords.sml,v $
 Revision 1.3.2.1  1999/05/20 06:43:26  wells
 Port to sml97

 Revision 1.3  1999/02/03 12:24:52  wells
 updateLogFile removed from Data Collectors

 Revision 1.2  1998/10/26 08:53:08  wells
 structure names standardized, CPN' added to parameter names,
 updateLogFile added, more warnings added

 Revision 1.1  1998/10/06 11:02:27  wells
 Initial Version

*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfCreateRecords.sml,v 1.3.2.1 1999/05/20 06:43:26 wells Exp $";

structure PerfCreate = struct

local 
    fun insertTIList (Perf'Env.datacolls CPN'p, CPN'elm) = 
	(#tilist CPN'p) := CPN'elm::(!(#tilist CPN'p));
	
    fun insertTRList (Perf'Env.datacolls CPN'p, CPN'elm) = 
	(#trlist CPN'p) := CPN'elm::(!(#trlist CPN'p));
	
    fun insertUIList (Perf'Env.datacolls CPN'p, CPN'elm) = 
	(#uilist CPN'p) := CPN'elm::(!(#uilist CPN'p));
	
    fun insertURList (Perf'Env.datacolls CPN'p, CPN'elm) = 
	(#urlist CPN'p) := CPN'elm::(!(#urlist CPN'p));
in

fun newIntDataColl {Name = (CPN'newName:string),
		    PredFun = (CPN'pred: CPN'OGrec * Bind.Elem -> bool),
		    ObsFun = (CPN'evalFun: CPN'OGrec * Bind.Elem -> int),
		    DataCollType = CPN'dctype,
		    StatVar = CPN'sv,
		    LogFile = SOME "" 
		    } = DSUI_UserAckMessage("PERFORMANCE WARNING:\nA data collector was not created because the log file name is empty.\n\n  Name:  "^CPN'newName)

  | newIntDataColl {Name = (CPN'newName:string),
		    PredFun = (CPN'pred: CPN'OGrec * Bind.Elem -> bool),
		    ObsFun = (CPN'evalFun: CPN'OGrec * Bind.Elem -> int),
		    DataCollType = timedDC,
		    StatVar = (CPN'sv:bool),
		    LogFile = CPN'fileName
		    } = 
    (case !CPN'env.Timedef 
	     of (CPN'env.timeisint _) => (* Int-time *)
		 (Perf'Env.DataCollDeclared := true;
		  insertTIList(Perf'Env.DataCollectors, 
			      {name        = CPN'newName,
			       counter     = ref 0,
			       updateSV    = CPN'sv,
			       theSV       = TSV'CreateInt(),
			       obsFun      = ref CPN'evalFun,
			       predFun     = ref CPN'pred,
			       logFile     = ref std_out,
			       logFileName = CPN'fileName}))
	   | CPN'env.timenotdef =>
		 DSUI_UserAckMessage("PERFORMANCE WARNING:\nA timed statistical variable cannot be created because the model is untimed.\n\n  Name:  "^CPN'newName))
		 
  | newIntDataColl {Name = (CPN'newName:string),
		    PredFun = (CPN'pred: CPN'OGrec * Bind.Elem -> bool),
		    ObsFun = (CPN'evalFun: CPN'OGrec * Bind.Elem -> int),
		    DataCollType = untimedDC,
		    StatVar = CPN'sv,
		    LogFile = CPN'fileName
		    } = 
		 (Perf'Env.DataCollDeclared := true;
		  insertUIList(Perf'Env.DataCollectors, 
			      {name         = CPN'newName,
			       counter      = ref 0,
			       updateSV     = CPN'sv,
			       theSV        = USV'CreateInt(),
			       obsFun       = ref CPN'evalFun,
			       predFun      = ref CPN'pred,
			       logFile      = ref std_out,
			       logFileName  = CPN'fileName}));
		     
fun newRealDataColl {Name = (CPN'newName:string),
		     PredFun = (CPN'pred: CPN'OGrec * Bind.Elem -> bool),
		     ObsFun = (CPN'evalFun: CPN'OGrec * Bind.Elem -> real),
		     DataCollType = dctype,
		     StatVar = CPN'sv,
		     LogFile = SOME ""
		     } = DSUI_UserAckMessage("PERFORMANCE WARNING:\nA log file cannot be created because the log file name is empty.\n\n  Name:  "^CPN'newName)

  | newRealDataColl {Name = (CPN'newName:string),
		     PredFun = (CPN'pred: CPN'OGrec * Bind.Elem -> bool),
		     ObsFun = (CPN'evalFun: CPN'OGrec * Bind.Elem -> real),
		     DataCollType = timedDC,
		     StatVar = CPN'sv,
		     LogFile = CPN'fileName
		     } =
    (case !CPN'env.Timedef 
	      of (CPN'env.timeisint _) =>(* Int-time *)
		 (Perf'Env.DataCollDeclared := true;
		  insertTRList(Perf'Env.DataCollectors, 
			       {name         = CPN'newName,
				counter      = ref 0,
				updateSV     = CPN'sv,
				theSV        = TSV'CreateReal(),
				obsFun       = ref CPN'evalFun,
				predFun      = ref CPN'pred,
				logFile      = ref std_out,
				logFileName  = CPN'fileName}))
	    | CPN'env.timenotdef =>
		  DSUI_UserAckMessage("PERFORMANCE WARNING: \nA timed statistical variable cannot be created because the model is untimed.\n\n  Name:  "^CPN'newName))
		  
  | newRealDataColl {Name = (CPN'newName:string),
		     PredFun = (CPN'pred: CPN'OGrec * Bind.Elem -> bool),
		     ObsFun = (CPN'evalFun: CPN'OGrec * Bind.Elem -> real),
		     DataCollType = untimedDC,
		     StatVar = CPN'sv,
		     LogFile = CPN'fileName
		     } = 
		 (Perf'Env.DataCollDeclared := true;
		  insertURList(Perf'Env.DataCollectors, 
				   {name         = CPN'newName,
				    counter      = ref 0,
				    updateSV     = CPN'sv,
				    theSV        = USV'CreateReal(),
				    obsFun       = ref CPN'evalFun,
				    predFun      = ref CPN'pred,
				    logFile      = ref std_out,
				    logFileName  = CPN'fileName}));
		      
end; (* local *)

end; (* struct *)
