(*
 * Module:       Design/CPN Performance Analysis Library
 *
 * Description:  Create templates for observation-,
 *               predicate and insert functions.
 *
 * Created by:   Bo Lindstr<F8>m 
 *               Lisa Wells   
 *
 * Date:         September 1998   
 *
 * Design/CPN    Copyright (c) University of Aarhus 1998
 *)

(* 
 $Source: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfCreateTemplate.sml,v $

 $Log: PerfCreateTemplate.sml,v $
 Revision 1.13.2.1  1999/05/20 06:43:27  wells
 Port to sml97

 Revision 1.13  1999/02/03 12:22:06  wells
 LogFileName changed to LogFile

 Revision 1.12  1998/12/09 13:54:38  wells
 parameter added to getPerfBox

 Revision 1.11  1998/11/05 15:11:58  blind
 Error: Needed to select transitions first when wanting to create template.

 Revision 1.10  1998/11/05 14:15:17  blind
 Fixed error: Names of transitions and places was the full name, e.g.,
 "This name".

 Revision 1.9  1998/10/26 08:22:59  wells
 structure name standardized

*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfCreateTemplate.sml,v 1.13.2.1 1999/05/20 06:43:27 wells Exp $";

structure Perf'CreateTemplate = struct

(* Generate a val referring to the marking of each place in the list *)
fun genPlaceLines (CPN'page, CPN'pi:int, []) = ""
  | genPlaceLines (CPN'page, CPN'pi:int, CPN'placeName::CPN'placeNames) = 
    let 
	fun genPlaceLine (CPN'page, CPN'pi:int, CPN'place) = 
	    "     val mark_"^CPN'place^" = PerfMark."^CPN'page^"'"^
	    CPN'place^" "^(makestring CPN'pi)^" net_marking\n";
    in
	(genPlaceLine (CPN'page, CPN'pi, CPN'placeName))^
	(genPlaceLines (CPN'page, CPN'pi, CPN'placeNames))
    end;

(* Generate a function for pattern match on the transitions in the list *)
fun genTransLines (CPN'page, CPN'ti:int, []) = ""
  | genTransLines (CPN'page, CPN'ti:int, (CPN'gid, CPN'transName)::CPN'transNames) =
    let 
	(* Generate string "x1=x1, x2=x2,..., xn=xn" for all 
	 * variables, xi, on arcs to/from the transition with 'gid' *)
	fun genVarMatch CPN'gid =
	    let 
		val varList = ref [];
		val mlno = Cpndb.getMlNum(Cpndb.getDbId(CPN'gid));
		exception mlnofound;	
		fun str2number _ [] = 0
		  | str2number (CPN'n:int)  (CPN'x::CPN'xs) = 
		    (CPN'n*((Char.ord ((CPN'x)))-48)) + 
		    (str2number (CPN'n*10) CPN'xs);
		fun getVarList  CPN'transmlno
		    =(CPN'AvlTree.AvlApp
		      (fn {mlnos=ref mlnoslist,vars=vars}
		       => if CPN'OGListUtils.mem (map ((str2number 1) o rev o sml97explode) mlnoslist) (str2number 1 ((rev o sml97explode) CPN'transmlno)) (* This is a hack due to compiler bug - comparing strings *)
			      then (varList:=vars; raise mlnofound)
			  else (),
			      CPN'OGBTconvGen.varlists) handle mlnofound => ();
		      !varList);
	
		fun genVar [] = ""
		  | genVar (CPN'name::CPN'names) = 
		    CPN'name^"="^CPN'name^
		    (if CPN'names <> nil then "," else "")^
		    (genVar CPN'names);
	    in
		"{"^(genVar (getVarList mlno))^"}"
	    end;

	fun genTransLine (CPN'page, CPN'ti:int, CPN'gid, CPN'trans) =
	    "filterFun (Bind."^CPN'page^"'"^CPN'trans^" ("^(makestring CPN'ti)^", "^(genVarMatch CPN'gid)^")) = \n";
	    
	fun genFirstTransLine(CPN'page, CPN'ti:int, CPN'gid, CPN'transName) =
	"     fun "^(genTransLine (CPN'page, CPN'ti, CPN'gid, CPN'transName));
	
	fun genTransLines (CPN'page, CPN'ti:int, []) = ""
	  | genTransLines (CPN'page, CPN'ti:int, (CPN'gid, CPN'transName)::CPN'transNames) =
	    "       | "^genTransLine(CPN'page, CPN'ti, CPN'gid, CPN'transName)^
	    (genTransLines (CPN'page, CPN'ti:int, CPN'transNames));
    in
	genFirstTransLine (CPN'page, CPN'ti, CPN'gid, CPN'transName)^
	genTransLines (CPN'page, CPN'ti, CPN'transNames)
	
    end;

(* Returns true if gid is a place *)
fun isPlace CPN'gid = 
let fun gidIsPlace {objtyp=Place,objsubtyp=objsubtyp} = true
      | gidIsPlace _ = false;
in
    gidIsPlace (GetCpnInfo CPN'gid)
end;

(* Returns true if gid is a transtion *)
fun isTrans CPN'gid = 
let fun gidIsTrans {objtyp=Trans,objsubtyp=objsubtyp} = true
      | gidIsTrans _ = false;
in
    gidIsTrans (GetCpnInfo CPN'gid)
end;

(* Return list of elements where f applied to the element returns true *)
fun genList CPN'f []  = []
  | genList CPN'f (CPN'gid::CPN'gids) = 
    if (CPN'f CPN'gid) then 
	CPN'gid::(genList CPN'f CPN'gids)
    else genList CPN'f CPN'gids;

fun get2char CPN'c CPN'str = 
    let
	fun get2Char [] CPN'c = []
	  | get2Char (CPN'x::CPN'xs) CPN'c = 
	    if CPN'x=CPN'c then [] else (CPN'x::(get2Char CPN'xs CPN'c)); 
    in
	implode (get2Char (explode CPN'str) CPN'c)
    end;

fun getFirstNamePart CPN'str =
    (get2char " " (get2char "\n" (get2char "\t" CPN'str)));

(* Return a list of the names of the places in the list of gids *)
fun placeNames(CPN'gidList) = map (getFirstNamePart o GetNameText) (genList isPlace CPN'gidList);
	
(* Return a list of pairs (gids, name) of the corresponding
 * transitions referred to by the gids in 'gidList' *)
fun transNames (CPN'gidList) =
    let 
	val transgidList = genList isTrans CPN'gidList;
	val nameList = map (getFirstNamePart o GetNameText) transgidList;
	fun pairList [] _ = []
	  | pairList _ [] = []
	  | pairList (CPN'gid::CPN'gids) (CPN'name::CPN'names) =
	    (CPN'gid, CPN'name)::(pairList CPN'gids CPN'names)
    in
	pairList transgidList nameList
    end;

(* Return a list containing the current group 
 * - current object, if no group selected *)
fun curGroup() =
    let val curGroup = DSStr_GetCurGroup();
    in
	curGroup
    end
handle _ => [DSStr_GetCurObject()];

(* Generates a template for an obs./pred. function with name 'name'.
 * A variable referring to the marking is created for each place in gidList
 * and a match for binding elements for each transtion in gidList *)
fun genFunTemplate(CPN'name, CPN'gidList, CPN'inst) =
    let 
	fun get2Char [] CPN'c = []
	  | get2Char (CPN'x::CPN'xs) CPN'c = 
	    if CPN'x=CPN'c then [] else (CPN'x::(get2Char CPN'xs CPN'c));

	val totalPageName = GetPageName(DSStr_GetCurPage())
	val pageName = implode(get2Char (explode totalPageName) "#");
	val placeNameList = placeNames CPN'gidList;
	val transNameList = transNames CPN'gidList;
	val placeLines = (genPlaceLines(pageName, CPN'inst, placeNameList));
	val transLines = (genTransLines(pageName, CPN'inst, transNameList));
	val header = "fun "^CPN'name^" (net_marking, binding_element) =\n   let\n";

	fun footer (transLines)  =
	    let
		val inStm = "   in\n";
		val funCall = "      filterFun binding_element\n";
		val endStm  = "   end;\n";    
	    in
		if transLines <> "" then
		    "       | filterFun _ =\n"^inStm^funCall^endStm
		else (* No transitions selected *)
		    inStm^endStm
	    end;
    in
	header^
	placeLines^
	transLines^
	(footer (transLines))
    end;

fun underlineSpaces (CPN'str:string) = 
    let
	val strlist = explode CPN'str
	fun underline [] = []
	  | underline (CPN'ch::CPN'rest) = 
	    if (CPN'ch = " ")
		then ("_"::(underline CPN'rest))
	    else (CPN'ch::(underline CPN'rest))
	val newstrlist = underline strlist
    in
	implode newstrlist
    end;

(* Generates a template for a pred. fun (namePred), an obs.fun (nameObs)
 * and an insertfunction. *)
fun genTemplateString(CPN'inst, CPN'name:string, CPN'statistics:bool, 
		      CPN'logfile:bool, CPN'timed:bool) = 
    let
	val gidList = curGroup();
	val ulname = underlineSpaces (CPN'name)
	val predStr = (genFunTemplate (ulname^"Pred", gidList, CPN'inst))
	val obsStr  = (genFunTemplate (ulname^"Obs",  gidList, CPN'inst))
	val logfileStr = if CPN'logfile then ulname^".log" else "";
	val timedStr = if CPN'timed then "timedDC" else "untimedDC"
	val insertStrings = ["PerfCreate.new<Int|Real>DataColl \n",
			     "  {Name = \""^CPN'name^"\",\n",
			     "   PredFun = "^ulname^"Pred,\n",
			     "   ObsFun = "^ulname^"Obs,\n",
			     "   DataCollType = "^timedStr^",\n",
			     "   StatVar = "^Bool.toString(CPN'statistics)^",\n",
			     "   LogFile = "^
			     (if CPN'logfile
				  then "SOME \""^logfileStr^"\""
			      else "NONE")^"};\n\n"
			     ]
	val insertStr = implode insertStrings
    in
	"\n(* -=> "^CPN'name^" <=- *)\n\n"^
	predStr^"\n\n"^
	obsStr^"\n\n"^
	insertStr
    end;
    
fun genTemplate(CPN'statistics:bool, CPN'logfile:bool, CPN'timed:bool,
		CPN'inst, CPN'name) = 
    let 
	val noAuxBox = Perf'Page.noOfAuxBoxesML ();
	val templateString= genTemplateString(CPN'inst, CPN'name, CPN'statistics, CPN'logfile, CPN'timed);
	val perfBoxId =  Perf'Page.getPerfBox(Perf'Page.PerfPageName);
	val pageId = Perf'Page.getPerfPageId(Perf'Page.PerfPageName)
    in
	
	if noAuxBox <= 1 then (* <1; Newly created box not visible yet *)  
	    (DSText_Append ({obj=perfBoxId, text=templateString});
	     DSUI_MakePageVisible{page=pageId, front=true};
	     DSText_SetMode true)
	else
	    DSUI_UserAckMessage("Performance Error:\n\nThere must be exactly one auxiliary box on the "^Perf'Page.PerfPageName^" page.")
    end;
end;

