(*
 * Module:       Design/CPN Performance Analysis Library
 *
 * Description:  Performance analysis environment - datatypes, etc.
 *
 * Created by:   Bo Lindstrm 
 *               Lisa Wells   
 *
 * Date:         March 1998   
 *
 * Design/CPN   Copyright (c) University of Aarhus 1998
 *)

(* 
$Source: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfEnv.sml,v $

$Log: PerfEnv.sml,v $
Revision 1.4.2.1  1999/05/20 06:43:29  wells
Port to sml97

Revision 1.4  1999/02/03 12:24:53  wells
updateLogFile removed from Data Collectors

Revision 1.3  1998/11/13 13:38:58  blind
fun getSV added - used in PerfAccess.sml.

Revision 1.2  1998/10/26 08:48:21  wells
structure names standardized, CPN' added to parameters,
updateLogFile added, unused functions removed

Revision 1.1  1998/10/06 11:02:30  wells
Initial Version

*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfEnv.sml,v 1.4.2.1 1999/05/20 06:43:29 wells Exp $";

structure Perf'Env = struct
   
(* DCtimedInt: Data Collector, timed, integer values *) 
type DCtimedInt =  
    {name: string,
     counter: int ref,
     updateSV: bool,
     theSV: IntInf.int Timedsv,
     obsFun: (CPN'OGrec * Bind.Elem -> int) ref,
     predFun:  (CPN'OGrec * Bind.Elem  -> bool) ref,
     logFile: outstream ref,
     logFileName : string option
     };

(* DCtimedReal: Data Collector, timed, real values *) 
type DCtimedReal  =  
    {name: string,
     counter: int ref,
     updateSV: bool,
     theSV: real Timedsv,
     obsFun: (CPN'OGrec * Bind.Elem -> real) ref ,
     predFun:  (CPN'OGrec * Bind.Elem -> bool) ref,
     logFile: outstream ref,
     logFileName : string option
     };


(* DCuntimedInt: Data Collector, untimed, integer values *) 
type DCuntimedInt  = 
    {name: string,
     counter: int ref,
     updateSV: bool,
     theSV: IntInf.int Statvar,
     obsFun: (CPN'OGrec * Bind.Elem -> int) ref, 
     predFun:  (CPN'OGrec * Bind.Elem -> bool) ref,
     logFile: outstream ref,
     logFileName : string option
     };

(* DCuntimedReal: Data Collector, untimed, real values *) 
type DCuntimedReal  = 
    {name: string,
     counter: int ref,
     updateSV: bool,
     theSV: real Statvar,
     obsFun: (CPN'OGrec * Bind.Elem -> real) ref, 
     predFun:  (CPN'OGrec * Bind.Elem -> bool) ref,
     logFile: outstream ref,
     logFileName : string option
     };

(* -----=====>>>>>  <<<<<=====----- *)

(* datatype for lists of Data Collectors *)
datatype DataColls = datacolls of 
    {tilist:  DCtimedInt list ref,
     trlist:  DCtimedReal list ref,
     uilist:  DCuntimedInt  list ref,
     urlist:  DCuntimedReal  list ref
     };

val DataCollDeclared = ref false;

(* The actual lists of data collectors *)
val DataCollectors = 
     datacolls {tilist = ref ([]: DCtimedInt list),
		trlist = ref ([]: DCtimedReal list),
		uilist = ref ([]: DCuntimedInt  list),
		urlist = ref ([]: DCuntimedReal  list)};

(* -----=====>>>>> <<<<<=====----- *)

fun getTIList (datacolls CPN'p) = !(#tilist CPN'p);
fun getTRList (datacolls CPN'p) = !(#trlist CPN'p);
fun getUIList (datacolls CPN'p) = !(#uilist  CPN'p);
fun getURList  (datacolls CPN'p) = !(#urlist  CPN'p);

fun getSV {name=CPN'name,theSV=CPN's,obsFun=CPN'ob,predFun=CPN'pr,
		 logFile=CPN'l,logFileName=CPN'lf,counter=CPN'c,
		 updateSV=CPN'u} = CPN's;

fun listLookup (anElm, []) = false
  | listLookup (anElm, (elm::rest)) = 
    if anElm=elm then true
    else listLookup (anElm,rest);

end; (* struct *)

datatype DCtype = timedDC | untimedDC ;






