(*
 * Module:          Design/CPN Performance Tool
 * 
 * Description:     Handling of Exceptions in Performance Tool
 * 
 * Created by:      Bo Lindstrom 
 *                  Lisa Wells 
 *                   
 * Date:            November 1998
 *)

(* 
$Source: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfException.sml,v $

$Log: PerfException.sml,v $
Revision 1.4.2.1  1999/05/20 06:43:30  wells
Port to sml97

Revision 1.4  1999/02/04 09:00:01  wells
better reporting for PerfAccess and TSVException

Revision 1.3  1999/02/03 12:22:43  wells
current time added to exception report

Revision 1.2  1998/12/09 13:56:59  wells
function name corrected, step added to output from HandleUpdatingExcn

Revision 1.1  1998/12/09 13:16:34  wells
Initial version


*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfException.sml,v 1.4.2.1 1999/05/20 06:43:30 wells Exp $";

structure Perf'Exception = 
struct

val startStr = "-------------------------------------------------\n"^
               "Design/CPN Performance Tool Exception Report\n\n"^
               "Diagram: "^(DSFile_GetCurrentDiagName())^"\n\n";
    

val endStr = "\n-------------------------------------------------\n\n"

fun prepareBox() = 
    let
	val boxId = Perf'Page.getPerfBox("Perf_Exceptions")
	val curObj = DSStr_SetCurObject (boxId)
	val curPg = DSStr_GetCurPage()
	val show = DSUI_MakePageVisible {page=curPg, front= true}
	val setcolor = DSWtAttr_SetColor {obj=boxId,linecolor=BLACK,
					  fillcolor=BLACK,textcolor=RED}
	val setattr = DSText_SetAttr{obj=boxId,font = Geneva, 
				     size = 12, style = PlainText, 
				     just=LeftJustification}
    in
	boxId
    end;

(* use exnName when using SML97 
fun exn2Str Substring = "Substring"
  | exn2Str Chr = "Chr" 
  | exn2Str Ord = "Ord"
  | exn2Str (Io theStr) = "Io "^theStr 
  | exn2Str Hd = "Hd" 
  | exn2Str Tl = "Tl"
  | exn2Str Nth = "Nth" 
  | exn2Str NthTail = "NthTail"
  | exn2Str Sum = "Sum"
  | exn2Str Diff = "Diff"
  | exn2Str Prod = "Prod"
  | exn2Str Neg = "Neg"
  | exn2Str Quot = "Quot"
  | exn2Str Abs = "Abs"
  | exn2Str Div = "Div"
  | exn2Str Mod = "Mod"
  | exn2Str Overflow = "Overflow"
  | exn2Str Floor = "Floor"
  | exn2Str Sqrt = "Sqrt"
  | exn2Str Exp = "Exp"
  | exn2Str Ln = "Ln"
  | exn2Str Bind = "Bind"
  | exn2Str Match = "Match"
  | exn2Str Interrupt = "Interrupt"
  | exn2Str (Fail theStr) = "Fail "^theStr
  | exn2Str (CPN'userexcp theStr) = "CPN'userexcp "^theStr
  | exn2Str (PerfGenAccessDC.Perf_SVNotFoundInDC theStr) = 
    "Perf_SVNotFoundInDC "^theStr 
  | exn2Str (CPN'IITSV.TSVException theStr) = "TSVException "^theStr
  | exn2Str (CPN'RITSV.TSVException theStr) = "TSVException "^theStr
  | exn2Str (CPN'IRTSV.TSVException theStr) = "TSVException "^theStr
  | exn2Str (CPN'RRTSV.TSVException theStr) = "TSVException "^theStr
  | exn2Str _ = "Other";
*)

fun HandleExcn (CPN'userexcp "Perf Update Excn",_,_) = 
    raise CPN'userexcp "Perf Excp"
  | HandleExcn (theExcn,DCname,CPN'be) = 
    let
	val boxId = prepareBox()
	val outputStr = 
	    "An exception was raised while collecting data. It is likely \n"^
	    "that the exception was raised when evaluating either the \n"^
	    "predicate function or the observation function.\n\n"^
	    "Data collection: "^DCname^
	    "\nStep: "^(makestring (step()))^
	    (if with_time()
		 then "\nTime: "^IntInf.toString(time())
	     else "")^
	    "\nBinding Element: "^(st_BE CPN'be)^
	    "\nThe exception: "^exnName(theExcn)
	val setText = DSText_Put {obj=boxId, 
				  text=startStr^outputStr^endStr};   
    in
	raise CPN'userexcp "Perf Excp"
    end;

fun HandleFileExcn ({cause:exn, function:string, name:string}) =
    let
	val boxId = prepareBox()
	val outputStr = 
	    "An Io exception was raised while manipulating an observation "^
	    "log file. \n\nError: "^function^" "^name
	val setText = DSText_Put {obj=boxId, 
				  text=startStr^outputStr^endStr};   
    in
	raise CPN'userexcp "Perf Obs File Excp"
    end
    
fun HandleReportExcn ({cause:exn, function:string, name:string}) =
    let
	val boxId = prepareBox()
	val outputStr = 
	    "An Io exception was raised while saving a performance report. "^
	    "\n\nError: "^function^" "^name
	val setText = DSText_Put {obj=boxId, 
				  text=startStr^outputStr^endStr}; 
    in
	raise CPN'userexcp "Perf Report Excn"
    end

fun HandleUpdatingExcn (Perf'theExcn,Perf'nameStr) =
    let
	val boxId = prepareBox()
	val outputStr = 
	    "An exception was raised while updating a statistical variable. "^
	    "\nException: "^exnName(Perf'theExcn)^
	    "\nStep: "^(makestring (step()))^
	    (if with_time()
		 then "\nTime: "^IntInf.toString(time())
	     else "")^
	    "\nData Collection: "^Perf'nameStr
	val setText = DSText_Put {obj=boxId, 
				  text=startStr^outputStr^endStr}; 
    in
	raise CPN'userexcp "Perf Update Excn"
    end

end (* struct *)
