(*
 * Module:       Design/CPN Performance Analysis Library
 *
 * Description:  Implements function to pull a state from the simulator
 *
 * Created by:   Lars Michael Kristensen (kris@daimi.aau.dk)   
 *
 * Date:         27/03/98   
 *
 * Design/CPN   Copyright (c) University of Aarhus 1998
 *)

(* $Source: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfFromSimDataGen.sml,v $ *)

(* 
* $Log: PerfFromSimDataGen.sml,v $
* Revision 1.3.2.1  1999/05/20 06:43:31  wells
* Port to sml97
*
* Revision 1.3  1999/01/05 12:46:17  wells
* Hack for unassigned ports corrected
*
* Revision 1.2  1998/10/26 08:45:58  wells
* structure names standardized, OEMark changed to PerfMark
*
* Revision 1.1  1998/10/06 11:02:31  wells
* Initial Version
 *)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfFromSimDataGen.sml,v 1.3.2.1 1999/05/20 06:43:31 wells Exp $";

structure Perf'FromSimDataGen =
struct

local
    
open CPN'CodeGenUtils;

val first = ref false;

fun gen_field {fieldname=fieldname, colset=colset}
   = code_temp:=(!code_temp)^
                    ((if !first
                      then (first:=false;"")
                      else ",\n")^
                      fieldname^"=ref (CPN'sort_ms lt'"^(CPN'env.GetPrimename colset)^"("^
                      (if CPN'env.IsTimed colset then "striptime" else "")^
                      "(!("^
                      (let
                        val CPN'exploded=explode fieldname
                       in
                        (if (List.nth(CPN'exploded,0)="P")andalso
                           (List.nth(CPN'exploded,1)="P")
                           (*hack for: unassigned port => extra de-ref*)
                        then "!"
                        else "")
                       end(*let*))^
		     "(#"^fieldname^" CPN'ps)))))"^
                      (if CPN'env.IsTimed colset
                       then ",\n"^fieldname^"t="^
                             "let val CPN'tl=PerfMark.stripms((CPN'sort_tms lt'"^(CPN'env.GetPrimename colset)^"(!("^
                      (let
                        val CPN'exploded=explode fieldname
                       in
                        (if (List.nth(CPN'exploded,0)="P")andalso
                           (List.nth(CPN'exploded,1)="P")
                           (*hack for: unassigned port => extra de-ref*)
                        then "!"
                        else "")
                       end(*let*))^
			   "(#"^fieldname^" CPN'ps)))))in\n"^
                             "ref CPN'tl end"
                       else ""));

fun gen_fusfield {fieldname=fieldname, colset=colset}
   = code_temp:=(!code_temp)^
                    ((if !first
                      then (first:=false;"")
                      else ",\n")^
                      fieldname^"=ref (CPN'sort_ms lt'"^(CPN'env.GetPrimename colset)^"("^
                      (if CPN'env.IsTimed colset then "striptime" else "")^
                      "(!(#cur CPN'FS"^(implode(tl(explode fieldname)))^"))))"^
                      (if CPN'env.IsTimed colset
                       then ",\n"^fieldname^"t="^
                             "let val CPN'tl=PerfMark.stripms(CPN'sort_tms lt'"^(CPN'env.GetPrimename colset)^"((!(#cur CPN'FS"^(implode(tl(explode fieldname)))^"))))in\n"^
                             "ref CPN'tl end"
                       else ""));
                       
fun gen_localstore ("pgglobfus",ref fieldlist)
   = (code_temp:=("fun CPN'OEstore_pgglobfus()\n"^
          "= let\n"^
              "val pgglobfusrec\n"^
                 "=CPN'OGLocalpgglobfus\n{");
          
      first:=true;
      
      map gen_fusfield fieldlist;
        
      code_temp:=(!code_temp)^("};\n"^
       "val (newref,_)\n"^
          "= (ref pgglobfusrec,false)\n"^
             "in newref end;\n");
       
       code_storage:=(!code_temp)::(!code_storage))

  | gen_localstore (recname,ref fieldlist)
   = (code_temp:=("fun CPN'OEstore_"^recname^
        "(CPN'ps:CPN'PT"^(implode(tl(tl(explode(recname)))))^")\n"^
          "= let\n"^
              "val "^recname^"rec\n"^
                 "=CPN'OGLocal"^recname^"\n{");
          
      first:=true;
      
      map gen_field fieldlist;
        
      code_temp:=(!code_temp)^("};\n"^
       "val (newref,_)\n"^
          "= ("^recname^"rec,false)\n"^
             "in newref end;\n");
       
       code_storage:=(!code_temp)::(!code_storage))

in

fun gen () =

let
 val allOGrecs=CPN'AvlTree.AvlNew(): unit CPN'AvlTree.avltree;
 val _ = map
           (fn {instlist = instlist,pgmlno = pgmlno,...}
              =>(CPN'AvlTree.AvlLookup(CPN'OGIdsGen.DataRecFields, "pg"^pgmlno);
                   (* check that record is used - otherwise an exception is rasied *)
                 map
                  (fn {instno=instno,...}
                     => CPN'AvlTree.AvlInsert allOGrecs("pg"^pgmlno^"'"^instno,()))                       
                  instlist) handle CPN'AvlTree.ExcAvlLookup => [])
           (!CPN'NetCapture.Net);
in
 
 code_storage:=[];
 code_storage2:=[];
  
 CPN'AvlTree.AvlAppKey(gen_localstore, CPN'OGIdsGen.DataRecFields);

 code_temp:="";
 first:=true; 
 
 map
  (fn {trlist = trlist, pgname = pgname, pgno = pgno, instlist = instlist,
       pgmlno = pgmlno,...}
     => map
          (fn {surrports=surrports, surrpgglobfus=surrpgglobfus, trmlno=trmlno, ...}
             => map
                 (fn {instmlno=instmlno,instno=instno,instdbid=instdbid}
                    => let
                        val internalid = "("^trmlno^","^instno^")";
                        val funpostfix = trmlno^"'"^instno;
                        val pgglobfuschng = ref surrpgglobfus;
                        val unchngrecs = CPN'AvlTree.AvlCopy allOGrecs;
                        fun reg_chng "pgglobfus"
                           = pgglobfuschng:=true
                          | reg_chng recname
                           = CPN'AvlTree.AvlDelete unchngrecs recname
                              handle CPN'AvlTree.ExcAvlDelete =>() (*already removed*)
                       in
                        
                        (CPN'AvlTree.AvlLookup(CPN'OGIdsGen.DataRecFields, "pg"^pgmlno);
                          (* check that record is used - otherwise an exception is rasied *)
                         reg_chng("pg"^pgmlno^"'"^instno))
                         handle CPN'AvlTree.ExcAvlLookup => ();
                           
                        map
                         (fn portdbid
                            => reg_chng(
                                #OGdatarec
                                 (CPN'AvlTree.AvlLookup
                                  (CPN'OGIdsGen.PlaceInsts,
                                   Cpndb.makestringdbid(portdbid)^" "^
                                   (Cpndb.makestringdbid(instdbid))))))
                          surrports;
                          
                        code_temp2:=
                         "fun CPN'OEstore_state"^funpostfix^
                          "(srcnoderef as ref(CPN'OGnode{state=ref(CPN'OGState srcrec),...}))\n"^
                          "=CPN'OGState\n{"; 
                         
                        
                        map
                         (fn {instlist = instlist,pgmlno = pgmlno,...}
                            =>(CPN'AvlTree.AvlLookup(CPN'OGIdsGen.DataRecFields, "pg"^pgmlno);
                                (* check that record is used - otherwise an exception is rasied *)
                               map
                               (fn {instno=instno,instmlno=instmlno,...}
                                  => code_temp2:=(!code_temp2)^"pg"^pgmlno^"'"^instno^"= "^
                                      ((CPN'AvlTree.AvlLookup(unchngrecs,"pg"^pgmlno^"'"^instno);
                                        "#"^"pg"^pgmlno^"'"^instno^" srcrec,\n")
                                        handle CPN'AvlTree.ExcAvlLookup
                                          =>"ref (CPN'OEstore_pg"^pgmlno^" CPN'PS"^instmlno^"),\n"))
                               instlist) handle CPN'AvlTree.ExcAvlLookup => [])
                         (!CPN'NetCapture.Net);
                          
                                               
                        (CPN'AvlTree.AvlLookup(CPN'OGIdsGen.DataRecFields,"pgglobfus");
                         if !pgglobfuschng
                         then code_temp2:=(!code_temp2)^
                              "pgglobfus= CPN'OEstore_pgglobfus(),\n"
                         else code_temp2:=(!code_temp2)^"pgglobfus= #pgglobfus srcrec,\n")
                        handle CPN'AvlTree.ExcAvlLookup =>();
                        
                        if ((!CPN'env.Timedef)<>CPN'env.timenotdef)andalso(with_time())
                        then
                         code_temp2:=(!code_temp2)^"creationtime=time(),\n"
                        else
                         ();
             
                        code_temp2:=(!code_temp2)^"owner = ref srcnoderef};\n";

                        code_storage2:=(!code_temp2)::(!code_storage2);
                        
                        code_temp:=(!code_temp)^(
                         (if !first then (first:=false;"fun ") else "|")^
                         "CPN'OEstore_state(srcnoderef,"^internalid^")="^
                         "CPN'OEstore_state"^funpostfix^" srcnoderef\n")
                                            
			           end(*let*))
                 instlist)
          trlist)
 (!CPN'NetCapture.Net);
 
 
 code_temp:= (!code_temp)^";\n";
  
 !code_storage^^(!code_storage2)^^[!code_temp]
 
end (*let*)
  
end(*local*)

end(*struct*);

