(*
 * Module:       Design/CPN Performance Analysis Library
 *
 * Description:  Accessing measures of statistical variables
 *               in data collections.
 *
 * Created by:   Bo Lindstrm (blind@daimi.au.dk)   
 *               Lisa Wells   (wells@daimi.au.dk)
 *
 * Date:         August 1998   
 *
 * Design/CPN   Copyright (c) University of Aarhus 1998
 *)

(* 

$Source: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfGenAccessDC.sml,v $

$Log: PerfGenAccessDC.sml,v $
Revision 1.2.2.1  1999/05/20 06:43:31  wells
Port to sml97

Revision 1.2  1999/02/03 12:24:53  wells
updateLogFile removed from Data Collectors


*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfGenAccessDC.sml,v 1.2.2.1 1999/05/20 06:43:31 wells Exp $";

structure PerfGenAccessDC = struct
exception Perf_SVNotFoundInDC of string;

(* --==>> Auxilary functions <<==--*)

fun recordMatchOne (CPN'name1:string)
    ({name=CPN'name,theSV=CPN's,obsFun=CPN'ob,predFun=CPN'pr,
      logFile=CPN'l,logFileName=CPN'lf,counter=CPN'c,updateSV=CPN'u}) =
    (CPN'name=CPN'name1);

fun findName (CPN'name1:string) [] = false 
| findName (CPN'name1:string)
    ({name=CPN'name,theSV=CPN's,obsFun=CPN'ob,predFun=CPN'pr,
      logFile=CPN'l,logFileName=CPN'lf,counter=CPN'c,
      updateSV=CPN'u}::CPN'rest)= 
if CPN'name1=CPN'name then true else findName CPN'name1 CPN'rest;

fun getMeasure CPN'curFunc CPN'name [] = raise Perf_SVNotFoundInDC CPN'name
  | getMeasure CPN'curFunc CPN'name (CPN'curRecord::CPN'rest) = 
    if (recordMatchOne CPN'name CPN'curRecord) then
	CPN'curFunc (Perf'Env.getSV CPN'curRecord)
    else
	getMeasure CPN'curFunc CPN'name CPN'rest;

(* Return value by applying 'curFunc' to TSV with 'name' in list 'l' *)
fun accessMeasure (CPN'name:string) CPN'curFunc CPN'l =
    if (findName CPN'name CPN'l) then
	getMeasure CPN'curFunc CPN'name CPN'l
    else raise Perf_SVNotFoundInDC CPN'name

(* --==>> Supply correct list to function accessMeasure? <<==-- *)

fun accessMeasureI (CPN'name:string) CPN'curFunc = 
    accessMeasure CPN'name CPN'curFunc (Perf'Env.getUIList Perf'Env.DataCollectors);

fun accessMeasureR (CPN'name:string) CPN'curFunc = 
    accessMeasure CPN'name CPN'curFunc (Perf'Env.getURList Perf'Env.DataCollectors);

end; (* struct *)

