(*
 * Module:       Design/CPN Performance Analysis Library
 *
 * Description:  Initializing statistical variables in
 *               performance analysis records
 *
 * Created by:   Bo Lindstrm (blind@daimi.aau.dk)   
 *               Lisa Wells   (wells@daimi.aau.dk)
 *
 * Date:         March 1998   
 *
 * Design/CPN   Copyright (c) University of Aarhus 1998
 *)

(* 
$Source: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfInitRecords.sml,v $

$Log: PerfInitRecords.sml,v $
Revision 1.3.2.1  1999/05/20 06:43:32  wells
Port to sml97

Revision 1.3  1999/02/03 12:24:54  wells
updateLogFile removed from Data Collectors

Revision 1.2  1998/10/26 08:52:11  wells
structure names standardized, CPN' added to parameter names,
updateLogFile added

Revision 1.1  1998/10/06 11:02:33  wells
Initial Version

*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfInitRecords.sml,v 1.3.2.1 1999/05/20 06:43:32 wells Exp $";

structure Perf'InitDC = struct

fun initDC ([],CPN'initFunc) = ()
  | initDC ((CPN'rec as {name=CPN'nm,counter=CPN'c,updateSV=CPN'u,
			     theSV=CPN'anSV,obsFun=CPN'ob,
			     predFun=CPN'pr,logFile=CPN'lf,
			     logFileName=CPN'lfn})::CPN'ps,CPN'initFunc) =
    ((CPN'initFunc CPN'anSV);
     (#counter CPN'rec) := 0;
     (case CPN'lfn of 
	  NONE => ()
	| SOME CPN'fileName =>  let
				    val outfile = open_out(CPN'fileName)
				in
				    close_out(outfile)
				end);
      (initDC (CPN'ps,CPN'initFunc)))

(* Initializes all Data Collectors *)
fun initAll () =
  ((initDC ((Perf'Env.getTIList Perf'Env.DataCollectors),CPN'ITSV.Init));
   (initDC ((Perf'Env.getTRList Perf'Env.DataCollectors),CPN'RTSV.Init));
   (initDC ((Perf'Env.getURList Perf'Env.DataCollectors),CPN'RUSV.Init));
   (initDC ((Perf'Env.getUIList Perf'Env.DataCollectors),CPN'IUSV.Init)));


(* following used when the user selects one (or more) names from
 a selection list in the GUI. The name(s) that are returned correspond
 to statistical variables that are to be initialized. *)

fun initByName (CPN'names:string list,[],CPN'initFunc) = ()
  | initByName (CPN'names,
		      (CPN'rec as 
		       {name=CPN'nm,counter=CPN'c,updateSV=CPN'u,
			theSV=CPN'anSV,obsFun=CPN'ob,predFun=CPN'pr,
			logFile=CPN'lf,logFileName=CPN'lfn})::CPN'ps,CPN'initFunc) =
    let 
	val toBeInitialized = Perf'Env.listLookup (CPN'nm,CPN'names)  
    in
	if toBeInitialized
	    then ((CPN'initFunc CPN'anSV); 
		  (#counter CPN'rec) := 0;
		  (case CPN'lfn of
		       NONE => ()
		     | SOME CPN'fileName => 
			   let
			       val outfile = open_out(CPN'fileName)
			   in
			       close_out(outfile)
			   end);
		       (initByName (CPN'names,CPN'ps,CPN'initFunc)))
	else  initByName (CPN'names,CPN'ps,CPN'initFunc)
    end;

fun initDCByName CPN'name =
    ((initByName 
      (CPN'name,(Perf'Env.getTIList Perf'Env.DataCollectors),CPN'ITSV.Init));
     (initByName 
      (CPN'name,(Perf'Env.getTRList Perf'Env.DataCollectors),CPN'RTSV.Init));
     (initByName 
      (CPN'name,(Perf'Env.getURList Perf'Env.DataCollectors),CPN'RUSV.Init));
     (initByName 
      (CPN'name,(Perf'Env.getUIList Perf'Env.DataCollectors),CPN'IUSV.Init)));
    
end; (* struct *)
