(*
 * Module:       Design/CPN Performance Analysis Library
 *
 * Description:  File management for performance log files
 *
 * Created by:   Bo Lindstrm (blind@daimi.aau.dk)   
 *               Lisa Wells   (wells@daimi.aau.dk)
 *
 * Date:         March 1998   
 *
 * Design/CPN   Copyright (c) University of Aarhus 1998
 *)

(* 
$Source: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfLogFileManagement.sml,v $

$Log: PerfLogFileManagement.sml,v $
Revision 1.4.2.1  1999/05/20 06:43:33  wells
Port to sml97

Revision 1.4  1999/02/03 12:24:54  wells
updateLogFile removed from Data Collectors

Revision 1.3  1998/12/09 13:20:18  wells
Exception handling added, functions for flushing files added

Revision 1.2  1998/10/26 08:52:12  wells
structure names standardized, CPN' added to parameter names,
updateLogFile added

Revision 1.1  1998/10/06 11:02:34  wells
Initial Version

*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfLogFileManagement.sml,v 1.4.2.1 1999/05/20 06:43:33 wells Exp $";

structure Perf'LogFileManagement = struct

fun openLogFile ({name=CPN'nm,theSV=CPN's,obsFun=CPN'ob,predFun=CPN'pr,
		  logFile=CPN'lf,logFileName=CPN'lfn,counter=CPN'c,
		  updateSV=CPN'u}) = 
    case CPN'lfn of
	NONE => ()
      | SOME CPN'fileName =>  
	    (CPN'lf := open_out(CPN'fileName)
	     handle IO.Io exnStr => Perf'Exception.HandleFileExcn (exnStr));

	
fun openAll () =
    (map openLogFile  (Perf'Env.getUIList Perf'Env.DataCollectors);    
     map openLogFile  (Perf'Env.getURList Perf'Env.DataCollectors); 
     map openLogFile  (Perf'Env.getTIList Perf'Env.DataCollectors); 
     map openLogFile  (Perf'Env.getTRList Perf'Env.DataCollectors));

    
fun openAppendLogFile ({name=CPN'nm,theSV=CPN's,obsFun=CPN'ob,predFun=CPN'pr,
			logFile=CPN'lf,logFileName=CPN'lfn,counter=CPN'c,
			updateSV=CPN'u}) = 
    case CPN'lfn of
	NONE => ()
      | SOME CPN'fileName =>  
	    (CPN'lf := open_append(CPN'fileName)
	     handle IO.Io exnStr => Perf'Exception.HandleFileExcn (exnStr));

fun openAppendAll () =
    (map openAppendLogFile  (Perf'Env.getUIList Perf'Env.DataCollectors);    
     map openAppendLogFile  (Perf'Env.getURList Perf'Env.DataCollectors); 
     map openAppendLogFile  (Perf'Env.getTIList Perf'Env.DataCollectors); 
     map openAppendLogFile  (Perf'Env.getTRList Perf'Env.DataCollectors));

    
fun closeLogFile ({name=CPN'nm,theSV=CPN's,obsFun=CPN'ob,predFun=CPN'pr,
		   logFile=CPN'lf,logFileName=(CPN'lfn:string option),
		   counter=CPN'c,updateSV=CPN'u}) = 
    case CPN'lfn of
	NONE => ()
      | SOME CPN'fileName =>  
	    (close_out(!CPN'lf)
	     handle IO.Io exnStr => Perf'Exception.HandleFileExcn (exnStr));
 
fun closeAll() =
    (map closeLogFile  (Perf'Env.getUIList Perf'Env.DataCollectors);    
     map closeLogFile  (Perf'Env.getURList Perf'Env.DataCollectors); 
     map closeLogFile  (Perf'Env.getTIList Perf'Env.DataCollectors); 
     map closeLogFile  (Perf'Env.getTRList Perf'Env.DataCollectors));


val flushFilesNow = ref (fn () => (step() mod 500) = 0);

fun setFileFlush CPN'newFlushFun = (flushFilesNow := CPN'newFlushFun)

fun flushLogFile ({name=CPN'nm,theSV=CPN's,obsFun=CPN'ob,predFun=CPN'pr,
		   logFile=CPN'lf,logFileName=(CPN'lfn:string option),
		   counter=CPN'c,updateSV=CPN'u}) = 
    case CPN'lfn of
	NONE => ()
      | SOME CPN'fileName =>  
	    (flush_out(!CPN'lf)
	     handle IO.Io exnStr => Perf'Exception.HandleFileExcn (exnStr));

fun flushAll() = 
    (map flushLogFile (Perf'Env.getUIList Perf'Env.DataCollectors);    
     map flushLogFile  (Perf'Env.getURList Perf'Env.DataCollectors); 
     map flushLogFile  (Perf'Env.getTIList Perf'Env.DataCollectors); 
     map flushLogFile  (Perf'Env.getTRList Perf'Env.DataCollectors));

end; (* struct *)
