(*
 * Module:       For generation of the PerfMark function
 *
 * Description:  Design/CPN OG Tool
 *
 * Created by:   Lars Kristensen (kris@daimi.aau.dk)   
 *
 * Date:         02/01/98   
 *
 * Design/CPN (tm)  Copyright (c) University of Aarhus, Denmark 1998
 *)

(* $Source: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfMarkGen.sml,v $

$Log: PerfMarkGen.sml,v $
Revision 1.2  1998/10/26 08:44:38  wells
structure names standardized, CPN' added to parameter names

Revision 1.1  1998/10/09 11:42:27  blind
Initial version.

Revision 1.1  1998/01/03 13:07:59  kris
Initial Revision


*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfMarkGen.sml,v 1.2 1998/10/26 08:44:38 wells Exp $";

(* structure for generations of the PerfMark function *)


structure Perf'MarkGen =
struct

local
    
open CPN'CodeGenUtils;
 
fun extractfieldname ("'"::tl) fieldname
   = fieldname
  | extractfieldname (hd::tl) fieldname
   = extractfieldname tl (fieldname^hd)
  | extractfieldname nil fieldname
   = fieldname;

fun  gen_placeinstmark({OGdatarec=OGdatarec, OGdatarecfield=OGdatarecfield,
                        pli=pli,...}:CPN'OGIdsGen.PlaceInstDecr,CPN'csistimed)
   = "{"^OGdatarec^"=ref(CPN'OGLocal"^
     (extractfieldname (explode OGdatarec) "")^
     "{"^OGdatarecfield^"=ref CPN'ms,"^
        (if CPN'csistimed
         then (OGdatarecfield^"t=ref CPN'tl,")
         else "")^"...}),...}\n";

val firstinst = ref false;

in


 fun gen()
    = (code_temp:="structure PerfMark=struct\n"^
                  (if (!CPN'env.Timedef)<>CPN'env.timenotdef then
                  "fun CPN'tmscomb (empty,_) = tempty\n"^
                    "| CPN'tmscomb ((CPN'coef,CPN'col)!!CPN'ms,CPN'tl)\n"^
                     "= let val (CPN'rtl,CPN'ftl)=CPN'OGListUtils.split(CPN'coef,CPN'tl,[])in\n"^
                        "(CPN'coef,CPN'col,CPN'ftl)!!!(CPN'tmscomb(CPN'ms,CPN'rtl))end;\n"^
                  "fun stripms tempty=[]\n"^
                    "|stripms((_,_,CPN'tl)!!!ms1)=CPN'tl^^(stripms ms1);\n"

                   else "");
                         
       map
         (fn {pllist = pllist, pgname = pgname, instlist = instlist,...}
            => CPN'AvlTree.AvlApp
                 (fn {plname="",...} => ()
                   | {plname=plname, pldbid=pldbid,...}
		            => (firstinst:=true;
		                map
		                (fn {instno=instno,instdbid=instdbid,...}
		                   => let
		                        val CPN'p=pgname^"'"^plname;
		                        val CPN'plikey=Cpndb.makestringdbid(pldbid)^" "^(Cpndb.makestringdbid(instdbid));
		                        val CPN'pliinfo=(CPN'AvlTree.AvlLookup(CPN'OGIdsGen.PlaceInsts,CPN'plikey));
		                        val CPN'csistimed= CPN'env.IsTimed (#colset CPN'pliinfo)
		                       in
		                         code_temp:= !code_temp^
		                          (if !firstinst then (firstinst:=false;"fun ") else "|")^
		                          CPN'p^"("^instno^":Inst)(CPN'n:CPN'OGrec)=\n"^"let val"^
		                          " CPN'OGState"^(gen_placeinstmark (CPN'pliinfo,CPN'csistimed))^
		                          "=CPN'n in\n"^
		                            (if CPN'csistimed 
		                             then "CPN'tmscomb(CPN'ms,CPN'tl)"
		                             else "CPN'ms")^
		                            " end"
		                       end(*let*))
		                   instlist;
		                   code_temp:= !code_temp^(if !firstinst then "fun " else "|")^
		                                pgname^"'"^plname^" _ _ =raise IllegalId;\n"
		                  ),
		                 pllist))
          (!CPN'NetCapture.Net);       
        
       [!code_temp^"end(*struct*);\n"]);
       
       
  fun gen_st()
    = (code_temp:="structure st_OEMark=struct\n"^
                  "val format=ref(fn(\"\",\"\")=>\"\");\n";
                         
       map
         (fn {pllist = pllist, pgname = pgname, instlist = CPN'hdinst::_,...}
            => CPN'AvlTree.AvlApp
                 (fn {plname="",...} => ()
                   | {plname=plname, pldbid=pldbid,...}
		            => let
		                 val CPN'p=pgname^"'"^plname;
		                 val CPN'plikey=Cpndb.makestringdbid(pldbid)^" "^(Cpndb.makestringdbid(#instdbid CPN'hdinst));
		                 val CPN'colset= (#colset(CPN'AvlTree.AvlLookup(CPN'OGIdsGen.PlaceInsts,CPN'plikey)));
		                 val CPN'csistimed= CPN'env.IsTimed CPN'colset;
		                 val CPN'primecol= CPN'env.GetPrimename CPN'colset
		                in
		                 code_temp:= !code_temp^
		                              "fun "^CPN'p^"(CPN'i:Inst) CPN'n=\n"^"!OGSet.stringrepmarkref(!OGSet.stringreppiref(\""^pgname^"\",\""^plname^"\",makestring(CPN'i)),mkst_"^(if CPN'csistimed then "t"
 else "")^"ms'"^CPN'primecol^
                                              "(Mark."^CPN'p^" CPN'i CPN'n))"
		                 end(*let*),
		                 pllist))
          (!CPN'NetCapture.Net);       
        
       [!code_temp^"end(*struct*);\n"])       
  
end(*local*)

end(*struct*);

 
