(*
 * Module:       Design/CPN Performance Analysis Library
 *
 * Description:  Saving standard report for untimed and timed
 *               models with INTEGER time
 *
 * Created by:   Bo Lindstrm 
 *               Lisa Wells   
 *
 * Date:         January 1999
 *
 * Design/CPN   Copyright (c) University of Aarhus 1999
 *)

(* 
$Source: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfPrint.sml,v $

$Log: PerfPrint.sml,v $
Revision 1.1.2.2  2000/08/31 09:04:48  blind
Fixing bug when selecting HTML format

Revision 1.1.2.1  1999/05/20 06:43:34  wells
Port to sml97

Revision 1.1  1999/02/03 12:24:03  wells
Initial Version


*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfPrint.sml,v 1.1.2.2 2000/08/31 09:04:48 blind Exp $";

structure Perf'Print = struct

fun allRecordsASCII () = 
    let
	val TimePrint = 
	    (case (!CPN'env.Timedef) of
		 CPN'env.timenotdef => []
	       | _ => ([Perf'Report.TSVReportHeader ()]^^
		       (Perf'StatsIntTime.ASCII_String())^^
		       [(Perf'Report.makeHeaderLine (!Perf'Report.printTAttr))]))
	    
    in
	TimePrint^^
	[Perf'Report.SVReportHeader ()]^^
	(Perf'Report.printListUntimed (Perf'Env.getUIList Perf'Env.DataCollectors)  CPN'IUSV.Print)^^ 
	(Perf'Report.printListUntimed (Perf'Env.getURList Perf'Env.DataCollectors)  CPN'RUSV.Print)^^
	[(Perf'Report.makeHeaderLine (!Perf'Report.printUAttr))]^^
	[Perf'Report.currentStep()]^^[Perf'Report.currentTime()]
	
    end; 

fun allRecordsLaTeX () = 
    let
	val TSVHeader = Perf'Report.TSVReportHeaderElements()
	val SVHeader = Perf'Report.SVReportHeaderElements()

	val noOfTimedElms = length TSVHeader
	val noOfUntimedElms = length SVHeader

	fun addLaTeXSeparators (CPN'elm::[]) = CPN'elm^" \\\\\n"
	  | addLaTeXSeparators (CPN'elm::rest) = CPN'elm^" & "^(addLaTeXSeparators rest)

	val timedElementsList = 
	    (case (!CPN'env.Timedef) of
		 (CPN'env.timeisint _)  => Perf'StatsIntTime.ListsOfStats()
	       | _ => [[""]])

	val timedElementsString = 
	    (case (!CPN'env.Timedef) of
		 CPN'env.timenotdef => ""
	       | _ => implode (map addLaTeXSeparators timedElementsList))
	
	val untimedElementsList = 
	    (Perf'Report.getUntimedTableElements (Perf'Env.getUIList Perf'Env.DataCollectors) CPN'IUSV.StatsAsStrings)^^
	    (Perf'Report.getUntimedTableElements (Perf'Env.getURList Perf'Env.DataCollectors) CPN'RUSV.StatsAsStrings)

	val untimedElementsString = implode (map addLaTeXSeparators untimedElementsList)

	fun rightJustified 0 = ""
	  | rightJustified n = if n < 0 then "" 
			       else ("r"^(rightJustified (n-1)))

    in
	"%LaTeX2e file created by Design/CPN \n\n"^
	(case (!CPN'env.Timedef) of
	     CPN'env.timenotdef => ""
	   | _ => 
		 "\\begin{tabular}{l"^(rightJustified (noOfTimedElms-1))^"}\n"^
		 "\\multicolumn{"^makestring(noOfTimedElms)^"}"^"{c}{TIMED STATISTICS} \\\\ \n"^
		 (addLaTeXSeparators TSVHeader)^"\\hline \n"^
		 timedElementsString^"\\hline\n"^
		 "\\end{tabular} \n\n\\vspace{1cm} \n")^
	 "\\begin{tabular}{l"^(rightJustified (noOfUntimedElms-1))^"}\n"^
	 "\\multicolumn{"^makestring(noOfUntimedElms)^"}"^"{c}{UNTIMED STATISTICS} \\\\ \n"^
	 (addLaTeXSeparators SVHeader)^"\\hline \n"^
	 untimedElementsString^"\\hline\n"^
	 "\\end{tabular} \n"^
	 Perf'Report.currentStep()^"\n"^Perf'Report.currentTime()^"\n"
    end;

fun allRecordsHTML () = 
    let
	val TSVHeader = Perf'Report.TSVReportHeaderElements()
	val SVHeader = Perf'Report.SVReportHeaderElements()

	val noOfTimedElms = length TSVHeader
	val noOfUntimedElms = length SVHeader

	fun addHTMLSeparators (CPN'elm::[]) = "\t<TD> "^CPN'elm^" </TD>\n</TR>\n\n"
	  | addHTMLSeparators (CPN'elm::rest) = "\t<TD> "^CPN'elm^" </TD>\n"^(addHTMLSeparators rest)

	fun addHTMLHeaderSeparators (CPN'elm::[]) = "\t<TH> "^CPN'elm^" </TH>\n</TR>\n\n"
	  | addHTMLHeaderSeparators (CPN'elm::rest) = "\t<TH> "^CPN'elm^" </TH>\n"^(addHTMLHeaderSeparators rest)

	val timedElementsList = 
	    (case (!CPN'env.Timedef) of
		 CPN'env.timeisint _  => Perf'StatsIntTime.ListsOfStats()
	       | _ => [[""]])

	val timedElementsString = 
	    (case (!CPN'env.Timedef) of
		 CPN'env.timenotdef => ""
	       | _ => implode (map addHTMLSeparators timedElementsList))

	val untimedElementsList = 
	    (Perf'Report.getUntimedTableElements (Perf'Env.getUIList Perf'Env.DataCollectors) CPN'IUSV.StatsAsStrings)^^
	    (Perf'Report.getUntimedTableElements (Perf'Env.getURList Perf'Env.DataCollectors) CPN'RUSV.StatsAsStrings)

	val untimedElementsString = implode (map addHTMLSeparators untimedElementsList)

	fun rightJustified 0 = ""
	  | rightJustified n = if n < 0 then "" 
			       else ("r"^(rightJustified (n-1)))

    in
	"<!-- HTML file created by Design/CPN --> \n"^
	"<html>\n<head>\n<TITLE>Design/CPN Performance Report</TITLE></head>\n"^
	"<body>\n"^
	(case (!CPN'env.Timedef) of 
	     CPN'env.timenotdef => ""
	   | _ => 
		 "<P>\n<TABLE>\n<CAPTION> TIMED STATISTICS </CAPTION>\n<TR>\n"^
		 (addHTMLHeaderSeparators TSVHeader)^
		 timedElementsString^
		 "</TABLE></P>\n\n\n")^
	"<P>\n<TABLE>\n<CAPTION> UNTIMED STATISTICS </CAPTION>\n<TR>\n"^
	(addHTMLHeaderSeparators SVHeader)^
	untimedElementsString^
	"</TABLE>\n</P>\n\n"^
	"</body>\n"^
	"</html>\n"

    end;

val perfReportName = ref "";


fun saveReport () = 
    let
	val reportFileName = 
	    DSUI_GetFileName {path=(!perfReportName),
			      prompt="Save performance report as "}
	    
	val report = 
	    (case (!Perf'Report.theReportFormat) of
		 Perf'Report.ASCII_Format => implode (allRecordsASCII ())
	       | Perf'Report.LaTeX_Format => allRecordsLaTeX()
	       | Perf'Report.HTML_Format => allRecordsHTML())
		 
	val outFile = open_out (reportFileName) 
	    handle IO.Io exnStr => Perf'Exception.HandleReportExcn exnStr; 
		
	val out = output(outFile, report) 
    in 
	(close_out outFile; 
	 perfReportName := reportFileName;
	 DSUI_SetStatusBarMessage("Performance report saved.")) 
    end; (* saveReport*)

end; (* struct *)

fun savePerformanceReport (reportFileName) = 
    let	    
	val report = 
	    (case (!Perf'Report.theReportFormat) of
		 Perf'Report.ASCII_Format => implode (Perf'Print.allRecordsASCII ())
	       | Perf'Report.LaTeX_Format => Perf'Print.allRecordsLaTeX()
	       | Perf'Report.HTML_Format => Perf'Print.allRecordsHTML())
		 
	val outFile = open_out (reportFileName) 
	    handle IO.Io exnStr => Perf'Exception.HandleReportExcn exnStr; 
		
	val out = output(outFile, report) 
    in 
	close_out outFile
    end; (* savePerformanceReport *)
