(*
 * Module:       Design/CPN Performance Analysis Library
 *
 * Description:  Creating and saving standard report
 *
 * Created by:   Bo Lindstrm 
 *               Lisa Wells   
 *
 * Date:         March 1998   
 *
 * Design/CPN   Copyright (c) University of Aarhus 1998
 *)

(* 
$Source: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfReport.sml,v $

$Log: PerfReport.sml,v $
Revision 1.5.2.1  1999/05/20 06:43:35  wells
Port to sml97

Revision 1.5  1999/02/03 12:17:12  wells
getTableElement functions added, updateLogFile removed from DCs,
perfreport_format type added

Revision 1.4  1999/01/12 11:37:31  wells
#Updates changed to Count

Revision 1.3  1998/12/09 13:22:23  wells
Exception handling added, report name saved, model timed added to report

Revision 1.2  1998/10/26 08:52:13  wells
structure names standardized, CPN' added to parameter names,
updateLogFile added

Revision 1.1  1998/10/06 11:02:36  wells
Initial Version

*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfReport.sml,v 1.5.2.1 1999/05/20 06:43:35 wells Exp $";

structure Perf'Report = struct

val noDec      = ref 2;  (* Number of decimals in print                     *)
val col        = ref 8; (* Number of digits of numbers in print           *)
val txtCol     = ref 16; (* Number of chars for the desc. column in print *)

datatype perfreport_format = ASCII_Format | LaTeX_Format | HTML_Format;

val theReportFormat = ref ASCII_Format;

(* true = print the attribute, false = do NOT print the attribute *)
(* [0 Count, 1 Sum, 2 SS, 3 SSD, 4 Avg, 5 Variance, 6 Std, 7 Min, 8 Max, 
   9 First Val, 10 Last Val, 11 First Time, 12 Last Time, 13 Interval] *)
val printTAttr =  (* Timed *) 
    ref [true,true,true,true,true,true,true,true,true,true,true,true,true,true]; 
(* [0 Count, 1 Sum, 2 SS, 3 SSD, 4 Avg, 5 Variance,  
   6 Std, 7 Min, 8 Max, 9 First Val, 10 Last Val, ]*)
val printUAttr = (*Untimed*)
ref [true,true,true,true,true,true,true,true,true,true,true];

exception ChooseTimedValues;
fun chooseTimedValues (CPN'bools : bool list) = 
    if (length CPN'bools)=(length (!printTAttr)) then printTAttr := CPN'bools
    else raise ChooseTimedValues;

exception ChooseUntimedValues;
fun chooseUntimedValues (CPN'bools : bool list) = 
    if (length CPN'bools)=(length (!printUAttr)) then printUAttr := CPN'bools 
    else raise ChooseUntimedValues;

fun setLayout (CPN'decPos:int,CPN'numColWidth:int,CPN'firstColWidth:int) =
    (noDec := CPN'decPos;
     col   := CPN'numColWidth;
     txtCol:= CPN'firstColWidth);

(* ------------------------------------------------------ 
 * -----=====>>>>> PRINT REPORT FUNCTIONS <<<<<=====----- 
 * ------------------------------------------------------ 
 *) 

fun zip [] CPN'ys = CPN'ys
  | zip CPN'xs [] = CPN'xs
  | zip (CPN'x::CPN'xs) (CPN'y::CPN'ys) = CPN'x^CPN'y::(zip CPN'xs CPN'ys);

(* -----=====>>>>> <<<<<=====----- *)

fun nChars CPN'n CPN'c = 
    if CPN'n > 0 
	then CPN'c^(nChars (CPN'n-1) CPN'c) 
    else "";

fun strRN (CPN'n:int) (CPN's:string)  =
    if CPN'n <= (size CPN's) then 
	(substring(CPN's, 0, CPN'n-1))^"*" 
    else 
	(nChars (CPN'n-(size CPN's)) " ")^CPN's;

fun strLN (CPN'n:int) (CPN's:string)  =
    if CPN'n <= (size CPN's) then 
	substring(CPN's, 0, CPN'n-1)^"*"
    else 
	CPN's^(nChars (CPN'n-(size CPN's)) " ");


(* -----=====>>>>> <<<<<=====----- *)

(* If the timed statistical variable has been updated (updateSV=true), 
 * then format its name and values and include it in the performance report, 
 * otherwise do not include it *)
fun printListTime CPN'theTime [] CPN'printFunc = []
  | printListTime CPN'theTime 
    ({name=CPN'name,theSV=CPN'sv,obsFun=CPN'ob,predFun=CPN'pr,
      logFile=CPN'lf,logFileName=CPN'lfn,counter=CPN'c,
      updateSV=true}::CPN'theList) CPN'printFunc =
    let
	val formattedName = (strLN (!txtCol)) CPN'name
	val formattedValues = 
	    (CPN'printFunc CPN'theTime (!noDec) (!col) (!printTAttr)) CPN'sv
    in
	(formattedName^formattedValues)::(printListTime CPN'theTime CPN'theList CPN'printFunc)
    end
  | printListTime CPN'theTime (CPN'elm::CPN'theList) CPN'printFunc =
    (printListTime CPN'theTime CPN'theList CPN'printFunc);

fun getTimedTableElements CPN'theTime [] CPN'printFunc = []
  | getTimedTableElements CPN'theTime 
    ({name=CPN'name,theSV=CPN'sv,obsFun=CPN'ob,predFun=CPN'pr,
      logFile=CPN'lf,logFileName=CPN'lfn,counter=CPN'c,
      updateSV=true}::CPN'theList) CPN'printFunc =
    let
	val formattedValues = 
	    (CPN'printFunc CPN'theTime (!noDec) (!col) (!printTAttr)) CPN'sv
    in
	((CPN'name)::formattedValues)::(getTimedTableElements CPN'theTime CPN'theList CPN'printFunc)
    end
  | getTimedTableElements CPN'theTime (_::CPN'theList) CPN'printFunc = 
    (getTimedTableElements CPN'theTime CPN'theList CPN'printFunc);
    
(* If the untimed statistical variable has been updated (updateSV=true), 
 * then format its name and values and include it in the performance report, 
 * otherwise do not include it *)
fun printListUntimed [] CPN'printFunc = []
  | printListUntimed 
    ({name=CPN'name,theSV=CPN'sv,obsFun=CPN'ob,predFun=CPN'pr,
      logFile=CPN'lf,logFileName=CPN'lfn,counter=CPN'c,
      updateSV=true}::CPN'theList) CPN'printFunc =
    let
	val formattedName = (strLN (!txtCol)) CPN'name
	val formattedValues = 
	    (CPN'printFunc (!noDec) (!col) (!printUAttr)) CPN'sv
    in
	(formattedName^formattedValues)::(printListUntimed CPN'theList CPN'printFunc)
    end
  | printListUntimed (CPN'elm::CPN'theList) CPN'printFunc =
    (printListUntimed CPN'theList CPN'printFunc);

fun getUntimedTableElements [] CPN'printFunc = []
  | getUntimedTableElements 
    ({name=CPN'name,theSV=CPN'sv,obsFun=CPN'ob,predFun=CPN'pr,
      logFile=CPN'lf,logFileName=CPN'lfn,counter=CPN'c,
      updateSV=true}::CPN'theList) CPN'printFunc =
    let
	val formattedValues = 
	    (CPN'printFunc (!noDec) (!col) (!printTAttr)) CPN'sv
    in
	((CPN'name)::formattedValues)::(getUntimedTableElements CPN'theList CPN'printFunc)
    end
  | getUntimedTableElements (_::CPN'theList) CPN'printFunc = 
    (getUntimedTableElements CPN'theList CPN'printFunc);
     
    

(* -----==== >>>> printAllList <<<<====---- *)
fun countTrue [] = 0
  | countTrue (false::CPN'bs) = 0 +(countTrue CPN'bs)
  | countTrue (true::CPN'bs) = 1 + (countTrue CPN'bs);

fun makeHeaderLine (CPN'b:bool list) =
    nChars ((countTrue CPN'b)*(!col)+(!txtCol)) "-";

fun TSVReportHeader() = 
    "TIMED STATISTICS: \n\n"^
    (strLN (!txtCol) "Name")^
    (if (List.hd (!printTAttr))      then strRN (!col) "Count"         else "")^
    (if (List.nth((!printTAttr),1))  then strRN (!col) "Sum"           else "")^
    (if (List.nth((!printTAttr),2))  then strRN (!col) "SS"            else "")^
    (if (List.nth((!printTAttr),3))  then strRN (!col) "SSD"           else "")^
    (if (List.nth((!printTAttr),4))  then strRN (!col) "Average"       else "")^
    (if (List.nth((!printTAttr),5))  then strRN (!col) "Variance"      else "")^
    (if (List.nth((!printTAttr),6))  then strRN (!col) "StD"           else "")^
    (if (List.nth((!printTAttr),7))  then strRN (!col) "Minimum"       else "")^
    (if (List.nth((!printTAttr),8))  then strRN (!col) "Maximum"       else "")^   
    (if (List.nth((!printTAttr),9))  then strRN (!col) "First"         else "")^
    (if (List.nth((!printTAttr),10)) then strRN (!col) "Last"          else "")^
    (if (List.nth((!printTAttr),11)) then strRN (!col) "First Time"    else "")^
    (if (List.nth((!printTAttr),12)) then strRN (!col) "Last Time"     else "")^
    (if (List.nth((!printTAttr),13)) then strRN (!col) "Time Interval" else "")^
    "\n"^(makeHeaderLine (!printTAttr))^"\n"

fun TSVReportHeaderElements() = 
    ["Name"]^^
    (if (List.hd (!printTAttr))      then ["Count"]         else [])^^
    (if (List.nth((!printTAttr),1))  then ["Sum"]           else [])^^
    (if (List.nth((!printTAttr),2))  then ["SS"]            else [])^^
    (if (List.nth((!printTAttr),3))  then ["SSD"]           else [])^^
    (if (List.nth((!printTAttr),4))  then ["Average"]       else [])^^
    (if (List.nth((!printTAttr),5))  then ["Variance"]      else [])^^
    (if (List.nth((!printTAttr),6))  then ["StD"]           else [])^^
    (if (List.nth((!printTAttr),7))  then ["Minimum"]       else [])^^
    (if (List.nth((!printTAttr),8))  then ["Maximum"]       else [])^^   
    (if (List.nth((!printTAttr),9))  then ["First"]         else [])^^
    (if (List.nth((!printTAttr),10)) then ["Last"]          else [])^^
    (if (List.nth((!printTAttr),11)) then ["First Time"]    else [])^^
    (if (List.nth((!printTAttr),12)) then ["Last Time"]     else [])^^
    (if (List.nth((!printTAttr),13)) then ["Time Interval"] else []);

fun SVReportHeader () =     
    "\n\nUNTIMED STATISTICS: \n\n"^
    (strLN (!txtCol) "Name")^
    (if (List.hd (!printUAttr))      then strRN (!col) "Count"    else "")^
    (if (List.nth((!printUAttr),1))  then strRN (!col) "Sum"      else "")^
    (if (List.nth((!printUAttr),2))  then strRN (!col) "SS"       else "")^
    (if (List.nth((!printUAttr),3))  then strRN (!col) "SSD"      else "")^
    (if (List.nth((!printUAttr),4))  then strRN (!col) "Average"  else "")^
    (if (List.nth((!printUAttr),5))  then strRN (!col) "Variance" else "")^
    (if (List.nth((!printUAttr),6))  then strRN (!col) "StD"      else "")^
    (if (List.nth((!printUAttr),7))  then strRN (!col) "Minimum"  else "")^
    (if (List.nth((!printUAttr),8))  then strRN (!col) "Maximum"  else "")^
    (if (List.nth((!printUAttr),9))  then strRN (!col) "First"    else "")^
    (if (List.nth((!printUAttr),10))  then strRN (!col) "Last"    else "")^"\n"^
    (makeHeaderLine (!printUAttr))^"\n";

fun SVReportHeaderElements() = 
    ["Name"]^^
    (if (List.hd (!printTAttr))      then ["Count"]         else [])^^
    (if (List.nth((!printTAttr),1))  then ["Sum"]           else [])^^
    (if (List.nth((!printTAttr),2))  then ["SS"]            else [])^^
    (if (List.nth((!printTAttr),3))  then ["SSD"]           else [])^^
    (if (List.nth((!printTAttr),4))  then ["Average"]       else [])^^
    (if (List.nth((!printTAttr),5))  then ["Variance"]      else [])^^
    (if (List.nth((!printTAttr),6))  then ["StD"]           else [])^^
    (if (List.nth((!printTAttr),7))  then ["Minimum"]       else [])^^
    (if (List.nth((!printTAttr),8))  then ["Maximum"]       else [])^^   
    (if (List.nth((!printTAttr),9))  then ["First"]         else [])^^
    (if (List.nth((!printTAttr),10)) then ["Last"]          else []);

fun currentStep() = "\n\nCurrent step: "^makestring(step())^"\n"

fun currentTime() = if with_time() 
			then "Current time: "^IntInf.toString(time())^"\n" 
		    else "\n";

end; (* struct *)

