(*
 * Module:       Design/CPN Performance Analysis Library
 *
 * Description:  Setting start, step and stop functions which are
 *               called in the simulator to collect data
 *
 * Created by:   Bo Lindstrm 
 *               Lisa Wells   
 *
 * Date:         March 1998   
 *
 * Design/CPN   Copyright (c) University of Aarhus 1998
 *)

(* 
 $Source: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfSetSimInterfaceFuncs.sml,v $

 $Log: PerfSetSimInterfaceFuncs.sml,v $
 Revision 1.4.2.1  1999/05/20 06:43:36  wells
 Port to sml97

 Revision 1.4  1999/02/03 12:18:36  wells
 marking calculated only when Data Collectors are declared

 Revision 1.3  1998/12/09 13:25:45  wells
 advancedStepFun, updateStatusBar, and setUpdateStatusBar added

 Revision 1.2  1998/10/26 08:44:38  wells
 structure names standardized, CPN' added to parameter names

 Revision 1.1  1998/10/06 11:02:38  wells
 Initial Version

*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfSetSimInterfaceFuncs.sml,v 1.4.2.1 1999/05/20 06:43:36 wells Exp $";

structure Perf'SimInterfaceFuncs = 
struct

fun startFunc () =  (if (step()=0)
			 then Perf'LogFileManagement.openAll()
		     else Perf'LogFileManagement.openAppendAll();
		     ());

val updateStatusBar = ref (fn () => (step() mod 10) = 0);

fun setStatusBarUpdate (CPN'updateFun) = (updateStatusBar:= CPN'updateFun);

fun stepFunc (CPN'name,CPN'trmlno) = 
    (if (step() mod 100) = 0 
	 then DSUI_SetStatusBarMessage("Current step: "^(makestring(step()))^
				       (if with_time() 
					   then ("  Current time: "^(IntInf.toString(time())))
				       else ""))
     else ();
     if (!Perf'Env.DataCollDeclared)
	 then Perf'Update.UpdateAll(GetCurrentSimState (), 
			   (Perf'SimStepToBind (CPN'trmlno,Perf'FindInstance CPN'name)))
     else [()];
     ());

fun simpleStepFunc (CPN'name,CPN'trmlno) = 
    (if (!Perf'Env.DataCollDeclared)
	 then Perf'Update.UpdateAll(GetCurrentSimState (), 
				    (Perf'SimStepToBind (CPN'trmlno,Perf'FindInstance CPN'name)))
     else [()];
     ());

fun advancedStepFunc (CPN'name,CPN'trmlno) = 
    (if (!updateStatusBar ()) 
	 then DSUI_SetStatusBarMessage("Current step: "^(makestring(step()))^
				       (if with_time() 
					   then ("  Current time: "^(IntInf.toString(time())))
				       else ""))
     else ();
     if (!Perf'Env.DataCollDeclared)
	 then Perf'Update.UpdateAll(GetCurrentSimState (), 
				    (Perf'SimStepToBind (CPN'trmlno,Perf'FindInstance CPN'name)))
     else [()];
     if ((!Perf'LogFileManagement.flushFilesNow) ())
	 then Perf'LogFileManagement.flushAll()
     else [()];
     ());

fun stopFunc () = 
   (Perf'LogFileManagement.closeAll();
    ());

end; (* struct *)

val theTrmlno = ref 0;
val theName = ref "";


Perf'SimInterface.Perf'SetStartFunction (Perf'SimInterfaceFuncs.startFunc);
(*
Perf'SimInterface.Perf'SetStepFunction (Perf'SimInterfaceFuncs.stepFunc);
*)
Perf'SimInterface.Perf'SetStepFunction (Perf'SimInterfaceFuncs.advancedStepFunc);

Perf'SimInterface.Perf'SetStopFunction (Perf'SimInterfaceFuncs.stopFunc);

