(*
 * Module:       Design/CPN Performance Analysis Library
 *
 * Description:  Implements callback interface to automatic simulations  
 *
 * Created by:   Lars Michael Kristensen (kris@daimi.aau.dk)   
 *
 * Date:         27/03/98   
 *
 * Design/CPN   Copyright (c) University of Aarhus 1998
 *)

(* 
 $Source: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfSimInterface.sml,v $ 

 $Log: PerfSimInterface.sml,v $
 Revision 1.2  1999/02/03 12:18:37  wells
 marking calculated only when Data Collectors are declared

 Revision 1.1  1998/10/06 11:02:39  wells
 Initial Version
 
*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfSimInterface.sml,v 1.2 1999/02/03 12:18:37 wells Exp $";

structure Perf'SimInterface :
    sig
	type Perf'SimState
	val Perf'SetStepFunction  : (string * int -> unit) -> unit
	val Perf'SetStartFunction : (unit -> unit) -> unit
	val Perf'SetStopFunction   : (unit -> unit) -> unit
	val Perf'InitSimInterface : unit -> unit
end

= struct
      type Perf'SimState = CPN'OGrec

      (* flag used to signal start of simulation *)
      val Perf'first = ref true;

      (* set function invoked when automatic run is started *) 
      fun Perf'SetStartFunction Perf'startfun = 
	  CPN'InternalSimCallBack.CPN'SetInternalStartCallBack 
	  (fn () => if (!Perf'first)
			then 
			    (Perf'startfun ();
			     Perf'first := false)
		    else 
			());

      (* set function invoked during step in automatic run *)
      fun Perf'SetStepFunction Perf'stepfun = 
	  CPN'InternalSimCallBack.CPN'SetInternalStepCallBack 
	  (fn (CPN'name,CPN'trmlno) => 
	   Perf'stepfun (CPN'name,CPN'trmlno));
			 
	  
      (* set function invoked when automatic run stops *)
      fun Perf'SetStopFunction Perf'stopfun = 
	  CPN'InternalSimCallBack.CPN'SetInternalStopCallBack 
	  (fn () => (Perf'first := true;
		     Perf'stopfun ()));

      (* initialize/clear the installed functions -- might speed up simulation *)
      fun Perf'InitSimInterface () = 
	  (Perf'first := true;
	   CPN'InternalSimCallBack.CPN'SetInternalStartCallBack (fn () => ());
	   CPN'InternalSimCallBack.CPN'SetInternalStepCallBack (fn (CPN'n:string,CPN't:int) => ());
	   CPN'InternalSimCallBack.CPN'SetInternalStopCallBack (fn () => ()));

end;


(* TODO

   - Implement CurrentStep should be updated in the generated 
     sac functions similar to write_report 

   - Step callback with trmlno,instance,"binding-string". MLtoTIs functions needs to be generated

   - Implement Start call back. Here we probably need a flag which can be cleared by the stop-function.


   - For the stop functions which should handle two cases (top criteria and dead markings). 
     We will not be able to handle exception case

   - Probably it should be possible to invoked start/step and stop functions manually also. 
     To prepare for the later support in interactive and manual mode.

   Last Update: Tue Mar 24 15:35:55 MET 1998
*)



(* example on use 

val mylist = ref [] : string list ref; 

fun MyStartFun () = (mylist := (" start")::(!mylist));

fun MyStepFun (CPN'a,CPN'b) = (DSUI_UserAckMessage ("STEPCALLBACK");
			       mylist := "step"::(!mylist));

fun MyStopFun () = (mylist := (" end")::(!mylist));

val xx = Perf'SimInterface.Perf'SetStartFunction (MyStartFun);

val xx = Perf'SimInterface.Perf'SetStepFunction (MyStepFun);

val xx = Perf'SimInterface.Perf'SetStopFunction (MyStopFun);

*)
