(*
 * Module:       Design/CPN Performance Analysis Library
 *
 * Description:  Implements function to pull a step from the simulator
 *
 * Created by:   Lars Michael Kristensen (kris@daimi.aau.dk)   
 *
 * Date:         27/03/98   
 *
 * Design/CPN   Copyright (c) University of Aarhus 1998
 *)

(* 
$Source: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfSimStepToBind.sml,v $ 
*)

(* 
$Log: PerfSimStepToBind.sml,v $
Revision 1.1.2.1  1999/05/20 06:43:36  wells
Port to sml97
 
*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfSimStepToBind.sml,v 1.1.2.1 1999/05/20 06:43:36 wells Exp $";

exception Perf'FindCharPosExn;
fun Perf'FindCharPos _ "" _ = raise Perf'FindCharPosExn
  | Perf'FindCharPos Perf'char Perf'string Perf'pos = 
    let
	val Perf'nextchar = substring (Perf'string,0,1);
    in
	if (Perf'nextchar = Perf'char)
	    then Perf'pos
	else
	    Perf'FindCharPos 
	    Perf'char (substring (Perf'string,1,((size (Perf'string))-1))) (Perf'pos + 1)
    end;

fun Perf'StringToInt "" Perf'res = Perf'res
  | Perf'StringToInt Perf'string Perf'res = 
    let
	val Perf'val = (ord (substring (Perf'string,0,1))) - (ord "0")
    in
	 Perf'StringToInt (substring (Perf'string,1,((size Perf'string) - 1))) 
	 ((10 * Perf'res) + Perf'val)
    end;

(* Extract instance number from string on the form:

   <transition-name>@(<instance>:<page-name>#<page-number>) 

*)

exception Perf'FindInstanceExn;
fun Perf'FindInstance Perf'name = 
    let
	val perf'p = Perf'FindCharPos "(" Perf'name 0;
	val perf'col = Perf'FindCharPos ":" Perf'name 0
    in
	if (perf'p < perf'col)
	    then
		let
		    val perf'int = Int.fromString (substring (Perf'name,perf'p + 1,perf'col - perf'p - 1))
		in
		    case perf'int of
			SOME i => i
		      | _ => raise Perf'FindInstanceExn
		end
	else
	    raise Perf'FindInstanceExn
    end;

structure Perf'SimStepToBind =
    struct
	
	local
	    
	    open CPN'CodeGenUtils;
		
	    val FirstConstructor = ref true;
		
	    fun lgen_SimStepToBind()
		=( patterncount:=0;
		  partfuncount:=0;
		  code_temp:="";
		  code_storage:= ["exception Perf'SimStepToBindExn; fun Perf'SimStepToBind0 _ = Bind.unnamedBE;\n"];
		  
		  map
		  (fn {trlist = trlist, pgname = pgname,...}
		   => map
		   (fn {trname = "", ...} => ()
		     | {trname = trname, trmlno = trmlno, ...}
		    => let
			   val internalid = "("^trmlno^",CPN'instno)";
			   val (intpatt,extpatt)
			       = CPN'OGBTconvGen.genBEbindpatterns trmlno 
		       in
			   newpattern("Perf'SimStepToBind",
				      "("^trmlno^",Perf'instno) = \n"^
				      "Perf_ArcToBE (("^trmlno^",Perf'instno),ref (CPN'BT"^(CPN'OGBTconvGen.findmlnoforbinding trmlno)^"conv (case (!CPN'SAC"^trmlno^"RecordStep) of \n [] => raise Perf'SimStepToBindExn | (CPN'x::CPN'xs) => CPN'x)))")
		       end(*let*))
		   trlist)
		  (!CPN'NetCapture.Net);
		  
		  if !patterncount>0
		      then (* finish the last function in code_temp *)
			  (code_storage:= !code_storage^^
			   [!code_temp^
			    "|Perf'SimStepToBind"^(makestring(!partfuncount+1))^" CPN'x=Perf'SimStepToBind"^
			    (makestring(!partfuncount))^" CPN'x;\n"];
			   code_temp:="")
		  else
		      ();
		      
		      code_storage:= !code_storage^^
		      ["fun Perf'SimStepToBind CPN'a=Perf'SimStepToBind"^
		       (makestring(!partfuncount+(if !patterncount=0 then 0 else 1)))^
		       "(CPN'a);\n"]);

	in

	    fun gen_SimStepToBind () = 
		(lgen_SimStepToBind();
		 !code_storage);

	end
    end;
