(*
 * Module:       Design/CPN Performance Analysis Library
 *
 * Description:  Extracting statistics from timed statistical variables
 *               for INTEGER time (used for creating reports)
 *
 * Created by:   Bo Lindstrm 
 *               Lisa Wells   
 *
 * Date:         January 1999
 *
 * Design/CPN   Copyright (c) University of Aarhus 1999
 *)

(* 
$Source: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfStatsIntTime.sml,v $

$Log: PerfStatsIntTime.sml,v $
Revision 1.1.2.1  1999/05/20 06:43:37  wells
Port to sml97

Revision 1.1  1999/02/03 12:24:04  wells
Initial Version


*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfStatsIntTime.sml,v 1.1.2.1 1999/05/20 06:43:37 wells Exp $";
    
structure Perf'StatsIntTime = struct
	

(* ASCII_String returns a list of strings. Each string contains the name
 * of a data collector and some or all statistics from the corresponding
 * timed statistical variable. Each statistic has been formated, in other
 * words the number will fit into the column width that has been chosen.
*)

fun ASCII_String () = (Perf'Report.printListTime (time()) (Perf'Env.getTIList Perf'Env.DataCollectors) CPN'ITSV.Print)^^
	(Perf'Report.printListTime (time()) (Perf'Env.getTRList Perf'Env.DataCollectors) CPN'RTSV.Print)
	

(* ListsOfStats() returns a list of lists of strings. Each list
 * corresponds to one row in a performance report. The elements of
 * each list are string representations of the name of a data
 * collector and some or all statistics from the corresponding timed
 * statistical variable as separate.  *)

fun ListsOfStats () = (Perf'Report.getTimedTableElements (time()) (Perf'Env.getTIList Perf'Env.DataCollectors) CPN'ITSV.StatsAsStrings)^^
	    (Perf'Report.getTimedTableElements (time()) (Perf'Env.getTRList Perf'Env.DataCollectors) CPN'RTSV.StatsAsStrings)



end (* struct *)

