(*
 * Module:       Design/CPN Performance Analysis Library
 *
 * Description:  Implements the switch to the library 
 *
 * Created by:   Lars Michael Kristensen (kris@daimi.aau.dk)   
 *
 * Date:         24/03/98   
 *
 * Design/CPN   Copyright (c) University of Aarhus 1998
 *)

(* 
$Source: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfSwitch.sml,v $ 

$Log: PerfSwitch.sml,v $
Revision 1.10.2.2  1999/05/21 13:21:58  sts
Fixed some unbalanced comments.

Revision 1.10.2.1  1999/05/20 06:43:38  wells
Port to sml97

*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfSwitch.sml,v 1.10.2.2 1999/05/21 13:21:58 sts Exp $";

(******* Disable check to speed up and save stack space ******)
CPN'CompDep.CheckOff();

DSUI_SetStatusBarMessage("Loading Performance Library...");

(******* net capture ******)

DSUI_SetStatusBarMessage("[1:4] Reading Net Information...");

CPN'NetCapture.exe trans_seq;
CPN'NetCapture.check_names();         

(*** net dependent 1 ***)

usestring(CPN'OGIdsGen.gen_TI());   
usestring(CPN'OGIdsGen.gen_PI());   
usestring(CPN'OGBTconvGen.gen_binding());
usestring(CPN'OGBTconvGen.gen_conv());
usestring(CPN'OGBTconvGen.gen_encode());
usestring(CPN'OGBTconvGen.gen_st_binding());      
usestring(CPN'OGBasicRefsGen.gen());     

DSUI_SetStatusBarMessage("[2:4] Defining Basic Performance Functions...");

use (perfpath^"PerfMarkGen.sml");
usestring(Perf'MarkGen.gen());

use (perfpath^"PerfFromSimDataGen.sml");
usestring(Perf'FromSimDataGen.gen());

use (perfpath^"PerfBind.sml");
usestring(Perf'BindGen.gen_BindStruct());

usestring(Perf'BindGen.gen_ArcToBE());
use (ogpath^"OGBindMap.sml");
usestring(CPN'OGBindGenNew.gen_BEToArc ());

usestring(Perf'BindGen.gen_BEToTI());

DSUI_SetStatusBarMessage("[3:4] Defining Printing Functions...");
use (perfpath^"PerfSetAttrOptions.sml");
usestring(CPN'OGIdsGen.gen_stTI());   
usestring(CPN'OGIdsGen.gen_stPI());   

usestring(Perf'BindGen.gen_st_BE());

DSUI_SetStatusBarMessage("[4:4] Defining Data Collection Functions...");

if (!CPN'env.Timedef = CPN'env.timenotdef)
    then usestring["fun time () = IntInf.fromInt(0);"]
else ();

use (perfpath^"GetSimulatorState.sml");
usestring (CPN'GetSimulatorState.gen ());

use (perfpath^"PerfEnv.sml");
use (perfpath^"PerfGenAccessDC.sml");
use (perfpath^"PerfException.sml");
(case !CPN'env.Timedef of
     (CPN'env.timeisint _)  =>     (* INT *)
	 use (perfpath^"PerfAccessDCTimed.sml")	
   |  _  =>                        (* UNTIMED *)
	 use (perfpath^"PerfAccessDCUntimed.sml"));
use (perfpath^"PerfCreateRecords.sml");
use (perfpath^"PerfCreateTemplate.sml");
use (perfpath^"PerfDeleteRecords.sml");
use (perfpath^"PerfInitRecords.sml");
use (perfpath^"PerfReport.sml");
use (perfpath^"PerfStatsIntTime.sml");
use (perfpath^"PerfPrint.sml");
use (perfpath^"PerfUpdateIntTime.sml");
use (perfpath^"PerfLogFileManagement.sml");

use (perfpath^"PerfSimStepToBind.sml");
usestring (Perf'SimStepToBind.gen_SimStepToBind ());
use (perfpath^"PerfSimInterface.sml");

(* definition of simulation interface functions *)
use (perfpath^"PerfSetSimInterfaceFuncs.sml");

CPN'CompDep.CheckOn();

DSStr_SetCurObject(Perf'Page.getPerfBox(Perf'Page.PerfPageName));
