(*
 * Module:          Design/CPN Performance Analysis Library
 * 
 * Description:     Functions used to update data collectors
 *                  (for untimed simulations and integer timed
 *                  simulations)
 * 
 * Created by:      Bo Lindstrom 
 *                  Lisa Wells 
 *                   
 * Date:            February 1999
 *)

(* 
$Source: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfUpdateIntTime.sml,v $

$Log $

*)

val rcsid = "$Header: /users/cpn/designcpnR/src/current/perf/PerfMLfiles/PerfUpdateIntTime.sml,v 1.1.2.1 1999/05/20 06:43:39 wells Exp $";

structure Perf'Update = 
struct

fun updateUntimedIntDC (CPN'ogrec:CPN'OGrec) (CPN'BE: Bind.Elem) (CPN'p:Perf'Env.DCuntimedInt)  = 
    let 
	val CPN'pred = (!(#predFun CPN'p)) (CPN'ogrec,CPN'BE)
    in  
	if CPN'pred then  
	    ((#counter CPN'p) := !(#counter CPN'p) + 1;  
	     let  
		 val v = (#theSV CPN'p);  
		 val newVal = (!(#obsFun CPN'p)) (CPN'ogrec,CPN'BE)   
		 val updateFile = (#logFileName CPN'p)  
		 val updateTimeStr = makestring(!(#counter CPN'p))^" "  
		 val outputStr =  updateTimeStr^makestring(newVal)^"\n"
		     
	     in  
		 if (updateFile <> NONE)
		     then output(!(#logFile CPN'p),outputStr) handle IO.Io exnStr => Perf'Exception.HandleFileExcn (exnStr)  
		 else ();  
		     if (#updateSV CPN'p) then (CPN'IUSV.Upd(v,newVal) handle Perf'theExcn => Perf'Exception.HandleUpdatingExcn(Perf'theExcn,(#name CPN'p)))  
		     else ()  
	     end)  
	else ()  
    end handle theExcn => Perf'Exception.HandleExcn(theExcn, (#name CPN'p),CPN'BE);  

	
fun updateUntimedRealDC (CPN'ogrec:CPN'OGrec) (CPN'BE: Bind.Elem) (CPN'p:Perf'Env.DCuntimedReal) =  
    let  
	val pred = (!(#predFun CPN'p)) (CPN'ogrec,CPN'BE)  
    in  
	if pred then  
	    ((#counter CPN'p) := !(#counter CPN'p) + 1;  
	     let  
		 val v = (#theSV CPN'p);  
		 val newVal = (!(#obsFun CPN'p)) (CPN'ogrec,CPN'BE)  
		 val updateFile = (#logFileName CPN'p)  
		 val updateTimeStr = makestring(!(#counter CPN'p))^" "  
		 val outputStr =  updateTimeStr^Real.toString(newVal)^"\n"
		     
	     in  
		 if (updateFile <> NONE)
		     then output(!(#logFile CPN'p),outputStr)  handle IO.Io exnStr => Perf'Exception.HandleFileExcn(exnStr)  
		 else ();  
		     if (#updateSV CPN'p) then (CPN'RUSV.Upd(v,newVal) handle Perf'theExcn => Perf'Exception.HandleUpdatingExcn(Perf'theExcn,(#name CPN'p)))  
		     else ()  
	     end)  
	else ()  
    end  handle theExcn => Perf'Exception.HandleExcn(theExcn, (#name CPN'p),CPN'BE);  

   
fun updateTimedIntDC (CPN'ogrec:CPN'OGrec) (CPN'BE: Bind.Elem) (CPN'p:Perf'Env.DCtimedInt) =  
    let  
	val pred = (!(#predFun CPN'p)) (CPN'ogrec,CPN'BE)  
    in  
	if pred then  
	    ((#counter CPN'p) := !(#counter CPN'p) + 1;  
	     let  
		 val v = (#theSV CPN'p);  
		 val newVal = (!(#obsFun CPN'p)) (CPN'ogrec,CPN'BE)  
		 val updateFile = (#logFileName CPN'p)  
		 val updateTimeStr = IntInf.toString(time())^" "   
		 val outputStr =  updateTimeStr^makestring(newVal)^"\n" 
	     in   
		 if (updateFile <> NONE)
		     then output(!(#logFile CPN'p),outputStr) handle IO.Io exnStr => Perf'Exception.HandleFileExcn (exnStr)  
		 else ();  
		     if (#updateSV CPN'p) then CPN'ITSV.Upd(v,newVal,time()) handle Perf'theExcn => Perf'Exception.HandleUpdatingExcn(Perf'theExcn,(#name CPN'p))  
		     else ()  
	     end)  
	else ()  
    end handle theExcn => Perf'Exception.HandleExcn(theExcn, (#name CPN'p),CPN'BE);


fun updateTimedRealDC (CPN'ogrec:CPN'OGrec) (CPN'BE: Bind.Elem) (CPN'p:Perf'Env.DCtimedReal) =   
    let  
	val pred = (!(#predFun CPN'p)) (CPN'ogrec,CPN'BE)  
    in  
	if pred then  
	    ((#counter CPN'p) := !(#counter CPN'p) + 1;  
	     let  
		 val v = (#theSV CPN'p);  
		 val newVal = (!(#obsFun CPN'p)) (CPN'ogrec,CPN'BE)  
		 val updateFile = (#logFileName CPN'p)  
		 val updateTimeStr = IntInf.toString(time())^" "   
		 val outputStr =  updateTimeStr^Real.toString(newVal)^"\n"  
	     in   
		 if (updateFile <> NONE)
		     then output(!(#logFile CPN'p),outputStr) handle IO.Io exnStr => Perf'Exception.HandleFileExcn (exnStr)  
		 else ();  
		     if (#updateSV CPN'p) then CPN'RTSV.Upd(v,newVal,time()) handle Perf'theExcn => Perf'Exception.HandleUpdatingExcn(Perf'theExcn,(#name CPN'p))  
		     else ()  
	     end)  
	else ()  
    end handle theExcn => Perf'Exception.HandleExcn(theExcn, (#name CPN'p),CPN'BE);
	

fun UpdateAll(CPN'ogrec:CPN'OGrec, CPN'BE: Bind.Elem) = 
    (map (updateUntimedIntDC CPN'ogrec CPN'BE) (Perf'Env.getUIList Perf'Env.DataCollectors); 
     map (updateUntimedRealDC CPN'ogrec CPN'BE) (Perf'Env.getURList Perf'Env.DataCollectors); 
     map (updateTimedIntDC CPN'ogrec CPN'BE) (Perf'Env.getTIList Perf'Env.DataCollectors);
     map (updateTimedRealDC CPN'ogrec CPN'BE) (Perf'Env.getTRList Perf'Env.DataCollectors))

end (*struct*);

